@@ -5,7 +5,7 @@
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --
---  Copyright (C) 1998-2014 OpenLink Software
+--  Copyright (C) 1998-2016 OpenLink Software
 --
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -757,7 +757,7 @@ create function "CalDAV_DAV_RES_CONTENT" (
 
   type := 'text/calendar';
   whenever not found goto endline;
-  tz := timezone(now());
+  tz := timezone(curdatetime_tz());
   if (id[4] is not null)
     content := CAL.WA.export_vcal (id[3], vector (id[4]));
 endline:
