@@ -4,7 +4,7 @@
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --
---  Copyright (C) 1998-2014 OpenLink Software
+--  Copyright (C) 1998-2016 OpenLink Software
 --
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -362,7 +362,7 @@ FCT_LABEL_S (in x any, in g_id iri_id_8, in ctx varchar, in lng varchar)
   if (__proc_exists ('rdf_resolve_labels_s') is not null)
     {
       declare ret any;
-      ret := rdf_resolve_labels_s (adler32 (lng), vector (x));
+      ret := rdf_resolve_labels_s (adler32 (coalesce (lng, '')), vector (x));
       ret := coalesce (ret[0], '');
       return __ro2sq (ret);
     }
@@ -551,6 +551,20 @@ fct_graph_clause (in tree any)
 }
 ;
 
+create procedure
+fct_named_graph_clause (in tree any)
+{
+  declare xt, xp, inx, s any;
+  inx := 1;
+  s := string_output ();
+  while ((xp := xpath_eval (sprintf ('//query/@graph%d', inx), tree)) is not null)
+    {
+      inx := inx + 1;
+      http (sprintf (' define input:default-graph-uri <%s> ', cast (xp as varchar)), s);
+    }
+  return string_output_string (s);
+}
+;
 create procedure
 fct_post (in tree any, in post any, in lim int, in offs int)
 {
@@ -785,7 +799,7 @@ fct_view (in tree any, in this_s int, in txt any, in pre any, in post any, in fu
   offs := xpath_eval ('./@offset', tree, 1);
   lim  := xpath_eval ('./@limit', tree, 1);
 
-  http (sprintf (' %s %s %s ', fct_graph_clause (tree), fct_inf_clause (tree), fct_sas_clause (tree)), pre);
+  http (sprintf (' %s %s %s %s ', fct_graph_clause (tree), fct_named_graph_clause (tree), fct_inf_clause (tree), fct_sas_clause (tree)), pre);
 
   mode := fct_get_mode (tree, './@type');
 
@@ -803,20 +817,24 @@ fct_view (in tree any, in this_s int, in txt any, in pre any, in post any, in fu
 
   if ('list' = mode or 'propval-list' = mode)
     {
-      http (sprintf ('select distinct ?s%d as ?c1 ', this_s), pre);
-      http (sprintf (' order by desc (<LONG::IRI_RANK> (?s%d)) ', this_s), post);
+      http (sprintf ('select ?s%d as ?c1 ', this_s), pre);
+      http (sprintf (' group by (?s%d) order by desc (<LONG::IRI_RANK> (?s%d)) ', this_s, this_s), post);
     }
 
   if ('list-count' = mode)
     {
-      http (sprintf ('select ?s%d as ?c1 count (*) as ?c2 ', this_s), pre);
-      http (sprintf (' group by ?s%d order by desc 2', this_s), post);
+      declare i int;
+      http (sprintf ('select ?s%d as ?c1 count (*) as ?c2 where { select distinct ', this_s), pre);
+      for (i := 1; i <= this_s; i := i + 1)
+        http (sprintf ('?s%d ', i), pre);
+      if (xpath_eval ('ancestor::query/text', tree)) http ('?g ', pre);
+      http (sprintf (' } group by ?s%d order by desc 2', this_s), post);
     }
 
   if ('entities-list' = mode)
     {
-      http (sprintf ('select distinct ?s%d as ?c1 ', this_s), pre);
-      http (sprintf (' order by desc (<LONG::IRI_RANK> (?s%d)) ', this_s), post);
+      http (sprintf ('select ?s%d as ?c1 ', this_s), pre);
+      http (sprintf (' group by (?s%d) order by desc (<LONG::IRI_RANK> (?s%d)) ', this_s, this_s), post);
     }
 
   if ('properties' = mode)
@@ -1257,6 +1275,7 @@ fct_text (in tree any,
     {
       declare ciri varchar;
       ciri := fct_curie (cast (xpath_eval ('./@iri', tree) as varchar));
+      ciri := replace (ciri, ' ', '%20');
 
       fct_dbg_msg (sprintf ('class: %s', cast (ciri as varchar)));
 
@@ -1311,7 +1330,9 @@ fct_text (in tree any,
       txs_qr := fti_make_search_string_inner (charset_recode (xpath_eval ('string (.)', tree), '_WIDE_', 'UTF-8'), txs_arr);
       if (length (txs_arr) > wlimit)
 	signal ('22023', 'The request is too large');
+      http (' quad map virtrdf:DefaultQuadMap { graph ?g { ', txt);
       http (sprintf (' ?s%d %s ?o%d . ?o%d bif:contains  ''%s'' %s .', this_s, prop, this_s, this_s, txs_qr, sc_opt), txt);
+      http (' } } ', txt);
     }
 
   if ('property' = n)
@@ -1324,6 +1345,7 @@ fct_text (in tree any,
       new_s := max_s;
 
       piri := fct_curie (cast (xpath_eval ('./@iri', tree, 1) as varchar));
+      piri := replace (piri, ' ', '%20');
 
 --      fct_dbg_msg (sprintf ('property: <%s>', piri));
 
@@ -1345,9 +1367,12 @@ fct_text (in tree any,
   if ('property-of' = n)
     {
       declare new_s int;
+      declare piri any;
       max_s := max_s + 1;
       new_s := max_s;
-      http (sprintf (' ?s%d <%s> ?s%d .', new_s, fct_curie (cast (xpath_eval ('./@iri', tree, 1) as varchar)), this_s), txt);
+      piri := fct_curie (cast (xpath_eval ('./@iri', tree, 1) as varchar));
+      piri := replace (piri, ' ', '%20');
+      http (sprintf (' ?s%d <%s> ?s%d .', new_s, piri, this_s), txt);
       fct_text_1 (tree, new_s, max_s, txt, pre, post, full_tree, plain);
     }
 
@@ -1413,11 +1438,11 @@ fct_query (in tree any, in plain integer := 0)
 
   fct_text (xpath_eval ('//query', tree), 0, s, txt, pre, post, tree, plain);
 
-  http (' where {', pre);
-  if (add_graph) http (' quad map virtrdf:DefaultQuadMap { graph ?g { ', pre);
+  if (xpath_eval ('//view[@type="list-count"]', tree) is null)
+    http (' where ', pre);
+  http (' {', pre);
   http (txt, pre);
   http (' }', pre);
-  if (add_graph) http (' } } ', pre);
   http (post, pre);
 
   return string_output_string (pre);
@@ -1559,6 +1584,8 @@ fct_exec (in tree any,
 
   declare v_pos integer;
   v_pos := fct_view_pos(tree);
+  if (sqls <> 'S1TAT' and n_rows = 0)
+    connection_set ('noresult', 1);
 
   res := xmlelement ("facets", xmlelement ("sparql", query),
                                xmlelement ("time", msec_time () - start_time),
