@@ -4,7 +4,7 @@
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --
---  Copyright (C) 1998-2014 OpenLink Software
+--  Copyright (C) 1998-2016 OpenLink Software
 --
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -25,6 +25,11 @@ cl_exec ('registry_set (''fct_max_timeout'',''10000'')');
 DB.DBA.VHOST_REMOVE (lpath=>'/fct/service');
 DB.DBA.VHOST_DEFINE (lpath=>'/fct/service', ppath=>'/SOAP/Http/fct_svc', soap_user=>'SPARQL');
 
+-- http://{cname}/fct/search(?q,view:type,c-term,s-term,same-as,inference,offet,limit,graph)
+DB.DBA.VHOST_REMOVE (lpath=>'/fct/search');
+DB.DBA.VHOST_DEFINE (lpath=>'/fct/search', ppath=>'/SOAP/Http/fct_search', soap_user=>'SPARQL');
+
+
 create procedure fct_init ()
 {
   if (__proc_exists ('WS.WS.host_meta_add') is not null)
@@ -110,6 +115,73 @@ fct_svc_exec (in tree any, in timeout int, in accept varchar, in lines any)
 }
 ;
 
+
+create procedure fct_http_param (in n any, in def any := '')
+{
+  declare v any;
+  v := http_param (n);
+  if (v = 0)
+    v := def;
+  return v;
+}
+;
+
+-- http://{cname}/fct/search(?q,view:type,c-term,s-term,same-as,inference,offet,limit,graph)
+create procedure fct_search () __soap_http 'text/html'
+{
+  declare cnt, tp, ret, timeout, xt, xslt, maxt, tmp, lines, accept any;
+  declare inf, sas, st, ct, qr, vt, lim, offs any;
+
+  lines := http_request_header ();
+  accept := http_request_header_full (lines, 'Accept', '*/*');
+  accept := DB.DBA.HTTP_RDF_GET_ACCEPT_BY_Q (accept);
+  set http_charset='utf-8';
+  declare exit handler for sqlstate '*'
+    {
+      http_status_set (500);
+      resignal;
+    };
+  inf := fct_http_param ('inference');  
+  sas := fct_http_param ('same-as');
+  st := fct_http_param ('s-term');
+  ct := fct_http_param ('c-term');
+  qr := fct_http_param ('q');
+  vt := fct_http_param ('view:type', 'text-d');
+  if (vt = 'text') vt := 'text-d';
+  if (vt = 'entity-types') vt := 'classes';
+  if (vt = 'attribute-names') vt := 'properties';
+  if (vt = 'attribute-values') vt := 'properties-in';
+  if (vt = 'values') vt := 'properties-in';
+  lim := atoi (fct_http_param ('limit', '20'));
+  offs := atoi (fct_http_param ('offset', '0'));
+  cnt := sprintf ('<query inference="%s" same-as="%s" s-term="%s" c-term="%s"><text>%V</text><view type="%s" limit="%d" offset="%d" /></query>', 
+     inf, sas, st, ct, qr, vt, lim, offs);
+  if (accept = 'text/html' or accept = '*/*' or accept = 'application/xhtml+xml')
+    {
+      declare full_path, p_full_path, long_url, pars any;
+      full_path := '/fct/facet.vsp';
+      long_url := null;
+      if (length (qr))
+	pars := vector ('qxml', cnt);
+      else
+        pars := vector ();
+      p_full_path := http_physical_path_resolve (full_path, 1);
+      http_internal_redirect (full_path, p_full_path, long_url, 0);
+      http_set_params (pars);
+      lines := http_request_header ();
+      WS.WS.GET (vector ('DAV', 'VAD', 'fct', 'facet.vsp'), pars, lines);
+      --http_header (sprintf ('Location: /fct/facet.vsp?qxml=%U\r\n', cnt));
+    }
+  else  
+    {
+      http_status_set (303);
+      http_header (sprintf ('Location: /sparql?query=%U&format=%U\r\n', fct_query (xtree_doc (cnt)), accept));
+    }
+ret:
+  return '';
+}
+;
+
 create procedure fct_svc () __soap_http 'text/xml'
 {
   declare cnt, tp, ret, timeout, xt, xslt, maxt, tmp, lines, accept any;
@@ -119,21 +191,26 @@ create procedure fct_svc () __soap_http 'text/xml'
   accept := http_request_header_full (lines, 'Accept', '*/*');
   accept := DB.DBA.HTTP_RDF_GET_ACCEPT_BY_Q (accept);
   set http_charset='utf-8';
-  if (tp <> 'text/xml')
+  if (tp <> 'text/xml' and tp <> 'application/json')
     {
       http_status_set (500);
-      return '<error><code>22023</code><message>Invalid content type</message><diagnostics/></error>';
+      http_header ('Content-Type: text/plain\r\n');
+      return 'Invalid content type, supported are: application/json and text/xml';
     }
   cnt := http_body_read ();
 
+  if (tp = 'application/json')
+    cnt := json2xml (string_output_string (cnt));
+
 --  dbg_obj_print ('fct_svc');
 --  dbg_obj_print (string_output_string (cnt));
 
   declare exit handler for sqlstate '*'
     {
       http_status_set (500);
-      return sprintf ('<error><code>%V</code><message>Error while executing query</message><diagnostics>%V</diagnostics></error>',
+      ret := sprintf ('<error><code>%V</code><message>Error while executing query</message><diagnostics>%V</diagnostics></error>',
 	  __SQL_STATE, __SQL_MESSAGE);
+      goto ret;
     };
   xt := xtree_doc (cnt);
   xslt := xslt (registry_get ('_fct_xslt_') || 'fct_req.xsl', xt);
@@ -148,11 +225,18 @@ create procedure fct_svc () __soap_http 'text/xml'
   if (0 >= timeout or timeout > maxt)
     timeout := maxt;
   ret := fct_svc_exec (xslt, timeout, accept, lines);
+ret:
+  if (tp = 'application/json')
+    {
+      http_header ('Content-Type: application/json\r\n');
+      ret := xml2json (ret);
+    }
   return ret;
 }
 ;
 
 grant execute on fct_svc to SPARQL_SELECT;
+grant execute on fct_search to SPARQL_SELECT;
 
 DB.DBA.VHOST_REMOVE (lpath=>'/fct/soap');
 DB.DBA.VHOST_DEFINE (lpath=>'/fct/soap', ppath=>'/SOAP/', soap_user=>'SPARQL');
