@@ -4,7 +4,7 @@
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --
---  Copyright (C) 1998-2014 OpenLink Software
+--  Copyright (C) 1998-2016 OpenLink Software
 --
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -149,6 +149,33 @@ b3s_handle_ses (inout _path any, inout _lines any, inout _params any)
 ;
 
 -- XXX should probably find the most specific if more than one class and inference rule is set
+create procedure 
+b3s_e_type (in subj varchar)
+{
+  declare meta, data, ll any;
+  declare i int;
+  declare stat, msg any;
+
+  ll := 'http://www.w3.org/2002/07/owl#Thing';
+
+  if (length (subj))
+    {	
+      stat := '00000';
+      data := null;
+      exec (sprintf ('sparql select ?tp where { <%s> a ?tp }', subj), stat, msg, vector (), 100, meta, data);
+
+      if (length (data))
+	{
+	  for (i := 0; i < length (data); i := i + 1) 
+            {
+              if (data[i][0] is not null)
+  	        return data[i][0];
+            }
+	}
+    }
+  return ll;
+}
+;
 
 create procedure
 b3s_type (in subj varchar,
@@ -165,7 +192,7 @@ b3s_type (in subj varchar,
 
   if (length (subj))
     {
-      exec (sprintf ('sparql select ?l ?tp %s where { <%S> a ?tp optional { ?tp rdfs:label ?l } }', _from, subj), 
+      exec (sprintf ('sparql select ?l ?tp %s where { <%s> a ?tp optional { ?tp rdfs:label ?l } }', _from, subj), 
 	  null, null, vector (), 100, meta, data);
 
       if (length (data))
@@ -220,7 +247,7 @@ create procedure b3s_find_class_type (in _s varchar, in _f varchar, inout types_
   st := '00000';
   msg:= '';
 
-  stmt := sprintf ('sparql select ?to %s where { quad map virtrdf:DefaultQuadMap { ?to a <%S> }}', _f, _s);
+  stmt := sprintf ('sparql select ?to %s where { quad map virtrdf:DefaultQuadMap { ?to a <%s> }}', _f, _s);
 
   exec (stmt, st, msg, vector(), 1, meta, data);
 
@@ -259,7 +286,7 @@ b3s_get_types (in _s varchar,
   declare i int;
   declare stmt varchar;
 
-  stmt := sprintf ('sparql select distinct ?tp %s where { quad map virtrdf:DefaultQuadMap { <%S> a ?tp } }', _from, _s);
+  stmt := sprintf ('sparql select distinct ?tp %s where { quad map virtrdf:DefaultQuadMap { <%s> a ?tp } }', _from, _s);
   data := null;
   t_a := vector();
 
@@ -293,7 +320,7 @@ b3s_get_all_types (in _s varchar,
   declare i int;
   declare stmt varchar;
 
-  stmt := sprintf ('sparql select distinct ?tp %s where { <%S> a ?tp }', _from, _s);
+  stmt := sprintf ('sparql select distinct ?tp %s where { <%s> a ?tp }', _from, _s);
   data := null;
   t_a := vector();
 
@@ -442,6 +469,21 @@ b3s_parse_inf (in sid varchar, inout params any)
 	vectorbld_acc (grs, params[i+1]);
     }
   vectorbld_final (grs);
+  if (length (grs) = 0 and sid is not null and get_keyword ('set_graphs', params) is null)
+    {
+      declare xt, xp, inx any;
+      xt := (select fct_state from fct_state where fct_sid = sid);
+      inx := 1;
+      vectorbld_init (grs);
+      while (xt is not null and (xp := xpath_eval (sprintf ('//query/@graph%d', inx), xt)) is not null)
+	{
+	  vectorbld_acc (grs, cast (xp as varchar));
+	  inx := inx + 1;
+	}
+      vectorbld_final (grs);
+    }
+  if (get_keyword ('clear_graphs', params) is not null)
+    grs := vector ();
   connection_set ('graphs', grs);
 }
 ;
@@ -585,6 +627,8 @@ create procedure b3s_label_get (inout data any, in langs any)
    if (not isstring (label))
      label := cast (label as varchar);
    --label := regexp_replace (label, '<[^>]+>', '', 1, null);
+  if (label is null)
+    label := ''; 
   if (0 and sys_stat ('cl_run_local_only'))
     {
       label := xpath_eval ('string(.)', xtree_doc (label, 2));
@@ -596,44 +640,6 @@ create procedure b3s_label_get (inout data any, in langs any)
 }
 ;
 
-create procedure
-b3s_rel_print (in val any, in rel any, in flag int := 0)
-{
-  declare delim, delim1, delim2, delim3 integer;
-  declare inx int;
-  declare nss, loc, nspref varchar;
-
-  delim1 := coalesce (strrchr (val, '/'), -1);
-  delim2 := coalesce (strrchr (val, '#'), -1);
-  delim3 := coalesce (strrchr (val, ':'), -1);
-  delim := __max (delim1, delim2, delim3);
-  nss := null;
-  loc := val;
-  if (delim < 0) return loc;
-  nss := subseq (val, 0, delim + 1);
-  loc := subseq (val, delim + 1);
-
-  nspref := __xml_get_ns_prefix (nss, 2);
-  if (nspref is null)
-    {
-      inx := connection_get ('ns_ctr');
-      connection_set ('ns_ctr', inx + 1);
-      nspref := sprintf ('ns%d', inx);
-    }
-
-
-  nss := sprintf ('xmlns:%s="%s"', nspref, nss);
-  if (flag)
-    loc := sprintf ('property="%s:%s"', nspref, loc);
-  else if (rel)
-    loc := sprintf ('rel="%s:%s"', nspref, loc);
-  else
-    loc := sprintf ('rev="%s:%s"', nspref, loc);
-  return concat (loc, ' ', nss);
-}
-;
-
-
 create procedure
 b3s_uri_curie (in uri varchar)
 {
@@ -642,9 +648,11 @@ b3s_uri_curie (in uri varchar)
 
   delim := -1;
 
-  uriSearch := uri;
   if (uri is null)
     return '';
+  if (iswidestring (uri))
+    uri := charset_recode (uri, '_WIDE_', 'UTF-8');
+  uriSearch := uri;
   nsPrefix := null;
   while (nsPrefix is null and delim <> 0) {
 
@@ -774,7 +782,7 @@ create procedure b3s_label (in _S any, in langs any, in lbl_order_pref_id int :=
     }
   stat := '00000';
   --exec (sprintf ('sparql define input:inference "facets" '||
-  --'select ?o (lang(?o)) where { <%S> virtrdf:label ?o }', _S), stat, msg, vector (), 0, meta, data);
+  --'select ?o (lang(?o)) where { <%s> virtrdf:label ?o }', _S), stat, msg, vector (), 0, meta, data);
   exec (sprintf ('select __ro2sq (O), DB.DBA.RDF_LANGUAGE_OF_OBJ (__ro2sq (O)) , cast (b3s_lbl_order (P, %d) as int) from RDF_QUAD table option (with ''facets'')
 	where S = __i2id (?) and P = __i2id (''http://www.openlinksw.com/schemas/virtrdf#label'', 0) and not is_bnode_iri_id (O) order by 3 option (same_as)', lbl_order_pref_id), 
 	stat, msg, vector (_S), 0, meta, data);
@@ -813,10 +821,39 @@ create procedure b3s_xsd_link (in t varchar)
 }
 ;
 
-create procedure
-b3s_http_print_r (in _object any, in sid varchar, in prop any, in langs any, in rel int := 1, in acc any := null, in _from varchar := null, in flag int := 0)
+create procedure b3s_o_is_out (in x any)
 {
-   declare lang, rdfs_type, rdfa, visible any;
+  declare f, s, og any;
+  f := 'http://xmlns.com/foaf/0.1/';
+  s := 'http://schema.org/';
+  og := 'http://opengraphprotocol.org/schema/';
+  -- foaf:page, foaf:homePage, foaf:img, foaf:logo, foaf:depiction
+  if (__ro2sq (x) in (f||'page', f||'homePage', f||'img', f||'logo', f||'depiction', 'http://schema.org/url', 'http://schema.org/downloadUrl', 'http://schema.org/potentialAction', s||'logo', s||'image', s || 'mainEntityOfPage', og || 'image', 'http://www.openlinksw.com/ontology/webservices#usageExample'))
+    {
+      return 1;
+    }
+  return 0;
+}
+;
+
+create procedure b3s_o_is_img (in x any)
+{
+  declare f, s, og any;
+  f := 'http://xmlns.com/foaf/0.1/';
+  s := 'http://schema.org/';
+  og := 'http://opengraphprotocol.org/schema/';
+  if (__ro2sq (x) in (f||'img', f||'logo', f||'depiction', s||'logo', s||'image', og || 'image', 'http://ogp.me/ns#image'))
+    {
+      return 1;
+    }
+  return 0;
+}
+;
+
+create procedure
+b3s_http_print_r (in subj any, in _object any, in sid varchar, in prop any, in langs any, in rel int := 1, in acc any := null, in _from varchar := null, in flag int := 0)
+{
+   declare lang, rdfs_type, rdfa, visible, itemid any;
 
    if (_object is null)
      return;
@@ -840,9 +877,19 @@ b3s_http_print_r (in _object any, in sid varchar, in prop any, in langs any, in
    if (__tag of IRI_ID = __tag (rdfs_type))
      rdfs_type := id_to_iri (rdfs_type);
 
-   rdfa := b3s_rel_print (prop, rel, 1);
+   rdfa := sprintf ('itemprop="%s"', prop);
    visible := b3s_str_lang_check (lang, acc);
-   http (sprintf ('\t<li%s><span class="literal">', case visible when 0 then ' style="display:none;"' else '' end));
+   itemid := subj;
+   if (flag = 1 and (__tag (_object) = 243 or 
+     (isstring (_object) and (__box_flags (_object)= 1 or _object like 'nodeID://%' or _object like 'http://%'))))
+     {
+       if (__tag of IRI_ID = __tag (_object))
+	 itemid := id_to_iri (_object);
+       else
+	 itemid := _object;
+     }
+   http (sprintf ('\t<li%s itemid="%s" itemscope itemtype="%s"><span class="literal">', 
+   	case visible when 0 then ' style="display:none;"' else '' end, itemid, b3s_e_type (itemid)));
 again:
    if (__tag (_object) = 246)
      {
@@ -865,17 +912,24 @@ again:
 
        http (sprintf ('<!-- %d -->', length (_url)));
 
-       rdfa := b3s_rel_print (prop, rel, 0);
        if (prop = 'http://bblfish.net/work/atom-owl/2006-06-06/#content' and _object like '%#content%')
 	 {
 	   declare src any;
 	   whenever not found goto usual_iri;
 	   select id_to_iri (O) into src from DB.DBA.RDF_QUAD where
 	   	S = iri_to_id (_object, 0) and P = iri_to_id ('http://bblfish.net/work/atom-owl/2006-06-06/#src', 0);
-	   http (sprintf ('<div id="x_content"><iframe src="%s" width="100%%" height="100%%" frameborder="0"><p>Your browser does not support iframes.</p></iframe></div><br/>', src));
+	   http (sprintf ('<div id="x_content"><iframe src="%s" width="100%%" height="100%%" frameborder="0" sandbox=""><p>Your browser does not support iframes.</p></iframe></div><br/>', src));
+	   http (sprintf ('<link %s href="%s"/>', rdfa, case when flag = 0 then _object else subj end));
+	 }
+       else if (http_mime_type (_url) like 'image/%' or http_mime_type (_url) = 'application/x-openlink-photo' or b3s_o_is_img (prop))
+	 {
+	   declare u any;
+	   if (b3s_o_is_out (prop))
+	     u := _url;
+	   else
+	     u := b3s_http_url (_url, sid, _from);
+	   http (sprintf ('<a class="uri" %s href="%s"><img src="%s" height="160" style="border-width:0" alt="External Image" /></a>', rdfa, u, _url));
 	 }
-       else if (http_mime_type (_url) like 'image/%' or http_mime_type (_url) = 'application/x-openlink-photo')
-	 http (sprintf ('<a class="uri" %s href="%s"><img src="%s" height="160" style="border-width:0" alt="External Image" /></a>', rdfa, b3s_http_url (_url, sid, _from), _url));
        else
 	 {
 	   usual_iri:;
@@ -887,14 +941,14 @@ again:
 	     lbl := b3s_uri_curie(_url);
 	   -- XXX: must encode as wide label to print correctly
 	   --http (sprintf ('<a class="uri" %s href="%s">%V</a>', rdfa, b3s_http_url (_url, sid, _from), lbl));
-	   http (sprintf ('<a class="uri" %s href="%s">', rdfa, b3s_http_url (_url, sid, _from)));
+	   http (sprintf ('<a class="uri" href="%s">', b3s_http_url (_url, sid, _from)));
 	   vlbl := charset_recode (lbl, 'UTF-8', '_WIDE_');
 	   http_value (case when vlbl <> 0 then vlbl else lbl end);
 	   http (sprintf ('</a>'));
+	   http (sprintf ('<link %s href="%s"/>', rdfa, case when flag = 0 then _url else subj end));
+	   if (b3s_o_is_out (prop) and flag = 0)
+	     http (sprintf ('&nbsp;<a href="%s"><img src="/fct/images/fct-linkout-16-blk.png" border="0"/></a>', _url));
 	 }
-       --if (registry_get ('fct_sponge') = '1' and _url like 'http://%' or _url like 'https://%')
-       --	 http (sprintf ('&nbsp;<a class="uri" href="%s&sp=1"><img src="/fct/images/goout.gif" alt="Sponge" title="Sponge" border="0"/></a>', 
-       --	       b3s_http_url (_url, sid)));
 
      }
    else if (__tag (_object) = 189)
@@ -920,6 +974,11 @@ again:
    else if (__tag (_object) = 182)
      {
        declare vlbl any;
+       if (b3s_o_is_img (prop))
+	 {
+	   __box_flags_set (_object, 1);
+	   goto again;
+	 }
        http (sprintf ('<span %s>', rdfa));
        if (strstr (_object, 'http://') is not null)
 	 {
@@ -929,7 +988,7 @@ again:
        vlbl := charset_recode (_object, 'UTF-8', '_WIDE_');
        if (vlbl = 0)
          vlbl := charset_recode (_object, current_charset (), '_WIDE_');
-       if (vlbl = 0 or _object like '<object%')
+       if (vlbl = 0 or _object like '<object%' or _object like '<iframe%' or ltrim (_object) like '<%' or strstr (_object, '<a href') is not null)
          http (_object);
        else
          http_value (vlbl);
@@ -939,7 +998,7 @@ again:
    else if (__tag (_object) = 211)
      {
        http (sprintf ('<span %s>%s</span>', rdfa, datestring (_object)));
-       lang := b3s_xsd_link ('date');
+       lang := b3s_xsd_link ('dateTime');
      }
    else if (__tag (_object) = 230)
      {
@@ -1072,26 +1131,6 @@ fct_make_selector (in subj any, in sid integer)
 }
 ;
 
-create procedure fct_make_qr_code (in data_to_qrcode any, in src_width int := 120, in src_height int := 120, in qr_scale int := 3)
-{
-  declare qrcode_bytes, mixed_content, content varchar;
-  declare qrcode any;
-
-  if (__proc_exists ('QRcode encodeString8bit', 2) is null)
-    return null;
-
-  declare exit handler for sqlstate '*' { return null; };
-
-  content := "IM CreateImageBlob" (src_width, src_height, 'white', 'jpg');
-  qrcode := "QRcode encodeString8bit" (data_to_qrcode);
-  qrcode_bytes := aref_set_0 (qrcode, 0);
-  mixed_content := "IM PasteQRcode" (qrcode_bytes, qrcode[1], qrcode[2], qr_scale, qr_scale + 2, 0, 0, cast (content as varchar), length (content));
-  mixed_content := encode_base64 (cast (mixed_content as varchar));
-  mixed_content := replace (mixed_content, '\r\n', '');
-  return mixed_content;
-}
-;
-
 create procedure fct_make_curie (in url varchar, in lines any)
 {
   declare curie, chost, dhost varchar;
@@ -1242,4 +1281,221 @@ create procedure b3s_gs_check_needed ()
 }
 ;
 
+--- Identifies the graph(s) containing the given entity
+--- * For entity URIs of the form /about/id[/entity]/{data_source_uri}[#child_entity_id] or /proxy-iri/xxx
+---   ensures that the correct data source URI is sponged
+-- * Allows us the check the permissions on the graph before an attempted read or sponge and, if reading or
+--   sponging is denied, provide some feedback in the /describe UI, rather than just display an empty result set.
+-- * If we're not handling a sponge request and the given entity is present in multiple graphs:
+--     * null is returned for the entity graph
+--     * we then make no attempt to determine the user's permissions on these graphs prior to the select to
+--       fetch the results for display. It's assumed that RDF_GRAPH_USER_PERMS_ACK() will filter the results
+--       from any graphs for which the user doesn't have read permission.
+create procedure b3s_get_entity_graph (in entity_uri varchar, in sponge_request int)
+{
+  declare arr, pa, sch, nhost, tmp, npath, entity_graph any;
+
+  arr := rfc1808_parse_uri (entity_uri);
+  if (arr[0] = 'nodeID')
+    return rtrim (entity_uri, '/');
+
+  if (not (arr[2] like '/about/id%' or arr[2] like '/proxy-iri/%'))
+  {
+    if (sponge_request)
+      return entity_uri; -- the entity description is sponged to a graph with the same URI
+    else
+    {
+      declare num_containing_graphs int;
+      num_containing_graphs := (
+	select count(distinct G) from DB.DBA.RDF_QUAD where 
+          S = iri_to_id (entity_uri) and 
+	  G not in (select RGGM_MEMBER_IID from DB.DBA.RDF_GRAPH_GROUP_MEMBER 
+                    where RGGM_GROUP_IID = iri_to_id('http://www.openlinksw.com/schemas/virtrdf#PrivateGraphs')));
+      if (num_containing_graphs > 1)
+      {
+	return null;
+      }
+      else
+      {
+	entity_graph := (select top 1 id_to_iri(G) from DB.DBA.RDF_QUAD where 
+          S = iri_to_id (entity_uri) and 
+	  G not in (select RGGM_MEMBER_IID from DB.DBA.RDF_GRAPH_GROUP_MEMBER 
+                    where RGGM_GROUP_IID = iri_to_id('http://www.openlinksw.com/schemas/virtrdf#PrivateGraphs')));
+	-- Assume client is attempting to view an empty graph
+	if (entity_graph is not null)
+	{
+	  ;
+	}
+	else
+	{
+	  entity_graph := entity_uri;
+	}
+	return entity_graph;
+      }
+    }
+  }
+
+  -- Handle /about/id/* and /proxy-iri/* style entity URIs
+
+  entity_graph := (select top 1 id_to_iri(G) from DB.DBA.RDF_QUAD where 
+	  S = iri_to_id (entity_uri) and
+	  G not in (select RGGM_MEMBER_IID from DB.DBA.RDF_GRAPH_GROUP_MEMBER 
+                    where RGGM_GROUP_IID = iri_to_id('http://www.openlinksw.com/schemas/virtrdf#PrivateGraphs')));
+  if (entity_graph is not null)
+  {
+    return entity_graph;
+  }
+
+  -- Assume the original containing graph has been cleared. Deduce it.
+
+  if (arr[2] like '/proxy-iri/%')
+    return RDF_SPONGE_PROXY_IRI_GET_GRAPH (entity_uri);
+
+  entity_graph := entity_uri;
+  -- Strip off fragment - entity_uri could be a child entity with a hash URI
+  arr[5] := ''; 
+
+  pa := split_and_decode (arr[2], 0, '\0\0/');
+  if (length (pa) > 5 and pa[3] = 'entity' and pa[4] <> '' and pa [5] <> '')
+{
+    -- Set entity_graph to the URI following /about/id/entity/
+    sch := pa[4];
+    nhost := pa [5];
+    tmp := '/about/id/entity/' || sch || '/' || nhost;
+    npath := subseq (arr[2], length (tmp));    
+    arr[0] := sch;
+    arr[1] := nhost;
+    arr[2] := npath;
+    
+    if (lower(arr[0]) in ('acct', 'mailto')) 
+    {
+      arr [2] := arr[1];
+      arr [1] := '';
+    }
+
+    entity_graph := DB.DBA.vspx_uri_compose (arr);
+  }
+  else if (length (pa) > 4 and pa[3] <> '' and pa [4] <> '')
+  {
+    -- Set entity_graph to the URI following /about/id/
+    sch := pa[3];
+    nhost := pa [4];
+    tmp := '/about/id/' || sch || '/' || nhost;
+    npath := subseq (arr[2], length (tmp));    
+    arr[0] := sch;
+    arr[1] := nhost;
+    arr[2] := npath;
+    
+    if (sch in ('acct', 'mailto'))
+    {
+      arr[2] := arr[1];
+      arr[1] := '';
+    }
+	    
+    entity_graph := DB.DBA.vspx_uri_compose (arr);
+  }
+
+  return entity_graph;
+}
+;
+
+-- Checks a user's permissions on a single graph
+create procedure b3s_get_user_graph_permissions (
+  in graph varchar,
+  in pageUrl varchar,
+  in sponge_request int,
+  in val_vad_present int,
+  in val_serviceId varchar,
+  in val_auth_method int,
+  inout graph_perms_allow_sponge int,
+  inout view_mode varchar
+  )
+{
+  declare user_permissions int;
+
+  view_mode := 'full';
+  graph_perms_allow_sponge := 1;
+  user_permissions := 15;
+
+  -- graph == null indicates that the subject entity URI being viewed is contained in multiple graphs.
+  -- Don't attempt to check permissions here if this is the case, as here we only check permissions
+  -- on a single graph
+  -- FIX ME: 
+  -- See use of RDF_GRAPH_USER_PERMS_ACK [1] by dt1 and dt2 in description.vsp. 
+  -- Filtering of results from multiple graphs should be done at this point [1]
+
+  if (graph is not null)
+    user_permissions := DB.DBA.RDF_GRAPH_USER_PERMS_GET (graph, http_nobody_uid());
+
+  if (bit_and (user_permissions, 1) = 0)
+  {
+    -- User doesn't have read permission
+    view_mode := 'none';
+    graph_perms_allow_sponge := 0;
+  }
+  else if (bit_and (user_permissions, 4) = 0)
+  {
+    graph_perms_allow_sponge := 0;
+    if (bit_and (user_permissions, 2))
+      view_mode := 'read-write';
+    else
+      view_mode := 'read-only';
+  }
+}
+;
+
 grant execute on b3s_gs_check_needed to public;
+
+create procedure fct_set_graphs (in sid any, in graphs any)
+{
+  declare xt, newx, s any;
+  if (sid is null) return;
+  xt := (select fct_state from fct_state where fct_sid = sid);
+  s := string_output ();
+  http ('<graphs>', s);
+  foreach (any g in graphs) do
+    {
+      http (sprintf ('<graph name="%V" />', g), s);
+    } 
+  http ('</graphs>', s);
+  newx := xslt (registry_get ('_fct_xslt_') || 'fct_set_graphs.xsl', xt, vector ('graphs', xtree_doc (s)));
+  update fct_state set fct_state = newx where fct_sid = sid; 
+  commit work;
+}
+;
+
+
+create procedure FCT.DBA.build_page_url_on_current_host (
+  in path varchar,
+  in query varchar)
+{
+  declare protocol varchar;
+  declare host any;
+
+  host := http_request_header (http_request_header (), 'X-Forwarded-Host', null, null);
+  if (host is null)
+    host := http_request_header (http_request_header (), 'Host');
+
+  protocol := 'http'; if (is_https_ctx()) protocol := 'https';
+
+  return sprintf ('%s://%s%s?%s', protocol, host, path, query);
+}
+;
+
+create procedure FCT.DBA.get_describe_request_params (
+  in params any
+  )
+{
+  declare desc_params varchar;
+
+  desc_params := '';
+  if (get_keyword ('sp', params) is not null)
+    desc_params := desc_params || '&sp=' || get_keyword ('sp', params);
+  if (get_keyword ('sponger:get', params) is not null)
+    desc_params := desc_params || '&sponger:get=' || get_keyword ('sponger:get', params);
+  if (get_keyword ('sr', params) is not null)
+    desc_params := desc_params || '&sr=' || get_keyword ('sr', params);
+
+  return desc_params;
+}
+;
