@@ -4,7 +4,7 @@
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --
---  Copyright (C) 1998-2014 OpenLink Software
+--  Copyright (C) 1998-2016 OpenLink Software
 --
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -117,10 +117,10 @@ create procedure DB.DBA.DBP_GRAPH_PARAM1 (in par varchar, in fmt varchar, in val
       if (length (val) = 0)
 	val := '';
       if (val = 'en')
-        val := '';  
+        val := '';
       if (val <> '')
 	{
-          val := 'http://' || val || '.dbpedia.org';	
+          val := 'http://' || val || '.dbpedia.org';
 	  tmp := tmp || sprintf ('&named-graph-uri=%U', val);
 	}
     }
@@ -234,7 +234,7 @@ DB.DBA.VHOST_DEFINE ( lhost=>registry_get ('dbp_lhost'), vhost=>registry_get ('d
 
 create procedure DB.DBA.DBP_LINK_HDR (in in_path varchar)
 {
-  declare host, lines, accept, loc, alt, exp any;
+  declare host, lines, accept, loc, alt, exp, lic any;
   lines := http_request_header ();
 --  dbg_obj_print ('in_path: ', in_path);
 --  dbg_obj_print ('lines: ', lines);
@@ -252,11 +252,11 @@ create procedure DB.DBA.DBP_LINK_HDR (in in_path varchar)
 	  if (accept is null)
 	    accept := 'application/rdf+xml';
 	  if (accept = 'application/rdf+xml')
-	    loc := 'Content-Location: ' || tmp || '.xml\r\n';	
+	    loc := 'Content-Location: ' || tmp || '.xml\r\n';
 	  else if (accept = 'text/rdf+n3')
-	    loc := 'Content-Location: ' || tmp || '.n3\r\n';	
+	    loc := 'Content-Location: ' || tmp || '.n3\r\n';
 	  else if (accept = 'text/n3')
-	    loc := 'Content-Location: ' || tmp || '.n3\r\n';	
+	    loc := 'Content-Location: ' || tmp || '.n3\r\n';
 	}
     }
   if (in_path like '/data/%')
@@ -302,7 +302,8 @@ create procedure DB.DBA.DBP_LINK_HDR (in in_path varchar)
 	alt := alt || sprintf ('<%s>; rel="hub", ', registry_get ('dbp_pshb_hub'));
       exp := sprintf ('Expires: %s\r\n', date_rfc1123 (dateadd ('day', 7, now ())));
     }
-  return sprintf ('%s%sLink: %s<http://mementoarchive.lanl.gov/dbpedia/timegate/http://%s%s>; rel="timegate"', exp, loc, alt, host, in_path);
+  lic := '<http://creativecommons.org/licenses/by-sa/3.0/>;rel="license",';
+  return sprintf ('%s%sLink: %s%s<http://mementoarchive.lanl.gov/dbpedia/timegate/http://%s%s>; rel="timegate"', exp, loc, lic, alt, host, in_path);
 }
 ;
 
@@ -325,7 +326,7 @@ create procedure DB.DBA.DBP_DATA_IRI1 (in par varchar, in fmt varchar, in val va
 }
 ;
 DB.DBA.URLREWRITE_CREATE_REGEX_RULE ( 'dbp_rule_12', 1, '/resource/([^\\?]*)(\\?lang=.*)?\x24', vector ('par_1', 'par_2'), 1,
-    '/data/@__@%s', vector ('par_1'), 'DB.DBA.DBP_DATA_IRI1', 
+    '/data/@__@%s', vector ('par_1'), 'DB.DBA.DBP_DATA_IRI1',
     '(application/rdf.xml)|(text/rdf.n3)|(text/n3)|(application/x-turtle)|(application/rdf.json)|(application/json)|(application/atom.xml)|(application/odata.json)', 2, 303, '^{sql:DB.DBA.DBP_LINK_HDR}^');
 
 create procedure DB.DBA.DBP_TCN_LOC (in id any, in var any)
@@ -387,7 +388,7 @@ DB.DBA.VHOST_DEFINE ( lhost=>registry_get ('dbp_lhost'), vhost=>registry_get ('d
 DB.DBA.URLREWRITE_CREATE_RULELIST ( 'dbp_wc_rule_list1', 1, vector ('dbp_wc_rule1', 'dbp_wc_rule2'));
 
 DB.DBA.URLREWRITE_CREATE_REGEX_RULE ( 'dbp_wc_rule1', 1, '(/[^#]*)', vector ('par_1'), 1,
-registry_get('_dbpedia_path_')||'description_white.vsp?res=%s', vector ('par_1'), NULL, NULL, 2, 0, '');
+registry_get('_dbpedia_path_')||'description.vsp?res=%s', vector ('par_1'), NULL, NULL, 2, 0, '');
 
 DB.DBA.URLREWRITE_CREATE_REGEX_RULE ( 'dbp_wc_rule2', 1, '(/[^#]*)', vector ('par_1'), 1,
 '/sparql?query=describe%%20%%3Chttp%%3A%%2F%%2Fdbpedia.openlinksw.com%s%%3E%%20from%%20%%3Chttp%%3A%%2F%%2Fdbpedia.openlinksw.com%%2Fwikicompany%%3E&format=%U',
@@ -437,7 +438,7 @@ DB.DBA.URLREWRITE_CREATE_REGEX_RULE ( 'pvsp_data4_rule', 1, '/data4/(.*)\\.(n3|r
 '/sparql?default-graph-uri=http%%3A%%2F%%2F'||replace(registry_get('dbp_graph'),'http://','')||'&query=DESCRIBE+%%3Chttp%%3A%%2F%%2Fdbpedia.org%%2Fproperty%%2F%U%%3E&format=%U',
 vector ('par_1', 'f'), NULL, NULL, 2, null, '');
 
---# about 
+--# about
 DB.DBA.VHOST_DEFINE (
 	 lhost=>registry_get ('dbp_lhost'),
 	 vhost=>registry_get ('dbp_vhost'),
@@ -450,22 +451,22 @@ DB.DBA.VHOST_DEFINE (
 	 is_default_host=>0
 );
 
-DB.DBA.URLREWRITE_CREATE_RULELIST ( 
-    'ext_about_http_proxy_rule_list1', 1, 
+DB.DBA.URLREWRITE_CREATE_RULELIST (
+    'ext_about_http_proxy_rule_list1', 1,
       vector ('dbp_about_rule_1'));
 
-DB.DBA.URLREWRITE_CREATE_REGEX_RULE ( 
-    'dbp_about_rule_1', 1, 
-      '/about/html/(.*)\x24', 
-      vector ('par_1'), 
-      1, 
-      '/DAV/VAD/dbpedia/description.vsp?res=%U', 
-      vector ('par_1'), 
-      NULL, 
-      NULL, 
-      2, 
-      0, 
-      '' 
+DB.DBA.URLREWRITE_CREATE_REGEX_RULE (
+    'dbp_about_rule_1', 1,
+      '/about/html/(.*)\x24',
+      vector ('par_1'),
+      1,
+      '/DAV/VAD/dbpedia/description.vsp?res=%U',
+      vector ('par_1'),
+      NULL,
+      NULL,
+      2,
+      0,
+      ''
       );
 
 DB.DBA.VHOST_REMOVE (
@@ -601,7 +602,7 @@ DB.DBA.VHOST_DEFINE ( lhost=>registry_get ('dbp_lhost'), vhost=>registry_get ('d
 	 opts=>vector ('url_rewrite', 'dbpl_void_page_rule_list'),
 	 is_default_host=>0
 );
-    
+
 DB.DBA.VHOST_DEFINE ( lhost=>registry_get ('dbp_lhost'), vhost=>registry_get ('dbp_vhost'), lpath=>'/isparql',
 	 ppath=>'/DAV/VAD/iSPARQL/',
 	 is_dav=>1,
@@ -627,7 +628,7 @@ DB.DBA.VHOST_DEFINE ( lhost=>registry_get ('dbp_lhost'), vhost=>registry_get ('d
 	 is_default_host=>0
 );
 
-create procedure DB.DBA.SPARQL_DESC_DICT_DBPEDIA_PHYSICAL 
+create procedure DB.DBA.SPARQL_DESC_DICT_DBPEDIA_PHYSICAL
 (in subj_dict any, in consts any, in good_graphs any, in bad_graphs any, in storage_name any, in options any)
 {
   declare res, subjs any;
@@ -635,7 +636,7 @@ create procedure DB.DBA.SPARQL_DESC_DICT_DBPEDIA_PHYSICAL
   if (is_http_ctx ())
     {
       subjs := dict_to_vector (subj_dict, 0);
-      for (declare i int, i := 0; i < length (subjs); i := i + 2) 
+      for (declare i int, i := 0; i < length (subjs); i := i + 2)
       {
 	declare s any;
 	s := subjs [i];
@@ -656,15 +657,15 @@ DB.DBA.VHOST_REMOVE ( lhost=>registry_get ('dbp_lhost'), vhost=>registry_get ('d
 DB.DBA.VHOST_REMOVE ( lhost=>registry_get ('dbp_lhost'), vhost=>registry_get ('dbp_vhost'), lpath=>'/services/rdf/iriautocomplete.get');
 DB.DBA.VHOST_REMOVE ( lhost=>registry_get ('dbp_lhost'), vhost=>registry_get ('dbp_vhost'), lpath=>'/describe');
 
-DB.DBA.VHOST_DEFINE (lhost=>registry_get ('dbp_lhost'), vhost=>registry_get ('dbp_vhost'), 
-    lpath=>'/fct', ppath=>'/DAV/VAD/fct/', 
+DB.DBA.VHOST_DEFINE (lhost=>registry_get ('dbp_lhost'), vhost=>registry_get ('dbp_vhost'),
+    lpath=>'/fct', ppath=>'/DAV/VAD/fct/',
     is_dav=>1, def_page=>'facet.vsp', vsp_user=>'dba', ses_vars=>0, is_default_host=>0);
 
 
 DB.DBA.VHOST_DEFINE (lhost=>registry_get ('dbp_lhost'), vhost=>registry_get ('dbp_vhost'),
 	 lpath=>'/fct/service', ppath=>'/SOAP/Http/fct_svc',
 	 is_dav=>0, soap_user=>'dba', ses_vars=>0, is_default_host=>0);
-    
+
 DB.DBA.VHOST_DEFINE (lhost=>registry_get ('dbp_lhost'), vhost=>registry_get ('dbp_vhost'),
 	 lpath=>'/fct/soap', ppath=>'/SOAP/',
 	 is_dav=>0, soap_user=>'dba', ses_vars=>0, is_default_host=>0);
