@@ -5,7 +5,7 @@
 #  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 #  project.
 #
-#  Copyright (C) 1998-2014 OpenLink Software
+#  Copyright (C) 1998-2016 OpenLink Software
 #
 #  This project is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -164,8 +164,10 @@ directory_init() {
   mkdir vad/vsp
   mkdir vad/vsp/dbpedia
   mkdir vad/vsp/dbpedia/statics
-  mkdir vad/vsp/dbpedia/images
-  mkdir vad/vsp/dbpedia/style
+  mkdir vad/vsp/dbpedia/statics/css
+  mkdir vad/vsp/dbpedia/statics/fonts
+  mkdir vad/vsp/dbpedia/statics/images
+  mkdir vad/vsp/dbpedia/statics/js
 
   cd vsp > /dev/null 2>&1
   for f in `find . -type f | grep -v "/CVS/" | grep -v "\.sql" `
@@ -249,7 +251,7 @@ sticker_init() {
   echo "  <name package=\"$VAD_NAME\">" >> $STICKER
   echo "    <prop name=\"Title\" value=\"$VAD_DESC\"/>" >> $STICKER
   echo "    <prop name=\"Developer\" value=\"OpenLink Software\"/>" >> $STICKER
-  echo "    <prop name=\"Copyright\" value=\"(C) 1998-2014 OpenLink Software\"/>" >> $STICKER
+  echo "    <prop name=\"Copyright\" value=\"(C) 1998-2016 OpenLink Software\"/>" >> $STICKER
   echo "    <prop name=\"Download\" value=\"http://www.openlinksw.com/virtuoso\"/>" >> $STICKER
   echo "    <prop name=\"Download\" value=\"http://www.openlinksw.co.uk/virtuoso\"/>" >> $STICKER
   echo "  </name>" >> $STICKER
