@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *  
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *  
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -291,16 +291,7 @@ public class JDBCDemo extends Frame  implements Runnable
     Info.remove(label1);
     Info.remove(textField1);
     urlList = URL;
-    ////BEGIN_NOT_WEBJDBC
-    if (jdkVersion.equalsIgnoreCase("1.0") ||
-        jdkVersion.equalsIgnoreCase("1.1"))
-      loadDriver("virtuoso.jdbc.Driver");
-    else if (jdkVersion.equalsIgnoreCase("1.2") ||
-             jdkVersion.equalsIgnoreCase("1.3"))
-      loadDriver("virtuoso.jdbc2.Driver");
-    else
-      loadDriver("virtuoso.jdbc3.Driver");
-    ////END_NOT_WEBJDBC
+    loadDriver("virtuoso.jdbc2.Driver");
   }
 
   public void show()
