@@ -4,7 +4,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -66,10 +66,12 @@ public class VirtuosoRepository implements Repository {
 
 	private DataSource _ds;
 	private ConnectionPoolDataSource _pds;
+	private XADataSource _xads;
     
-	boolean useLazyAdd = false;
+	boolean useLazyAdd = true;
+	boolean insertBNodeAsVirtuosoIRI = false;
 	String defGraph;
-	int prefetchSize = 200;
+	int prefetchSize = 100;
 	int batchSize = 5000;
 	int queryTimeout = 0;
 	String ruleSet;
@@ -88,6 +90,13 @@ public class VirtuosoRepository implements Repository {
 		this._ds = ds;
 	}
 
+	public VirtuosoRepository(XADataSource ds, String defGraph) {
+	        super();
+		this.defGraph = defGraph;
+		this.useLazyAdd = false;
+		this._xads = ds;
+	}
+
 	/**
 	 * Construct a VirtuosoRepository with a specified parameters
 	 * 
@@ -113,6 +122,7 @@ public class VirtuosoRepository implements Repository {
 	 *        </pre>
          *        methods, when autoCommit mode is off. The triples will be sent to DBMS on commit call
          *        or when batch size become more than predefined batch max_size. 
+         *        LazyAdd will be set false for XADataSource connection
          *
 	 */
 	public VirtuosoRepository(String url_hostlist, String user, String password, String defGraph, boolean useLazyAdd) {
@@ -147,6 +157,7 @@ public class VirtuosoRepository implements Repository {
 	 *        </pre>
          *        methods, when autoCommit mode is off. The triples will be sent to DBMS on commit call
          *        or when batch size become more than predefined batch max_size. 
+         *        LazyAdd will be set false for XADataSource connection
          *
 	 */
 	public VirtuosoRepository(String url_hostlist, String user, String password, boolean useLazyAdd) {
@@ -174,7 +185,7 @@ public class VirtuosoRepository implements Repository {
          *
 	 */
 	public VirtuosoRepository(String url_hostlist, String user, String password, String defGraph) {
-	        this(url_hostlist, user, password, defGraph, false);
+	        this(url_hostlist, user, password, defGraph, true);
 	}
 
 	/**
@@ -195,7 +206,7 @@ public class VirtuosoRepository implements Repository {
 	 *        the user's password
 	 */
 	public VirtuosoRepository(String url_hostlist, String user, String password) {
-	        this(url_hostlist, user, password, false);
+	        this(url_hostlist, user, password, true);
 	}
 
 	/**
@@ -219,7 +230,19 @@ public class VirtuosoRepository implements Repository {
 	 *         If something went wrong during the creation of the Connection.
 	 */
 	public RepositoryConnection getConnection() throws RepositoryException {
-	        if (_pds != null) {
+	        if (_xads != null) {
+	           try {
+		     javax.sql.XAConnection xconn = _xads.getXAConnection();
+		     java.sql.Connection connection = xconn.getConnection();
+		     this.useLazyAdd = false;
+		     return new VirtuosoRepositoryConnection(this, connection);
+		   }
+		   catch (Exception e) {
+		     System.out.println("Connection to " + url_hostlist + " is FAILED.");
+		     throw new RepositoryException(e);
+		   }
+	        }
+	        else if (_pds != null) {
 	           try {
 		     javax.sql.PooledConnection pconn = _pds.getPooledConnection();
 		     java.sql.Connection connection = pconn.getConnection();
@@ -258,6 +281,13 @@ public class VirtuosoRepository implements Repository {
 	     					url = url + "roundrobin=1";
 				}
 		
+				if (url.toLowerCase().indexOf("log_enable=") == -1) {
+	   				if (url.charAt(url.length()-1) != '/')
+	     					url = url + "/log_enable=2";
+	   				else
+	     					url = url + "log_enable=2";
+				}
+		
 				java.sql.Connection connection = DriverManager.getConnection(url, user, password);
 				return new VirtuosoRepositoryConnection(this, connection);
 			}
@@ -285,7 +315,7 @@ public class VirtuosoRepository implements Repository {
 	}
 
 	/**
-	 * Set the buffer fetch size(default 200) 
+	 * Set the buffer fetch size(default 100) 
 	 * 
 	 * @param sz
 	 *        buffer fetch size.
@@ -335,11 +365,29 @@ public class VirtuosoRepository implements Repository {
 		return this.queryTimeout;
 	}
 
+	/**
+	 * Set the UseLazyAdd state for connection(default true) 
+	 * for XADataSource connection set false and can't be changed
+	 * @param v
+	 *        true - useLazyAdd
+	 */
+	public void setUseLazyAdd(boolean v) {
+		this.useLazyAdd = v;
+	}
+
+	/**
+	 * Get the UseLazyAdd state for connection
+	 */
+	public boolean getUseLazyAdd() {
+		return this.useLazyAdd;
+	}
+
+
 	/**
 	 * Set the RoundRobin state for connection(default false) 
 	 * 
-	 * @param sz
-	 *        buffer fetch size.
+	 * @param v
+	 *        true - use roundrobin
 	 */
 	public void setRoundrobin(boolean v) {
 		this.roundrobin = v;
@@ -352,6 +400,26 @@ public class VirtuosoRepository implements Repository {
 		return this.roundrobin;
 	}
 
+
+	/**
+	 * Set the insertBNodeAsURI state for connection(default false) 
+	 * 
+	 * @param v
+	 *        true - insert BNode as Virtuoso IRI
+	 *        false - insert BNode as Virtuoso Native BNode
+	 */
+	public void setInsertBNodeAsVirtuosoIRI(boolean v) {
+		this.insertBNodeAsVirtuosoIRI = v;
+	}
+
+	/**
+	 * Get the insertBNodeAsURI state for connection
+	 */
+	public boolean getInsertBNodeAsVirtuosoIRI() {
+		return this.insertBNodeAsVirtuosoIRI;
+	}
+
+
 	
 	/**
 	 * Set inference RuleSet name
