@@ -5,7 +5,7 @@
 #  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 #  project.
 #
-#  Copyright (C) 1998-2014 OpenLink Software
+#  Copyright (C) 1998-2016 OpenLink Software
 #
 #  This project is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -642,7 +642,7 @@ waitAll ()
    while [ "$clients" -gt "0" ]
      do
        sleep 1
-       clients=`ps -e | grep urlsimu | grep -v grep | wc -l`
+       clients=`ps -e | grep urlsimu | grep -v .deps/ | grep -v grep | wc -l`
 #     echo -e "Running clients $clients\r" 
      done 
 }
@@ -1000,23 +1000,6 @@ then
       LOG "***ABORTED: twsrm.sql"
       exit 1
    fi
-   if [ -f $HOME/vad/ods_framework_dav.vad -a -f $HOME/vad/ods_blog_dav.vad -a -f $HOME/vad/val_dav.vad ]
-   then
-       cp $HOME/vad/ods_framework_dav.vad http
-       cp $HOME/vad/ods_blog_dav.vad http
-       cp $HOME/vad/val_dav.vad http
-       DoCommand $DSN "registry_set ('__blog_api_tests__', '1');" 
-       DoCommand $DSN "checkpoint;" 
-       DoCommand $DSN "VAD_INSTALL ('val_dav.vad', 0);" 
-       DoCommand $DSN "VAD_INSTALL ('ods_framework_dav.vad', 0);" 
-       DoCommand $DSN "VAD_INSTALL ('ods_blog_dav.vad', 0);" 
-       RUN $ISQL $DSN PROMPT=OFF VERBOSE=OFF ERRORS=STDOUT -u "HTTPPORT=$HTTPPORT"< $VIRTUOSO_TEST/tblog.sql 
-       if test $STATUS -ne 0
-       then
-	   LOG "***ABORTED: tblog.sql"
-	   exit 1
-       fi
-   fi
 
 
 if [ -f $PLUGINDIR/wbxml2.so -a -f $HOME/vad/syncml_dav.vad ]
