@@ -4,26 +4,26 @@
 --  $Id: tjoin.sql,v 1.24.6.2.4.4 2013/01/02 16:15:12 source Exp $
 --
 --  Outer Join tests
---  
+--
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
---  
---  Copyright (C) 1998-2014 OpenLink Software
---  
+--
+--  Copyright (C) 1998-2016 OpenLink Software
+--
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
 --  Free Software Foundation; only version 2 of the License, dated June 1991.
---  
+--
 --  This program is distributed in the hope that it will be useful, but
 --  WITHOUT ANY WARRANTY; without even the implied warranty of
 --  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 --  General Public License for more details.
---  
+--
 --  You should have received a copy of the GNU General Public License along
 --  with this program; if not, write to the Free Software Foundation, Inc.,
 --  51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
---  
---  
+--
+--
 
 update t1 set fi2 = 1111;
 
@@ -203,6 +203,10 @@ select top 10 row_no + 100, * from t1_fr;
 select top 10 row_no + 100, * from t1_fr order by string2;
 
 
+select  string2, count (*) from t1_fr group by  string2;
+
+select  string2, count (*) from t1_fr group by  string2 order by string2;
+
 select fi2, avg (row_no), min (row_no), count (*), count (row_no + 1) from t1_fr group by fi2;
 
 update t1 set fi2 = row_no / 3;
@@ -423,7 +427,7 @@ SELECT Product.ID AS ProdID, Price.ID AS PriceID, Catalog.ID AS CatID
 ECHO BOTH $IF $EQU $ROWCNT 5 "PASSED" "***FAILED";
 SET ARGV[$LIF] $+ $ARGV[$LIF] 1;
 ECHO BOTH ": B1166: Q4 returned " $ROWCNT " rows\n";
--- XXX 
+-- XXX
 -- ECHO BOTH $IF $EQU $LAST[2] prod5 "PASSED" "***FAILED";
 -- SET ARGV[$LIF] $+ $ARGV[$LIF] 1;
 -- ECHO BOTH ": B1166: Q4 col2=" $LAST[2] "\n";
@@ -437,7 +441,7 @@ SELECT DISTINCT Product.ID AS ProdID, Price.ID AS PriceID, Catalog.ID AS CatProd
 ECHO BOTH $IF $EQU $ROWCNT 5 "PASSED" "***FAILED";
 SET ARGV[$LIF] $+ $ARGV[$LIF] 1;
 ECHO BOTH ": B1166: Q5 returned " $ROWCNT " rows\n";
--- XXX 
+-- XXX
 -- ECHO BOTH $IF $EQU $LAST[2] prod5 "PASSED" "***FAILED";
 -- SET ARGV[$LIF] $+ $ARGV[$LIF] 1;
 -- ECHO BOTH ": B1166: Q5 col2=" $LAST[2] "\n";
