@@ -8,7 +8,7 @@
 #  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 #  project.
 #  
-#  Copyright (C) 1998-2014 OpenLink Software
+#  Copyright (C) 1998-2016 OpenLink Software
 #  
 #  This project is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -183,9 +183,8 @@ then
     exit 1
 fi
 
-# XXX
-#LOG + running sql script tplmodule
-#RUN $ISQL $DSN PROMPT=OFF VERBOSE=OFF ERRORS=STDOUT < $VIRTUOSO_TEST/tplmodule.sql
+LOG + running sql script tplmodule
+RUN $ISQL $DSN PROMPT=OFF VERBOSE=OFF ERRORS=STDOUT < $VIRTUOSO_TEST/tplmodule.sql
 if test $STATUS -ne 0
 then
     LOG "***ABORTED: PL modules tests -- tplmodule.sql"
@@ -219,9 +218,8 @@ then
     exit 1
 fi
 
-# XXX
-#LOG + running sql script trdfinf
-#RUN $ISQL $DSN PROMPT=OFF VERBOSE=OFF ERRORS=STDOUT < $VIRTUOSO_TEST/trdfinf.sql 
+LOG + running sql script trdfinf
+RUN $ISQL $DSN PROMPT=OFF VERBOSE=OFF ERRORS=STDOUT < $VIRTUOSO_TEST/trdfinf.sql 
 if test $STATUS -ne 0
 then
     LOG "***ABORTED: rdf inference -- trdfinf.sql"
@@ -229,18 +227,16 @@ then
 fi
 
 
-# XXX
-#LOG + running sql script trdfinfifp
-#RUN $ISQL $DSN PROMPT=OFF VERBOSE=OFF ERRORS=STDOUT < $VIRTUOSO_TEST/trdfinfifp.sql 
+LOG + running sql script trdfinfifp
+RUN $ISQL $DSN PROMPT=OFF VERBOSE=OFF ERRORS=STDOUT < $VIRTUOSO_TEST/trdfinfifp.sql 
 if test $STATUS -ne 0
 then
     LOG "***ABORTED: rdf inference -- trdfinfifp.sql"
     exit 1
 fi
 
-# XXX
-#LOG + running sql script ttrans2
-#RUN $ISQL $DSN PROMPT=OFF VERBOSE=OFF ERRORS=STDOUT < $VIRTUOSO_TEST/ttrans2.sql 
+LOG + running sql script ttrans2
+RUN $ISQL $DSN PROMPT=OFF VERBOSE=OFF ERRORS=STDOUT < $VIRTUOSO_TEST/ttrans2.sql 
 if test $STATUS -ne 0
 then
     LOG "***ABORTED: rdf inference -- ttrans2.sql"
@@ -277,6 +273,22 @@ then
     fi
 fi
 
+    LOG + running sql script tveccli
+    RUN $ISQL $DSN PROMPT=OFF VERBOSE=OFF ERRORS=STDOUT -u follow_std=0 < $VIRTUOSO_TEST/tveccli.sql
+    if test $STATUS -ne 0
+    then
+	LOG "***ABORTED:tveccli "
+	exit 1
+    fi
+
+cp -R $VIRTUOSO_TEST/tsparup/* .
+RUN $ISQL $DSN PROMPT=OFF VERBOSE=OFF ERRORS=STDOUT -u follow_std=0 < $VIRTUOSO_TEST/tsparup/tsparup_run.sql
+if test $STATUS -ne 0
+then
+    LOG "***ABORTED:tsparup "
+    exit 1
+fi
+
 SHUTDOWN_SERVER
 
 if [ "x$SQLOPTIMIZE" = "x" ]
