@@ -4,7 +4,7 @@
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --  
---  Copyright (C) 1998-2014 OpenLink Software
+--  Copyright (C) 1998-2016 OpenLink Software
 --  
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -230,7 +230,7 @@ create procedure TUT_generate_files(
 	  if (trim(gen_path,'/') <> '')
 	    _xsl_params := vector_concat(_xsl_params,vector('path',ltrim(gen_path,'/')));
 	    
-	  _xsl_params := vector_concat(_xsl_params,vector('now_rfc1123',date_rfc1123(now())));
+	  _xsl_params := vector_concat(_xsl_params,vector('now_rfc1123',date_rfc1123(curutcdatetime())));
 
 	  _stream := string_output();
 	  http_rewrite(_stream);
