@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -44,12 +44,20 @@
 #define NEW_VAR(type,var) \
 	type *var = (type *) dk_alloc (sizeof (type))
 
+#define B_NEW_VAR(type,var) \
+	type *var = (type *) tlsf_base_alloc (sizeof (type))
+
 #define DBG_NEW_VAR(file, line, type,var) \
 	type *var = (type *) dbg_malloc (file, line, sizeof (type))
 
 #define NEW_VARZ(type, var) \
 	NEW_VAR(type,var); \
-	memset (var, 0, sizeof (type))
+	memzero (var, sizeof (type))
+
+
+#define B_NEW_VARZ(type, var) \
+	B_NEW_VAR(type,var); \
+	memzero (var, sizeof (type))
 
 #define NEW_BOX_VAR(type,var) \
 	type *var = (type *) dk_alloc_box (sizeof (type), DV_BIN)
@@ -74,11 +82,15 @@ void malloc_cache_clear (void);
 
 #ifdef MALLOC_DEBUG
 # include <util/dbgmal.h>
+
+
+
 #ifndef _USRDLL
 #ifndef EXPORT_GATE
 # define dk_alloc(sz)		dbg_malloc (__FILE__, __LINE__, (sz))
 # define dk_try_alloc(sz)	dbg_malloc (__FILE__, __LINE__, (sz))
 # define dk_free(ptr, sz)	dbg_free_sized (__FILE__, __LINE__, (ptr), (sz))
+
 #endif
 #endif
 void dk_alloc_assert (void *ptr);
@@ -87,13 +99,16 @@ void dk_alloc_assert (void *ptr);
 #endif
 
 #ifdef MALLOC_DEBUG
+
 #define DBG_NAME(nm) 		dbg_##nm
 #define DBG_PARAMS 		const char *file, int line,
 #define DBG_PARAMS_0 		const char *file, int line
 #define DBG_ARGS 		file, line,
 #define DBG_ARGS_0 		file, line
+
 #define DK_ALLOC(SIZE) 		dbg_malloc(DBG_ARGS (SIZE))
 #define DK_FREE(BOX,SIZE) 	dbg_free_sized(DBG_ARGS (BOX), (SIZE))
+
 #else
 #define DBG_NAME(nm) 		nm
 #define DBG_PARAMS
@@ -109,4 +124,16 @@ void *dbg_dk_alloc (DBG_PARAMS size_t c);
 void *dbg_dk_try_alloc (DBG_PARAMS size_t c);
 #endif
 
+void dk_set_initial_mem (size_t);
+
+#define tlsf_base_alloc dk_alloc
+
+
+#define WITH_TLSF(n) {
+
+
+#define END_WITH_TLSF }
+
+
+
 #endif
