@@ -18,7 +18,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -168,6 +168,30 @@ dk_mmap_brk (size_t sz)
   ((sz) >= box_min_mmap && (sz) < 0xffffff ? mm_free_sized (ptr, sz) : dk_free (ptr, sz))
 #endif
 
+#ifndef NDEBUG
+#define dk_alloc_box_check_length(bytes,tag) do { \
+  if ((bytes) & ~0xffffff) \
+    GPF_T1 ("box to allocate is too large"); \
+  switch ((tag)) { \
+    case DV_SHORT_STRING_SERIAL: case DV_LONG_WIDE: \
+      GPF_T1 ("misused tag of box to allocate"); \
+      break; \
+    case DV_WIDE: \
+      if ((bytes) % sizeof (wchar_t)) \
+        GPF_T1 ("DV_WIDE of length that is not divisible on sizeof (wchar_t)"); \
+      if (0 == (bytes)) \
+        GPF_T1 ("DV_WIDE with no room for trailing zero"); \
+      break; \
+    case DV_STRING: case DV_C_STRING: \
+      if (0 == (bytes)) \
+        GPF_T1 ("String with no room for trailing zero"); \
+      break; \
+    } \
+} while (0)
+#else
+#define dk_alloc_box_check_length(bytes,tag) do { ; } while (0)
+#endif
+
 
 size_t box_min_mmap = (1024 * 100 ) - 8;
 #undef dk_alloc_box
@@ -176,11 +200,7 @@ dk_alloc_box (size_t bytes, dtp_t tag)
 {
   unsigned char *ptr;
   size_t align_bytes;
-#ifdef MALLOC_DEBUG
-  if (bytes & ~0xffffff)
-    GPF_T1 ("box to allocate is too large");
-#endif
-
+  dk_alloc_box_check_length (bytes, tag);
   /* This assumes dk_alloc aligns at least at 4 */
 #ifdef DOUBLE_ALIGN
   align_bytes = 8 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_8 (bytes));
@@ -248,11 +268,7 @@ dk_try_alloc_box (size_t bytes, dtp_t tag)
   unsigned char *ptr = NULL;
   size_t align_bytes;
 
-#ifdef MALLOC_DEBUG
-  if (bytes & ~0xffffff)
-    GPF_T1 ("box to allocate is too large");
-#endif
-
+  dk_alloc_box_check_length (bytes, tag);
   /* This assumes dk_alloc aligns at least at 4 */
 #ifdef DOUBLE_ALIGN
   align_bytes = 8 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_8 (bytes));
@@ -261,7 +277,7 @@ dk_try_alloc_box (size_t bytes, dtp_t tag)
 #endif
 
   if (align_bytes >= box_min_mmap)
-    ptr = dk_alloc_mmap (align_bytes);
+    ptr = (unsigned char *) dk_alloc_mmap (align_bytes);
   else
   ptr = (unsigned char *) dk_try_alloc (align_bytes);
   if (!ptr)
@@ -287,7 +303,7 @@ dk_alloc_box_zero (size_t bytes, dtp_t tag)
 {
   unsigned char *ptr;
   size_t align_bytes;
-
+  dk_alloc_box_check_length (bytes, tag);
   /* This assumes dk_alloc aligns at least at 4 */
 #ifdef DOUBLE_ALIGN
   align_bytes = 8 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_8 (bytes));
@@ -320,7 +336,7 @@ dbg_dk_alloc_box (DBG_PARAMS size_t bytes, dtp_t tag)
 {
   unsigned char *ptr;
   uint32 align_bytes;
-
+  dk_alloc_box_check_length (bytes, tag);
   /* This assumes dk_alloc aligns at least at 4 */
 #ifdef DOUBLE_ALIGN
   align_bytes = 8 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_8 (bytes));
@@ -328,7 +344,7 @@ dbg_dk_alloc_box (DBG_PARAMS size_t bytes, dtp_t tag)
   align_bytes = 4 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_4 (bytes));
 #endif
 
-  ptr = (unsigned char *) dbg_malloc (DBG_ARGS align_bytes);
+  ptr = (unsigned char *) dbg_malloc (file, line, align_bytes);
   if (!ptr)
     return (box_t) ptr;
 
@@ -352,7 +368,7 @@ dbg_dk_alloc_box_long (DBG_PARAMS size_t bytes, dtp_t tag)
 {
   unsigned char *ptr;
   uint32 align_bytes;
-
+  dk_alloc_box_check_length (bytes, tag);
   /* This assumes dk_alloc aligns at least at 4 */
 #ifdef DOUBLE_ALIGN
   align_bytes = 8 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_8 (bytes));
@@ -360,7 +376,7 @@ dbg_dk_alloc_box_long (DBG_PARAMS size_t bytes, dtp_t tag)
   align_bytes = 4 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_4 (bytes));
 #endif
 
-  ptr = (unsigned char *) dbg_malloc (DBG_ARGS align_bytes);
+  ptr = (unsigned char *) dbg_malloc (file, line, align_bytes);
   if (!ptr)
     return (box_t) ptr;
 
@@ -386,7 +402,7 @@ dbg_dk_try_alloc_box (DBG_PARAMS size_t bytes, dtp_t tag)
 {
   unsigned char *ptr;
   uint32 align_bytes;
-
+  dk_alloc_box_check_length (bytes, tag);
   /* This assumes dk_alloc aligns at least at 4 */
 #ifdef DOUBLE_ALIGN
   align_bytes = 8 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_8 (bytes));
@@ -394,7 +410,7 @@ dbg_dk_try_alloc_box (DBG_PARAMS size_t bytes, dtp_t tag)
   align_bytes = 4 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_4 (bytes));
 #endif
 
-  ptr = (unsigned char *) dbg_malloc (DBG_ARGS align_bytes);
+  ptr = (unsigned char *) dbg_malloc (file, line, align_bytes);
   if (!ptr)
     return (box_t) ptr;
 
@@ -418,7 +434,7 @@ dbg_dk_alloc_box_zero (DBG_PARAMS size_t bytes, dtp_t tag)
 {
   unsigned char *ptr;
   uint32 align_bytes;
-
+  dk_alloc_box_check_length (bytes, tag);
   /* This assumes dk_alloc aligns at least at 4 */
 #ifdef DOUBLE_ALIGN
   align_bytes = 8 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_8 (bytes));
@@ -426,7 +442,7 @@ dbg_dk_alloc_box_zero (DBG_PARAMS size_t bytes, dtp_t tag)
   align_bytes = 4 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_4 (bytes));
 #endif
 
-  ptr = (unsigned char *) dbg_malloc (DBG_ARGS align_bytes);
+  ptr = (unsigned char *) dbg_malloc (file, line, align_bytes);
   if (!ptr)
     return (box_t) ptr;
 
@@ -514,21 +530,22 @@ dk_free_box (box_t box)
 
   switch (tag)
     {
-#ifdef MALLOC_DEBUG
     case DV_WIDE:
+#ifndef NDEBUG
       if ((len % sizeof (wchar_t)) || (0 != ((wchar_t *)box)[len/sizeof (wchar_t) - 1]))
         GPF_T1 ("Free of a damaged wide string");
+#endif
 #ifdef DOUBLE_ALIGN
       len = ALIGN_8 (len);
 #else
       len = ALIGN_4 (len);
 #endif
       break;
-#endif
     case DV_STRING:
     case DV_C_STRING:
     case DV_SHORT_STRING_SERIAL:
     case DV_SYMBOL:
+    case DV_BIN:
       len = ALIGN_STR (len);
       break;
 
@@ -642,7 +659,7 @@ dk_free_box (box_t box)
 #ifdef MALLOC_DEBUG
   if (len >= 0xffffff)
     {
-      dbg_free (__FILE__, __LINE__, ptr - 8);
+      dbg_free_sized (__FILE__, __LINE__, ptr - 8, len);
       return 0;
     }
 #endif
@@ -705,21 +722,22 @@ dk_free_tree (box_t box)
 
   switch (tag)
     {
-#ifdef MALLOC_DEBUG
     case DV_WIDE:
+#ifndef NDEBUG
       if ((len % sizeof (wchar_t)) || (0 != ((wchar_t *)box)[len/sizeof (wchar_t) - 1]))
         GPF_T1 ("Free of a tree with a damaged wide string");
+#endif
 #ifdef DOUBLE_ALIGN
       len = ALIGN_8 (len);
 #else
       len = ALIGN_4 (len);
 #endif
       break;
-#endif
     case DV_STRING:
     case DV_C_STRING:
     case DV_SHORT_STRING_SERIAL:
     case DV_SYMBOL:
+    case DV_BIN:
       len = ALIGN_STR (len);
       break;
 
@@ -798,6 +816,18 @@ void
 box_reuse (caddr_t box, ccaddr_t data, size_t len, dtp_t dtp)
 {
   dk_alloc_box_assert (box);
+#ifndef NDEBUG
+  if (DV_WIDE == DV_TYPE_OF (box))
+    {
+      if ((len % sizeof (wchar_t)) || (0 != ((wchar_t *)box)[len/sizeof (wchar_t) - 1]))
+        GPF_T1 ("Reuse of a damaged wide string");
+    }
+  if (DV_WIDE == dtp)
+    {
+      if ((len % sizeof (wchar_t)) || (0 != ((wchar_t *)data)[len/sizeof (wchar_t) - 1]))
+        GPF_T1 ("Reuse of a box for a damaged wide string");
+    }
+#endif
   box_tag_modify (box, dtp);
   ((dtp_t *) box)[-4] = (dtp_t) (len & 0xff);
   ((dtp_t *) box)[-3] = (dtp_t) (len >> 8);
@@ -807,7 +837,6 @@ box_reuse (caddr_t box, ccaddr_t data, size_t len, dtp_t dtp)
   memcpy (box, data, len);
 }
 
-
 #ifdef DK_ALLOC_BOX_DEBUG
 void
 dk_check_tree_iter (box_t box, box_t parent, dk_hash_t * known)
@@ -824,7 +853,7 @@ dk_check_tree_iter (box_t box, box_t parent, dk_hash_t * known)
     GPF_T1 ("Tree contains a pointer to a freed box");
   if (TAG_BAD == tag)
     GPF_T1 ("Tree contains a pointer to a box marked bad");
-  if (!box_can_appear_twice_in_tree[tag])
+  if (known && !box_can_appear_twice_in_tree[tag])
     {
       box_t other_parent = gethash (box, known);
       if (NULL != other_parent)
@@ -834,8 +863,9 @@ dk_check_tree_iter (box_t box, box_t parent, dk_hash_t * known)
   if (IS_NONLEAF_DTP (tag))
     {
       box_t *obj = (box_t *) box;
-      for (count = box_length (box) / sizeof (box_t); count; count--)
-	dk_check_tree_iter (*obj++, box, known);
+      int len = box_length (box) / sizeof (box_t);
+      for (count = 0; count < len; count++)
+	dk_check_tree_iter (obj[count], box, known);
     }
   return;
 }
@@ -1117,12 +1147,12 @@ DBG_NAME (box_copy) (DBG_PARAMS cbox_t box)
   switch (tag)
     {
     case DV_WIDE:
-#ifdef MALLOC_DEBUG
+#ifndef NDEBUG
       len = box_length (box);
       if ((len % sizeof (wchar_t)) || (0 != ((wchar_t *)box)[len/sizeof (wchar_t) - 1]))
         GPF_T1 ("Copy of a damaged wide string");
-      break;
 #endif
+      break;
     case DV_STRING:
     case DV_ARRAY_OF_POINTER:
     case DV_LIST_OF_POINTER:
@@ -1248,13 +1278,13 @@ box_t DBG_NAME (box_copy_tree) (DBG_PARAMS cbox_t box)
   tag = box_tag (box);
   switch (tag)
     {
-#ifdef MALLOC_DEBUG
     case DV_WIDE:
+#ifndef NDEBUG
       len = box_length (box);
       if ((len % sizeof (wchar_t)) || (0 != ((wchar_t *)box)[len/sizeof (wchar_t) - 1]))
         GPF_T1 ("Copy of a tree with a damaged wide string");
-      break;
 #endif
+      break;
     case DV_ARRAY_OF_POINTER:
     case DV_LIST_OF_POINTER:
     case DV_ARRAY_OF_XQVAL:
@@ -1343,6 +1373,13 @@ DBG_NAME (box_try_copy_tree) (DBG_PARAMS box_t box, box_t stub)
   tag = box_tag (box);
   switch (tag)
     {
+    case DV_WIDE:
+#ifndef NDEBUG
+      len = box_length (box);
+      if ((len % sizeof (wchar_t)) || (0 != ((wchar_t *)box)[len/sizeof (wchar_t) - 1]))
+        GPF_T1 ("Copy of a tree with a damaged wide string");
+#endif
+      break;
     case DV_ARRAY_OF_POINTER:
     case DV_LIST_OF_POINTER:
     case DV_ARRAY_OF_XQVAL:
@@ -1476,7 +1513,8 @@ rdf_box_audit_impl (rdf_box_t * rb)
   if ((0 == rb->rb_ro_id) && (0 == rb->rb_is_complete))
     GPF_T1 ("RDF box is too incomplete");
 #endif
-  if (rb->rb_type < RDF_BOX_MIN_TYPE) GPF_T1 ("rb type out pof range");
+  if ((rb->rb_type < RDF_BOX_MIN_TYPE) || (rb->rb_type > RDF_BOX_MAX_TYPE)) GPF_T1 ("rb type out of range");
+  if ((rb->rb_lang < RDF_BOX_DEFAULT_LANG) || (rb->rb_lang > RDF_BOX_MAX_LANG)) GPF_T1 ("rb lang out of range");
   if (rb->rb_is_complete)
     rb_dt_lang_check(rb);
 }
@@ -1657,7 +1695,7 @@ dtp_set_strong_cmp (dtp_t dtp, box_hash_cmp_func_t f)
 int
 box_equal (cbox_t b1, cbox_t b2)
 {
-  uint32 l1, l2;
+  uint32 l1, l2, bf1, bf2;
   dtp_t b1_tag, b2_tag;
   boxint b1_long_val = 0, b2_long_val = 0;
 
@@ -1714,6 +1752,23 @@ box_equal (cbox_t b1, cbox_t b2)
       return 1;
     }
   memcmp_8 (b1, b2, l1, neq);
+  bf1 = box_flags (b1);
+  bf2 = box_flags (b2);
+  if (bf1 != bf2)
+    {
+      if (DV_UNAME == b1_tag)
+        {
+          b1_tag = DV_STRING;
+          bf1 = BF_IRI;
+        }
+      if (DV_UNAME == b2_tag)
+        {
+          b2_tag = DV_STRING;
+          bf2 = BF_IRI;
+        }
+      if ((b1_tag != b2_tag) || (bf1 != bf2))
+        return 0;
+    }
   return 1;
  neq:
   return 0;
@@ -1724,7 +1779,7 @@ box_equal (cbox_t b1, cbox_t b2)
 int
 box_strong_equal (cbox_t b1, cbox_t b2)
 {
-  uint32 l1, l2;
+  uint32 l1, l2, bf1, bf2;
   dtp_t b1_tag, b2_tag;
   boxint b1_long_val = 0, b2_long_val = 0;
 
@@ -1783,6 +1838,24 @@ box_strong_equal (cbox_t b1, cbox_t b2)
       return 1;
     }
   memcmp_8 (b1, b2, l1, neq);
+  bf1 = box_flags (b1);
+  bf2 = box_flags (b2);
+  if (bf1 != bf2)
+    {
+      if (DV_UNAME == b1_tag)
+        {
+          b1_tag = DV_STRING;
+          bf1 = BF_IRI;
+        }
+      if (DV_UNAME == b2_tag)
+        {
+          b2_tag = DV_STRING;
+          bf2 = BF_IRI;
+        }
+      if ((b1_tag != b2_tag) || (bf1 != bf2))
+        return 0;
+    }
+  return 1;
   return 1;
  neq:
   return 0;
@@ -2437,7 +2510,7 @@ dk_box_initialize (void)
 #ifdef MALLOC_DEBUG
   dk_mem_hooks_2 (DV_NON_BOX, box_copy_non_box, NULL, 0, box_mp_copy_non_box);
 #endif
-  dk_mem_hooks (DV_RBUF,  box_non_copiable, rbuf_free_cb, 0);
+  dk_mem_hooks (DV_RBUF,  box_non_copiable, (box_destr_f)rbuf_free_cb, 0);
   uname_mutex = mutex_allocate ();
   if (NULL == uname_mutex)
     GPF_T;
