@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -294,9 +294,6 @@ boxint_hashcmp (char *x, char *y)
   return k1 == k2 ? 1 : 0;
 }
 
-
-#define ROL(h) ((h << 1) | ((h >> 31) & 1))
-
 box_hash_func_t dtp_hash_func[256];
 
 extern id_hashed_key_t rdf_box_hash (caddr_t box);
@@ -457,7 +454,7 @@ DBG_NAME (box_dv_dict_iterator) (DBG_PARAMS caddr_t ht_box)
   id_hash_t *ht = (id_hash_t *) ht_box;
   res->hit_hash = ht;
   res->hit_bucket = -1;
-  res->hit_chilum = (void *)(-1);
+  res->hit_chilum = (char *)(-1);
   if (NULL != ht)
     {
       if (NULL != ht->ht_mutex)
@@ -539,12 +536,14 @@ box_dict_hashtable_destr_hook (caddr_t dict)
   else
     {
       id_hash_iterator (&hit, (id_hash_t *) (dict));
-      while (hit_next (&hit, (caddr_t *) (&key), (caddr_t *) (&val)))
+      while (!ht->ht_mp && hit_next (&hit, (caddr_t *) (&key), (caddr_t *) (&val)))
 	{
 	  dk_free_tree (key[0]);
 	  dk_free_tree (val[0]);
 	}
     }
+  if (ht->ht_mp)
+    mp_free ((mem_pool_t*)ht->ht_mp);
   id_hash_clear ((id_hash_t *) (dict));
   ID_HASH_FREE_INTERNALS ((id_hash_t *) (dict));
   return 0;
@@ -630,7 +629,7 @@ id_hash_set_rehash_pct (id_hash_t * ht, uint32 pct)
 
 #ifdef MALLOC_DEBUG
 #define DBG_HASHEXT_NAME(name) dbg_t_##name
-#define DBG_HASHEXT_ALLOC(SZ) dbg_mp_alloc_box (DBG_ARGS THR_TMP_POOL, (SZ), DV_CUSTOM)
+#define DBG_HASHEXT_ALLOC(SZ) dbg_mp_alloc_box (DBG_ARGS THR_TMP_POOL, (SZ), DV_NON_BOX)
 #else
 #define DBG_HASHEXT_NAME(name) t_##name
 #define DBG_HASHEXT_ALLOC(SZ) mp_alloc_box_ni (THR_TMP_POOL, (SZ), DV_NON_BOX)
