@@ -6,7 +6,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -46,6 +46,8 @@ typedef uint32 id_hashed_key_t;
 #define ID_HASHED_KEY_CHECK(i)
 #endif
 
+#define ROL(h) ((h << 1) | ((h >> 31) & 1))
+
 typedef id_hashed_key_t (*hash_func_t) (caddr_t p_data);
 typedef int (*cmp_func_t) (caddr_t d1, caddr_t d2);
 typedef int (*cmp_func_with_ctx_t) (caddr_t d1, caddr_t d2, void *ctx);
@@ -72,7 +74,8 @@ struct id_hash_s
     long		ht_deletes;
     long		ht_overflows;		/*!< Number of bucket overflows */
     uint32      	ht_count;
-    int         	ht_rehash_threshold;
+    short         	ht_rehash_threshold;
+    short               ht_tlsf_id;
     int			ht_dict_refctr;		/*!< Number of references to dictionary, if the hastable is used as a box */
     long		ht_dict_version;	/*!< Version of dictionary, to track parallel access */
     size_t 		ht_dict_mem_in_use;	/*!< Approximate size of dictonary in memory. Each stored item adds length of key + lenght of the data + size of hash table entry. That is filled only if \c ht_dict_max_mem_in_use is not zero */
@@ -80,6 +83,7 @@ struct id_hash_s
     size_t 		ht_dict_max_mem_in_use;	/*!< Memory size limit to prevent exausting of the physical memory */
     dk_mutex_t *	ht_mutex;		/*!< Optional mutex, esp. popular when this is a dictionary propagated across threads of async queue. The mutex is NOT owned by the hashtable box! */
     id_hash_free_t	ht_free_hook;
+    void * 		ht_mp;
 #ifdef RH_TRACE
     int64		ht_rem_k;
     char *		ht_rem_file;
