@@ -9,7 +9,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -85,6 +85,7 @@ struct mem_pool_s
 #endif
   mem_pool_size_cap_t	mp_size_cap;
   caddr_t	mp_comment;
+  struct TLSF_struct *	mp_tlsf;
 };
 #else
 struct mem_block_s
@@ -111,6 +112,7 @@ struct mem_pool_s
 #endif
   caddr_t	mp_comment;
   mem_pool_size_cap_t	mp_size_cap;
+  struct TLSF_struct *	mp_tlsf;
 };
 #endif
 
@@ -124,9 +126,11 @@ EXE_EXPORT (caddr_t, mp_alloc_box, (mem_pool_t * mp, size_t len, dtp_t dtp));
 EXE_EXPORT (caddr_t, mp_alloc_box_ni, (mem_pool_t * mp, int len, dtp_t dtp));
 EXE_EXPORT (caddr_t, mp_box_string, (mem_pool_t * mp, const char *str));
 EXE_EXPORT (caddr_t, mp_box_substr, (mem_pool_t * mp, ccaddr_t str, int n1, int n2));
-EXE_EXPORT (box_t, mp_box_dv_short_nchars, (mem_pool_t * mp, const char *str, size_t len));
+EXE_EXPORT (caddr_t, mp_box_dv_short_nchars, (mem_pool_t * mp, const char *str, size_t len));
+EXE_EXPORT (caddr_t, mp_box_dv_short_concat, (mem_pool_t * mp, ccaddr_t str1, ccaddr_t str2));
+EXE_EXPORT (caddr_t, mp_box_dv_short_strconcat, (mem_pool_t * mp, const char *str1, const char *str2));
 EXE_EXPORT (caddr_t, mp_box_dv_uname_string, (mem_pool_t * mp, const char *str));
-EXE_EXPORT (box_t, mp_box_dv_uname_nchars, (mem_pool_t * mp, const char *str, size_t len));
+EXE_EXPORT (caddr_t, mp_box_dv_uname_nchars, (mem_pool_t * mp, const char *str, size_t len));
 EXE_EXPORT (caddr_t, mp_box_copy, (mem_pool_t * mp, caddr_t box));
 EXE_EXPORT (caddr_t, mp_box_copy_tree, (mem_pool_t * mp, caddr_t box));
 EXE_EXPORT (caddr_t, mp_full_box_copy_tree, (mem_pool_t * mp, caddr_t box));
@@ -135,15 +139,18 @@ EXE_EXPORT (caddr_t, mp_box_iri_id, (mem_pool_t * mp, iri_id_t num));
 EXE_EXPORT (caddr_t, mp_box_double, (mem_pool_t * mp, double num));
 EXE_EXPORT (caddr_t, mp_box_float, (mem_pool_t * mp, float num));
 void * mp_large_alloc (mem_pool_t * mp, size_t sz);
+void mp_set_tlsf (mem_pool_t * mp, size_t  sz);
 
 #ifdef MALLOC_DEBUG
 extern caddr_t dbg_mp_alloc_box (const char *file, int line, mem_pool_t * mp, size_t len, dtp_t dtp);
 extern caddr_t dbg_mp_alloc_box_ni (const char *file, int line, mem_pool_t * mp, int len, dtp_t dtp);
 extern caddr_t dbg_mp_box_string (const char *file, int line, mem_pool_t * mp, const char *str);
 extern caddr_t dbg_mp_box_substr (const char *file, int line, mem_pool_t * mp, ccaddr_t str, int n1, int n2);
-extern box_t dbg_mp_box_dv_short_nchars (const char *file, int line, mem_pool_t * mp, const char *str, size_t len);
+extern caddr_t dbg_mp_box_dv_short_nchars (const char *file, int line, mem_pool_t * mp, const char *str, size_t len);
+extern caddr_t dbg_mp_box_dv_short_concat (const char *file, int line, mem_pool_t * mp, ccaddr_t str1, ccaddr_t str2);
+extern caddr_t dbg_mp_box_dv_short_strconcat (const char *file, int line, mem_pool_t * mp, const char *str1, const char *str2);
 extern caddr_t dbg_mp_box_dv_uname_string (const char *file, int line, mem_pool_t * mp, const char *str);
-extern box_t dbg_mp_box_dv_uname_nchars (const char *file, int line, mem_pool_t * mp, const char *str, size_t len);
+extern caddr_t dbg_mp_box_dv_uname_nchars (const char *file, int line, mem_pool_t * mp, const char *str, size_t len);
 extern caddr_t dbg_mp_box_copy (const char *file, int line, mem_pool_t * mp, caddr_t box);
 extern caddr_t dbg_mp_box_copy_tree (const char *file, int line, mem_pool_t * mp, caddr_t box);
 extern caddr_t dbg_mp_full_box_copy_tree (const char *file, int line, mem_pool_t * mp, caddr_t box);
@@ -158,6 +165,8 @@ extern caddr_t dbg_mp_box_float (const char *file, int line, mem_pool_t * mp, fl
 #define mp_box_string(mp, str) dbg_mp_box_string (__FILE__, __LINE__, (mp), (str))
 #define mp_box_substr(mp, str, n1, n2) dbg_mp_box_substr (__FILE__, __LINE__, (mp), (str), (n1), (n2))
 #define mp_box_dv_short_nchars(mp, str, len) dbg_mp_box_dv_short_nchars (__FILE__, __LINE__, (mp), (str), (len))
+#define mp_box_dv_short_concat(mp, str1, str2) dbg_mp_box_dv_short_concat (__FILE__, __LINE__, (mp), (str1), (str2))
+#define mp_box_dv_short_strconcat(mp, str1, str2) dbg_mp_box_dv_short_strconcat (__FILE__, __LINE__, (mp), (str1), (str2))
 #define mp_box_dv_uname_string(mp, str) dbg_mp_box_dv_uname_string (__FILE__, __LINE__, (mp), (str))
 #define mp_box_dv_uname_nchars(mp, str, len) dbg_mp_box_dv_uname_nchars (__FILE__, __LINE__, (mp), (str), (len))
 #define mp_box_copy(mp, box) dbg_mp_box_copy (__FILE__, __LINE__, (mp), (box))
@@ -246,6 +255,8 @@ caddr_t *mp_list (mem_pool_t * mp, long n, ...);
 #define dbg_t_box_string(str)			dbg_mp_box_string (DBG_ARGS THR_TMP_POOL, (str))
 #define dbg_t_box_substr(str,n1,n2)		dbg_mp_box_substr (DBG_ARGS THR_TMP_POOL, (str), (n1), (n2))
 #define dbg_t_box_dv_short_nchars(str,len) 	dbg_mp_box_dv_short_nchars (DBG_ARGS THR_TMP_POOL, (str), (len))
+#define dbg_t_box_dv_short_concat(str1,str2) 	dbg_mp_box_dv_short_concat (DBG_ARGS THR_TMP_POOL, (str1), (str2))
+#define dbg_t_box_dv_short_strconcat(str1,str2)	dbg_mp_box_dv_short_strconcat (DBG_ARGS THR_TMP_POOL, (str1), (str2))
 #define dbg_t_box_dv_uname_string(str)		dbg_mp_box_dv_uname_string (DBG_ARGS THR_TMP_POOL, (str))
 #define dbg_t_box_dv_uname_nchars(str,len) 	dbg_mp_box_dv_uname_nchars (DBG_ARGS THR_TMP_POOL, (str), (len))
 #define dbg_t_box_copy(box)			dbg_mp_box_copy (DBG_ARGS THR_TMP_POOL, (box))
@@ -255,6 +266,8 @@ caddr_t *mp_list (mem_pool_t * mp, long n, ...);
 #define t_box_string(str)			mp_box_string (THR_TMP_POOL, (str))
 #define t_box_substr(str,n1,n2)			mp_box_substr (THR_TMP_POOL, (str), (n1), (n2))
 #define t_box_dv_short_nchars(str,len)		mp_box_dv_short_nchars (THR_TMP_POOL, (str), (len))
+#define t_box_dv_short_concat(str1,str2)	mp_box_dv_short_concat (THR_TMP_POOL, (str1), (str2))
+#define t_box_dv_short_strconcat(str1,str2)	mp_box_dv_short_strconcat (THR_TMP_POOL, (str1), (str2))
 #define t_box_dv_uname_string(str)		mp_box_dv_uname_string (THR_TMP_POOL, (str))
 #define t_box_dv_uname_nchars(str,len)		mp_box_dv_uname_nchars (THR_TMP_POOL, (str), (len))
 #define t_box_copy(box)				mp_box_copy (THR_TMP_POOL, (box))
@@ -262,6 +275,7 @@ caddr_t *mp_list (mem_pool_t * mp, long n, ...);
 #define t_full_box_copy_tree(box)		mp_full_box_copy_tree (THR_TMP_POOL, (box))
 
 #define t_alloc_list(n) 			((caddr_t *)t_alloc_box ((n) * sizeof (caddr_t), DV_ARRAY_OF_POINTER))
+extern caddr_t *t_list_memcpy (long n, ccaddr_t *src);
 extern caddr_t *t_list_concat_tail (caddr_t list, long n, ...);
 extern caddr_t *t_list_concat (caddr_t list1, caddr_t list2);
 extern caddr_t *t_list_remove_nth (caddr_t list, int pos);
@@ -367,7 +381,11 @@ void mp_check_tree (mem_pool_t * mp, box_t box);
 
 #ifdef _DKSYSTEM_H
 caddr_t t_box_vsprintf (size_t buflen_eval, const char *format, va_list tail);
-caddr_t t_box_sprintf (size_t buflen_eval, const char *format, ...);
+caddr_t t_box_sprintf (size_t buflen_eval, const char *format, ...)
+#ifdef __GNUC__
+                __attribute__ ((format (printf, 2, 3)))
+#endif
+;
 #endif
 
 void mp_trash (mem_pool_t * mp, caddr_t box);
@@ -385,11 +403,11 @@ extern box_tmp_copy_f box_tmp_copier[256];
     mem_block_t * f = mp->mp_first; \
     if (f && f->mb_fill + __len <= f->mb_size) \
       { \
-	(x) = (void *)(((char*)f) + f->mb_fill); \
+	*(void**)&(x) = (void *)(((char*)f) + f->mb_fill);	\
 	f->mb_fill += __len; \
       } \
     else \
-      (x) = (void *)mp_alloc_box (mp, len2, DV_NON_BOX); \
+      *(void**)&(x) = (void *)mp_alloc_box (mp, len2, DV_NON_BOX);	\
   }
 #endif
 
@@ -398,7 +416,7 @@ extern box_tmp_copy_f box_tmp_copier[256];
     MP_BYTES (x, mp, 16); \
     x = ((char *)x) + 8; \
     *(int64 *)x = v; \
-    ((int32*)x)[-1] = tag_word; \
+    ((int64*)x)[-1] = tag_word; \
   }
 
 
@@ -407,7 +425,7 @@ extern box_tmp_copy_f box_tmp_copier[256];
     MP_BYTES (x, mp, 16); \
     x = ((char *)x) + 8; \
     *(double *)x = v; \
-    ((int32*)x)[-1] = tag_word; \
+    ((int64*)x)[-1] = tag_word; \
   }
 
 
@@ -416,7 +434,7 @@ extern box_tmp_copy_f box_tmp_copier[256];
     MP_BYTES (x, mp, 16); \
     x = ((char *)x) + 8; \
     *(float *)x = v; \
-    ((int32*)x)[-1] = tag_word; \
+    ((int64*)x)[-1] = tag_word; \
   }
 
 
