@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -468,6 +468,21 @@ dk_set_push_two (s_node_t ** set, void *car_item, void *cadr_item)
   dbg_dk_set_push_two (__FILE__, __LINE__, set, car_item, cadr_item);
 }
 
+#undef dk_set_pushnew
+void
+dk_set_pushnew (s_node_t ** set, void *item)
+{
+  dbg_dk_set_pushnew (__FILE__, __LINE__, set, item);
+}
+
+#undef dk_set_pop
+void *
+dk_set_pop (s_node_t ** set)
+{
+  return dbg_dk_set_pop (__FILE__, __LINE__, set);
+}
+
+
 #undef list_to_array
 caddr_t
 list_to_array (dk_set_t l)
