@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -27,13 +27,15 @@
 
 #include "Dk.h"
 
-#if defined(linux) || defined (__APPLE__)
+#if defined(HAVE_EXECINFO_H)
+
 #include <execinfo.h>
 
+#define N_FRAMES 100
+
 void
 print_trace (void)
 {
-#define N_FRAMES 100
   void *array[N_FRAMES];
   size_t size, i;
   char **strings;
@@ -84,8 +86,10 @@ gpf_notice (const char *file, int line, const char *text)
 #endif
 #endif
   *(long *) -1 = -1;
+
   call_exit (1);
-  return 0;
+
+  exit(1);
 }
 
 
