@@ -2,7 +2,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -40,11 +40,11 @@ public class TestExecuteClob
             url = args[1];
             filename = args[0];
          }
-         Class.forName("virtuoso.jdbc3.Driver");
-         System.out.println("--------------------- Test of the clob features (varchar) ------------");
+         Class.forName("virtuoso.jdbc2.Driver");
+         System.out.println("--------------------- Test of the clob features -------------------");
          System.out.print("Establish connection at " + url);
          Connection connection = DriverManager.getConnection(url,"dba","dba");
-         if(connection instanceof virtuoso.jdbc3.VirtuosoConnection)
+         if(connection instanceof virtuoso.jdbc2.VirtuosoConnection)
             System.out.println("    PASSED");
          else
          {
@@ -53,7 +53,7 @@ public class TestExecuteClob
          }
          System.out.print("Create a Statement class attached to this connection");
          Statement stmt = connection.createStatement();
-         if(stmt instanceof virtuoso.jdbc3.VirtuosoStatement)
+         if(stmt instanceof virtuoso.jdbc2.VirtuosoStatement)
             System.out.println("    PASSED");
          else
          {
@@ -89,32 +89,6 @@ public class TestExecuteClob
          System.out.print("Execute DROP TABLE");
          stmt.executeUpdate("drop table EX..ECLOB");
          System.out.println("    PASSED");
-
-         System.out.println("--------------------- Test of the clob features (nvarchar)-----------");
-         System.out.print("Execute CREATE TABLE");
-         if(stmt.executeUpdate("create table EX..ECLOB (ID integer,FILLER long nvarchar,primary key(ID))") == 0)
-            System.out.println("    PASSED");
-         else
-         {
-            System.out.println("    FAILED");
-            System.exit(-1);
-         }
-         System.out.print("Execute INSERT INTO");
-         fs = new java.io.FileInputStream(filename);
-         pstmt = connection.prepareStatement("insert into EX..ECLOB(id,FILLER) values(?,?)");
-         pstmt.setLong(1,3000);
-         pstmt.setAsciiStream(2,fs,1024 * 20);
-         pstmt.execute();
-         System.out.println("    PASSED");
-         System.out.println("Execute SELECT");
-         rs = stmt.executeQuery("select * from EX..ECLOB");
-         rs.next();
-         clob = rs.getClob(2);
-         System.out.println(new String(clob.getSubString(4060,1024 * 2)));
-         System.out.println("    PASSED");
-         System.out.print("Execute DROP TABLE");
-         stmt.executeUpdate("drop table EX..ECLOB");
-         System.out.println("    PASSED");
          System.out.print("Close statement at " + url);
          pstmt.close();
          stmt.close();
@@ -134,3 +108,4 @@ public class TestExecuteClob
    }
 
 }
+
