@@ -2,7 +2,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -21,7 +21,7 @@
 
 package testsuite;
 
-import virtuoso.jdbc2.*;
+import virtuoso.jdbc4.*;
 import java.sql.*;
 
 public class TestNumeric
@@ -35,11 +35,11 @@ public class TestNumeric
             url = "jdbc:virtuoso://localhost:1111";
          else
             url = args[0];
-         Class.forName("virtuoso.jdbc2.Driver");
+         Class.forName("virtuoso.jdbc4.Driver");
          System.out.println("-------------------------- Test of Numeric ------------------------");
          System.out.print("Establish connection at " + url);
          Connection connection = DriverManager.getConnection(url,"dba","dba");
-         if(connection instanceof virtuoso.jdbc2.VirtuosoConnection)
+         if(connection instanceof virtuoso.jdbc4.VirtuosoConnection)
             System.out.println("    PASSED");
          else
          {
@@ -48,7 +48,7 @@ public class TestNumeric
          }
 	 System.out.print("Create a Statement class attached to this connection");
          Statement stmt = connection.createStatement();
-         if(stmt instanceof virtuoso.jdbc2.VirtuosoStatement)
+         if(stmt instanceof virtuoso.jdbc4.VirtuosoStatement)
             System.out.println("    PASSED");
          else
          {
@@ -117,6 +117,40 @@ public class TestNumeric
             System.out.println("    FAILED");
             System.exit(-1);
          }
+
+	 System.out.print("Execute CREATE PROC");
+         if(stmt.executeUpdate("create procedure test_int () { declare _new, _old, _it int; _new := 2; _old := 1; result_names (_it, _new, _old); while (_new > _old) { _old := _new; _new := _new + _new; _it := _it + 1; result (_new / _old, _old, _new); } end_result (); return; }") == 0)
+            System.out.println("    PASSED");
+         else
+         {
+            System.out.println("    FAILED");
+            System.exit(-1);
+         }
+
+         System.out.print("Execute INT64 TEST");
+         rs = stmt.executeQuery("SELECT TEST_INT ()");
+
+	 for(int inx = 1; inx < 63 ; inx=inx+1)
+	     rs.next();
+
+	 if(rs.getInt(1) == -2)
+            System.out.println("    PASSED");
+         else
+         {
+            System.out.println(rs.getInt(1));
+            System.out.println("    FAILED");
+            System.exit(-1);
+         }
+         System.out.print("Execute DROP PROCEDURE");
+         if(stmt.executeUpdate("DROP PROCEDURE test_int") == 0)
+            System.out.println("    PASSED");
+         else
+         {
+            System.out.println("    FAILED");
+            System.exit(-1);
+         }
+
+
          System.out.print("Execute CREATE TABLE");
          if(stmt.executeUpdate("create table EX..DEMO (ID integer,VAL numeric(10,0))") == 0)
             System.out.println("    PASSED");
