@@ -2,7 +2,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -34,11 +34,11 @@ public class TestPrepareBatch
             url = "jdbc:virtuoso://localhost:1111";
          else
             url = args[0];
-         Class.forName("virtuoso.jdbc3.Driver");
+         Class.forName("virtuoso.jdbc4.Driver");
          System.out.println("---------------- Test of batch in PreparedStatement ---------------");
          System.out.print("Establish connection at " + url);
          Connection connection = DriverManager.getConnection(url,"dba","dba");
-         if(connection instanceof virtuoso.jdbc3.VirtuosoConnection)
+         if(connection instanceof virtuoso.jdbc4.VirtuosoConnection)
             System.out.println("    PASSED");
          else
          {
@@ -47,7 +47,7 @@ public class TestPrepareBatch
          }
          System.out.print("Create a Statement class attached to this connection");
          Statement stmt = connection.createStatement();
-         if(stmt instanceof virtuoso.jdbc3.VirtuosoStatement)
+         if(stmt instanceof virtuoso.jdbc4.VirtuosoStatement)
             System.out.println("    PASSED");
          else
          {
@@ -83,10 +83,10 @@ public class TestPrepareBatch
          int[] res = pstmt.executeBatch();
          pstmt.close();
          for(int i = 0;i < 100;i++)
-            if(res[i] != 1)
+            if(res[i] != 1 && res[i]!=PreparedStatement.SUCCESS_NO_INFO)
             {
                System.out.println(i + " " + res[i]);
-               System.out.println("    FAILED");
+               System.out.print("    FAILED");
                System.exit(-1);
             }
          System.out.println("    PASSED");
@@ -117,7 +117,7 @@ public class TestPrepareBatch
          res = pstmt.executeBatch();
          pstmt.close();
          for(int i = 0;i < 100;i++)
-            if(res[i] != 1)
+            if(res[i] != 1 && res[i]!=PreparedStatement.SUCCESS_NO_INFO)
             {
                System.out.print("    FAILED");
                System.exit(-1);
