@@ -2,7 +2,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -33,7 +33,7 @@ public class TestTimeUpdate
             url = "jdbc:virtuoso://localhost:1111";
          else
             url = args[0];
-         Class.forName("virtuoso.jdbc2.Driver");
+         Class.forName("virtuoso.jdbc4.Driver");
          System.out.println("--------------------- Test of the update col with setTime -------------------");
          System.out.print("Establish connection at " + url);
 	 Connection c = DriverManager.getConnection (url, "dba", "dba");
@@ -98,8 +98,8 @@ public class TestTimeUpdate
 	 ps.setDate (1, new java.sql.Date(72, 6, 29));
 	 rs = ps.executeQuery();
 	 rs.next();
-	 System.out.print(" (recv as " + rs.getString(1) + ")");
-	 if (rs.getString(1).equals ("1972-07-29"))
+	 System.out.print(" (recv as " + rs.getString(1) + " trim=[" + rs.getString(1).substring(0, 10) + "])");
+	 if (rs.getString(1).substring(0, 10).equals ("1972-07-29"))
 	   System.out.println("    PASSED");
 	 else
 	   System.out.println("    FAILED");
@@ -110,8 +110,8 @@ public class TestTimeUpdate
 	 ps.setTime (1, new java.sql.Time(15, 30, 45));
 	 rs = ps.executeQuery();
 	 rs.next();
-	 System.out.print(" (recv as " + rs.getString(1) + " trim=[" + rs.getString(1) + "])");
-	 if (rs.getString(1).equals ("15:30:45"))
+	 System.out.print(" (recv as " + rs.getString(1) + " trim=[" + rs.getString(1).substring(0, 8) + "])");
+	 if (rs.getString(1).substring(0, 8).equals ("15:30:45"))
 	   System.out.println("    PASSED");
 	 else
 	   System.out.println("    FAILED");
