@@ -4,7 +4,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -138,6 +138,10 @@ public class VirtuosoConnection implements Connection
    // utf8_encoding for statements
    protected boolean utf8_execs = false;
 
+   // timezoneless datetimes setting
+   protected int timezoneless_datetimes = 0;
+  
+
    // set if the connection is managed through VirtuosoPooledConnection;
 #if JDK_VER >= 14
    protected VirtuosoPooledConnection pooled_connection = null;
@@ -175,7 +179,7 @@ public class VirtuosoConnection implements Connection
 
   private boolean useRoundRobin;
   // The pingStatement to know if the connection is still available
-  private PreparedStatement pingStatement = null;
+  private Statement pingStatement = null;
 
 
    protected class VhostRec
@@ -359,18 +363,24 @@ public class VirtuosoConnection implements Connection
       // Connect to the database
       connect(host,port,(String)prop.get("database"), sendbs, recvbs, (prop.get("log_enable") != null ? (Integer.parseInt(prop.getProperty("log_enable"))) : -1));
 
-      pingStatement = prepareStatement("select 1");
+      pingStatement = createStatement();
    }
 
-   public boolean isConnectionLost(int timeout_sec) 
+   public synchronized boolean isConnectionLost(int timeout_sec) 
    {
+     ResultSet rs = null;
      try{
 	pingStatement.setQueryTimeout(timeout_sec);
-        pingStatement.execute();
+        rs = pingStatement.executeQuery("select 1");
         return false;
      } catch (Exception e ) {
         return true;
-     } 
+     } finally {
+       if (rs!=null)
+         try{
+           rs.close();
+         } catch(Exception e){}
+     }
    }
 
    protected int getIntAttr(java.util.Properties info, String key, int def)
@@ -430,7 +440,8 @@ public class VirtuosoConnection implements Connection
           break;
         } catch (VirtuosoException e) {
 
-          if (e.getErrorCode() != VirtuosoException.IOERROR)
+          int erc = e.getErrorCode();
+          if (erc != VirtuosoException.IOERROR && erc != VirtuosoException.NOLICENCE)
             throw e;
 
           hostIndex++;
@@ -689,10 +700,8 @@ public class VirtuosoConnection implements Connection
 			   //System.err.println ("txn timeout = " + txn_timeout);
 			 }
 
-			 if (trxisolation <= 0) {
-			   trxisolation = (int) cdef_param (client_defaults, "SQL_TXN_ISOLATION", trxisolation);
-			   //System.err.println ("txn isolation = " + trxisolation);
-			 }
+			 trxisolation = (int) cdef_param (client_defaults, "SQL_TXN_ISOLATION", trxisolation);
+			 //System.err.println ("txn isolation = " + trxisolation);
 
 			 utf8_execs = cdef_param (client_defaults, "SQL_UTF8_EXECS", 0) != 0;
 			 //System.err.println ("utf8_execs = " + utf8_execs);
@@ -705,6 +714,9 @@ public class VirtuosoConnection implements Connection
 			 //if ((new Integer (version.substring (6, 10))).intValue() > 2143)
 			 //  utf8_execs = true;
 
+			 timezoneless_datetimes = (int) cdef_param (client_defaults, "SQL_TIMEZONELESS_DATETIMES", 0);
+			 //System.err.println ("timezoneless_datetimes = " + timezoneless_datetimes);
+
 			 break;
 		     case VirtuosoTypes.QA_ERROR:
 			 // Remove the future reference
@@ -783,12 +795,8 @@ public class VirtuosoConnection implements Connection
    {
      if (VirtuosoFuture.rpc_log != null)
        {
-	 synchronized (VirtuosoFuture.rpc_log)
-	   {
-	     VirtuosoFuture.rpc_log.print ("(conn " + hashCode() + ") OUT ");
+	     VirtuosoFuture.rpc_log.print ("  >> (conn " + hashCode() + ") OUT ");
 	     VirtuosoFuture.rpc_log.println (obj != null ? obj.toString() : "<null>");
-	     VirtuosoFuture.rpc_log.flush();
-	   }
        }
 #if JDK_VER >= 14
     try {
@@ -938,12 +946,8 @@ public class VirtuosoConnection implements Connection
      //System.out.println ("req end");
      if (VirtuosoFuture.rpc_log != null)
        {
-	 synchronized (VirtuosoFuture.rpc_log)
-	   {
-	     VirtuosoFuture.rpc_log.print ("(conn " + hashCode() + ") IN ");
+	     VirtuosoFuture.rpc_log.print ("  << (conn " + hashCode() + ") IN ");
 	     VirtuosoFuture.rpc_log.println (_result != null ? _result.toString() : "<null>");
-	     VirtuosoFuture.rpc_log.flush();
-	   }
        }
 
      try
@@ -969,13 +973,9 @@ public class VirtuosoConnection implements Connection
        {
          if (VirtuosoFuture.rpc_log != null)
            {
-             synchronized (VirtuosoFuture.rpc_log)
-               {
-                 VirtuosoFuture.rpc_log.println ("(conn " + hashCode() + ") **** runtime2 " +
+                 VirtuosoFuture.rpc_log.println ("  **(conn " + hashCode() + ") **** runtime2 " +
                      e.getClass().getName() + " in read_request");
                  e.printStackTrace(VirtuosoFuture.rpc_log);
-		 VirtuosoFuture.rpc_log.flush();
-               }
            }
          throw new Error (e.getClass().getName() + ":" + e.getMessage());
        }
@@ -1075,37 +1075,39 @@ public class VirtuosoConnection implements Connection
     */
    public void close() throws VirtuosoException
    {
+      if (isClosed())
+        return;
+
       try
       {
-         // Is already closed ?
-         if(isClosed())
-            throw new VirtuosoException("The connection is already closed.",VirtuosoException.DISCONNECTED);
-         // Try to close all about the connection : socket and streams.
-         if(!in.isClosed())
-         {
-            in.close();
-            in = null;
-         }
-         if(!out.isClosed())
-         {
-            out.close();
-            out = null;
-         }
-         if(socket != null)
-         {
-            socket.close();
-            socket = null;
-         }
+         synchronized(this) {
+           // Try to close all about the connection : socket and streams.
+           if(!in.isClosed())
+           {
+             in.close();
+             in = null;
+           }
+           if(!out.isClosed())
+           {
+             out.close();
+             out = null;
+           }
+           if(socket != null)
+           {
+             socket.close();
+             socket = null;
+           }
 #if JDK_VER >= 16
-         pStatementCache.clear();
+           pStatementCache.clear();
 #endif
-         // Clear some variables
-         user = url = password = null;
-         futures = null;
+           // Clear some variables
+           user = url = password = null;
+           futures = null;
 #if JDK_VER >= 14
-         pooled_connection = null;
-         xa_connection = null;
+           pooled_connection = null;
+           xa_connection = null;
 #endif
+         }
       }
       catch(IOException e)
       {
@@ -1503,6 +1505,20 @@ public class VirtuosoConnection implements Connection
     */
    public void setCatalog(String catalog) throws VirtuosoException
    {
+      VirtuosoStatement st = null;
+      if (catalog!=null) {
+        try {
+          st = new VirtuosoStatement(this);
+          st.executeQuery("use "+catalog);
+          qualifier = catalog;
+        } finally {
+          if (st!=null) {
+            try {
+              st.close();
+            } catch (Exception e) {}
+          }
+        }
+      }
    }
 
    /**
@@ -1844,6 +1860,12 @@ public class VirtuosoConnection implements Connection
        return prepareStatement (sql);
      }
 
+   synchronized void checkClosed() throws SQLException
+   {
+        if (isClosed())
+            throw new VirtuosoException("The connection is already closed.",VirtuosoException.DISCONNECTED);
+    }
+
 #if JDK_VER >= 16
     //------------------------- JDBC 4.0 -----------------------------------
     /**
@@ -2148,7 +2170,13 @@ public class VirtuosoConnection implements Connection
   */
   public Array createArrayOf(String typeName, Object[] elements) throws SQLException
   {
-    throw new VirtuosoFNSException ("createArrayOf(typeName, elements)  not supported", VirtuosoException.NOTIMPLEMENTED);
+      checkClosed();
+      if (typeName == null)
+          throw new VirtuosoException("typeName is null.",VirtuosoException.MISCERROR);
+
+      if (elements == null)
+          return null;
+      return new VirtuosoArray(typeName, elements);
   }
 
 /**
@@ -2533,11 +2561,7 @@ public class VirtuosoConnection implements Connection
     boolean getGlobalTransaction() {
 	   if (VirtuosoFuture.rpc_log != null)
 	   {
-	       synchronized (VirtuosoFuture.rpc_log)
-	       {
 		   VirtuosoFuture.rpc_log.println ("VirtuosoConnection.getGlobalTransaction () (con=" + this.hashCode() + ") :" + global_transaction);
-		   VirtuosoFuture.rpc_log.flush();
-	       }
 	   }
         return global_transaction;
     }
@@ -2545,11 +2569,7 @@ public class VirtuosoConnection implements Connection
     void setGlobalTransaction(boolean value) {
 	   if (VirtuosoFuture.rpc_log != null)
 	   {
-	       synchronized (VirtuosoFuture.rpc_log)
-	       {
 		   VirtuosoFuture.rpc_log.println ("VirtuosoConnection.getGlobalTransaction (" + value + ") (con=" + this.hashCode() + ") :" + global_transaction);
-		   VirtuosoFuture.rpc_log.flush();
-	       }
 	   }
         global_transaction = value;
     }
