@@ -4,7 +4,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -69,7 +69,7 @@ class VirtuosoFuture
    // Set if there has been a DA_FUTURE_ANSWER message to this future
    private boolean is_complete = false;
 
-   protected static PrintStream rpc_log = null;
+   protected static PrintWriter rpc_log = null;
    // Mutex used to access to the queue of results
    //private Semaphore mutex;
    /**
@@ -223,18 +223,5 @@ class VirtuosoFuture
       return false;
    }
 
-   /**
-    * Method runs when the garbage collector want to erase the object
-    */
-   public void finalize() throws Throwable
-   {
-      connection = null;
-      if(results != null)
-      {
-         results.removeAllElements();
-         results = null;
-      }
-   }
-
 }
 
