@@ -4,7 +4,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -140,7 +140,7 @@ class VirtuosoOutputStream extends BufferedOutputStream
 #if JDK_VER >= 12
 	       {
 		 //System.out.println("DV_ARRAY_OF_POINTER");
-		 LinkedList o = (LinkedList)obj;
+		 List o = (List)obj;
 		 int length = o.size();
 		 write(VirtuosoTypes.DV_ARRAY_OF_POINTER);
 		 writeint(length);
@@ -720,28 +720,28 @@ class VirtuosoOutputStream extends BufferedOutputStream
 	{
 	  int day = yday;
 	  int sec = 0;
-	  sec = VirtuosoInputStream.time_to_sec (0, _hour, _minute, _second);
+	  sec = DateObject.time_to_sec (0, _hour, _minute, _second);
 	  sec -= 60 * tz;
 	  if (sec < 0)
 	    {
-	      day = day - (1 + ((-sec) / VirtuosoInputStream.SPERDAY));
+	      day = day - (1 + ((-sec) / DateObject.SPERDAY));
 
-	      sec = sec % VirtuosoInputStream.SPERDAY;
+	      sec = sec % DateObject.SPERDAY;
 
 	      if (sec == 0)
 		day++;
 
-	      sec = VirtuosoInputStream.SPERDAY + sec;
+	      sec = DateObject.SPERDAY + sec;
 	    }
 	  else
 	    {
-	      day = day + sec / VirtuosoInputStream.SPERDAY;
-	      sec = sec % VirtuosoInputStream.SPERDAY;
+	      day = day + sec / DateObject.SPERDAY;
+	      sec = sec % DateObject.SPERDAY;
 	    }
-	  int dummy_day = sec / VirtuosoInputStream.SPERDAY;
+	  int dummy_day = sec / DateObject.SPERDAY;
 
-	  _hour = (sec - (dummy_day * VirtuosoInputStream.SPERDAY)) / (60 * 60);
-	  _minute = (sec - (dummy_day * VirtuosoInputStream.SPERDAY) - (_hour * 60 * 60)) / 60;
+	  _hour = (sec - (dummy_day * DateObject.SPERDAY)) / (60 * 60);
+	  _minute = (sec - (dummy_day * DateObject.SPERDAY) - (_hour * 60 * 60)) / 60;
 	  _second = sec % 60;
 	  yday = day;
 	}
@@ -854,6 +854,8 @@ class VirtuosoOutputStream extends BufferedOutputStream
 #if JDK_VER >= 12
       if(obj instanceof LinkedList)
          return VirtuosoTypes.DV_LIST_OF_POINTER;
+       if(obj instanceof ArrayList)
+           return VirtuosoTypes.DV_LIST_OF_POINTER;
 #endif
       if(obj instanceof openlink.util.Vector)
          return VirtuosoTypes.DV_ARRAY_OF_POINTER;
