@@ -4,7 +4,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -81,6 +81,7 @@ public class VirtuosoPreparedStatement extends VirtuosoStatement implements Prep
    VirtuosoPreparedStatement(VirtuosoConnection connection, String sql, int type, int concurrency) throws VirtuosoException
    {
       super(connection,type,concurrency);
+      sparql_executed =  sql.trim().regionMatches(true, 0, "sparql", 0, 6);
       synchronized (connection)
 	{
 	  try
@@ -99,6 +100,7 @@ public class VirtuosoPreparedStatement extends VirtuosoStatement implements Prep
 	      future = connection.getFuture(VirtuosoFuture.prepare,args, this.rpc_timeout);
 	      // Process result to get information about results meta data
 	      vresultSet = new VirtuosoResultSet(this,metaData, true);
+	      result_opened = true;
               clearParameters();
 	    }
 	  catch(IOException e)
@@ -147,6 +149,11 @@ public class VirtuosoPreparedStatement extends VirtuosoStatement implements Prep
        {
 	 Object[] args = new Object[6];
 	 openlink.util.Vector vect = new openlink.util.Vector(1);
+         if (future != null) 
+           {
+             connection.removeFuture(future);
+             future = null;
+           }
 	 // Set arguments to the RPC function
 	 args[0] = statid;
 	 args[2] = (cursorName == null) ? args[0] : cursorName;
@@ -160,7 +167,10 @@ public class VirtuosoPreparedStatement extends VirtuosoStatement implements Prep
 	     // Put the options array in the args array
 	     args[5] = getStmtOpts();
 	     future = connection.getFuture(VirtuosoFuture.exec,args, this.rpc_timeout);
+             vresultSet.isLastResult = false;
 	     vresultSet.getMoreResults(false);
+             vresultSet.stmt_n_rows_to_get = this.prefetch;
+	     result_opened = true;
 	   }
 	 catch(IOException e)
 	   {
@@ -255,14 +265,21 @@ public class VirtuosoPreparedStatement extends VirtuosoStatement implements Prep
 	 args[4] = null;
 	 try
 	   {
+             if (future != null) 
+               {
+	         connection.removeFuture(future);
+	         future = null;
+               }
+
 	     // Put the options array in the args array
 	     args[5] = getStmtOpts();
 	     future = connection.getFuture(VirtuosoFuture.exec,args, this.rpc_timeout);
+             vresultSet.isLastResult = false;
 	     for (inx = 0; inx < size; inx++)
 	     {
 		 vresultSet.setUpdateCount (0);
 		 vresultSet.getMoreResults (false);
-		 res[inx] = vresultSet.getUpdateCount();
+		 res[inx] = SUCCESS_NO_INFO; //vresultSet.getUpdateCount();
 	     }
 	   }
 	 catch(IOException e)
@@ -328,10 +345,14 @@ public class VirtuosoPreparedStatement extends VirtuosoStatement implements Prep
     }
 #endif
 
+     if(close_flag)
+       return;
+
      synchronized (connection)
        {
 	 try
 	   {
+	     close_flag = true;
 	     // Check if a statement is treat
 	     if(statid == null)
 	       return;
@@ -349,6 +370,7 @@ public class VirtuosoPreparedStatement extends VirtuosoStatement implements Prep
 	     // Remove the future reference
 	     connection.removeFuture(future);
 	     future = null;
+	     result_opened = false;
 	   }
 	 catch(IOException e)
 	   {
@@ -766,132 +788,6 @@ public class VirtuosoPreparedStatement extends VirtuosoStatement implements Prep
       setObject(parameterIndex,x,targetSqlType, 0);
    }
 
-   protected Object mapJavaTypeToSqlType (Object x, int targetSqlType, int scale) throws VirtuosoException
-   {
-     if (x == null)
-       return x;
-     if (x instanceof java.lang.Boolean)
-       x = new Integer (((Boolean)x).booleanValue() ? 1 : 0);
-
-     switch (targetSqlType)
-       {
-	  case Types.CHAR:
-	  case Types.VARCHAR:
-	      if (x instanceof java.util.Date || x instanceof java.lang.String)
-		return x;
-	      else
-		return x.toString();
-
-	  case Types.LONGVARCHAR:
-#if JDK_VER >= 12
-              if (x instanceof java.sql.Clob || x instanceof java.sql.Blob || x instanceof java.lang.String)
-#else
-              if (x instanceof VirtuosoClob || x instanceof VirtuosoBlob || x instanceof java.lang.String)
-#endif
-                return x;
-              else
-		return x.toString();
-
-	  case Types.DATE:
-	  case Types.TIME:
-	  case Types.TIMESTAMP:
-	      if (x instanceof java.util.Date || x instanceof java.lang.String)
-		return x;
-              break;
-
-          case Types.NUMERIC:
-          case Types.DECIMAL:
-              {
-                java.math.BigDecimal bd = null;
-		if (x instanceof java.math.BigDecimal)
-		  bd = (java.math.BigDecimal) x;
-                else if (x instanceof java.lang.String)
-		  bd = new java.math.BigDecimal ((String) x);
-		else if (x instanceof java.lang.Number)
-		  bd = new java.math.BigDecimal (x.toString());
-                if (bd != null)
-		  return bd.setScale (scale);
-              }
-	      break;
-
-          case Types.BIGINT:
-              if (x instanceof java.math.BigDecimal || x instanceof java.lang.String)
-                return new Long(x.toString());
-              else if (x instanceof java.lang.Number)
-                return new Long(((Number)x).longValue());
-	      break;
-
-          case Types.FLOAT:
-          case Types.DOUBLE:
-              if (x instanceof java.lang.Double)
-                return x;
-              else if (x instanceof java.lang.Number)
-                return new Double (((Number)x).doubleValue());
-              else if (x instanceof java.lang.String)
-                return new Double ((String) x);
-	      break;
-          case Types.INTEGER:
-              if (x instanceof java.lang.Integer)
-                return x;
-              else if (x instanceof java.lang.Number)
-                return new Integer (((Number)x).intValue());
-              else if (x instanceof java.lang.String)
-                return new Integer ((String) x);
-	      break;
-
-          case Types.REAL:
-              if (x instanceof java.lang.Float)
-                return x;
-              else if (x instanceof java.lang.Number)
-                return new Float (((Number)x).floatValue());
-              else if (x instanceof java.lang.String)
-                return new Float ((String) x);
-	      break;
-
-          case Types.SMALLINT:
-          case Types.TINYINT:
-          case Types.BIT:
-#if JDK_VER >= 14
-          case Types.BOOLEAN:
-#endif
-              if (x instanceof java.lang.Short)
-                return x;
-              else if (x instanceof java.lang.String)
-                return new Short ((String) x);
-              else if (x instanceof java.lang.Number)
-                return new Short (((Number)x).shortValue());
-	      break;
-
-	  case Types.ARRAY:
-#if JDK_VER >= 14
-	  case Types.DATALINK:
-#endif
-#if JDK_VER >= 16
-      case Types.ROWID:
-#endif
-          case Types.DISTINCT:
-	  case Types.REF:
-	      throw new VirtuosoException ("Type not supported", VirtuosoException.NOTIMPLEMENTED);
-
-          case Types.VARBINARY:
-              if (x instanceof byte[])
-                return x;
-              break;
-
-          case Types.LONGVARBINARY:
-#if JDK_VER >= 12
-              if (x instanceof java.sql.Blob || x instanceof byte [])
-#else
-              if (x instanceof VirtuosoBlob || x instanceof byte [])
-#endif
-                return x;
-              break;
-
-	  default:
-	      return x;
-       }
-     throw new VirtuosoException ("Invalid value specified", VirtuosoException.BADPARAM);
-   }
 
    /**
     * Sets the value of a parameter using an object. The second
@@ -930,7 +826,7 @@ public class VirtuosoPreparedStatement extends VirtuosoStatement implements Prep
       }
       // Else create a Blob
       if(x == null) this.setNull(parameterIndex, Types.OTHER);
-      x = mapJavaTypeToSqlType (x, targetSqlType, scale);
+      x = VirtuosoTypes.mapJavaTypeToSqlType (x, targetSqlType, scale);
       if (x instanceof java.io.Serializable)
 	{
 	  //System.err.println ("setObject2 (" + parameterIndex + ", " + x + ", " + targetSqlType + ", " + scale);
@@ -990,8 +886,8 @@ public class VirtuosoPreparedStatement extends VirtuosoStatement implements Prep
 	  else
 	    {
 	    objparams.setElementAt(x,parameterIndex - 1);
-	}
-   }
+	    }
+        }
    }
 
    protected void setString(int parameterIndex, VirtuosoExplicitString x) throws VirtuosoException
@@ -1138,8 +1034,13 @@ public class VirtuosoPreparedStatement extends VirtuosoStatement implements Prep
       // Check parameters
       if(i < 1 || i > parameters.capacity())
          throw new VirtuosoException("Index " + i + " is not 1<n<" + parameters.capacity(),VirtuosoException.BADPARAM);
-      if(x == null) this.setNull(i, Types.ARRAY);
-      else objparams.setElementAt(x,i - 1);
+      if(x == null){
+          this.setNull(i, Types.ARRAY);
+      } else if (x instanceof VirtuosoArray) {
+          objparams.setElementAt(((VirtuosoArray)x).data, i - 1);
+      } else {
+          objparams.setElementAt(x,i - 1);
+      }
    }
 #endif
 
@@ -1328,8 +1229,7 @@ public class VirtuosoPreparedStatement extends VirtuosoStatement implements Prep
 	 {
 	   if(cal != null)
 	     {
-	       cal.setTime((java.util.Date)x);
-	       x = new java.sql.Date (cal.getTime().getTime());
+	       x = new java.sql.Date (VirtuosoTypes.timeFromCal(x, cal));
 	     }
 	   objparams.setElementAt(x,parameterIndex - 1);
 	 }
@@ -1357,8 +1257,7 @@ public class VirtuosoPreparedStatement extends VirtuosoStatement implements Prep
 	 {
 	   if(cal != null)
 	     {
-	       cal.setTime((java.util.Date)x);
-	       x = new java.sql.Time (cal.getTime().getTime());
+	       x = new java.sql.Time (VirtuosoTypes.timeFromCal(x, cal));
 	     }
 	   objparams.setElementAt(x,parameterIndex - 1);
 	 }
@@ -1387,8 +1286,7 @@ public class VirtuosoPreparedStatement extends VirtuosoStatement implements Prep
 	   if(cal != null)
 	     {
 	       int nanos = x.getNanos();
-	       cal.setTime((java.util.Date)x);
-	       x = new java.sql.Timestamp (cal.getTime().getTime());
+	       x = new java.sql.Timestamp(VirtuosoTypes.timeFromCal(x, cal));
                x.setNanos (nanos);
 	     }
 	   objparams.setElementAt(x,parameterIndex - 1);
