@@ -4,7 +4,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -127,6 +127,7 @@ aq_thread_func (aq_thread_t * aqt)
       async_queue_t *aq = aqt->aqt_aq;
       aq_request_t *aqr = aqt->aqt_aqr;
       assert (AQR_QUEUED == aqr->aqr_state);
+      aqt->aqt_thread->thr_tlsf = aqt->aqt_cli->cli_tlsf;
       if (!aq->aq_no_lt_enter)
 	{
 	  lt_enter_anyway (aqt->aqt_cli->cli_trx);
@@ -242,7 +243,7 @@ aqt_allocate ()
 	lt_done (cli->cli_trx);
 	LEAVE_TXN;
 	client_connection_free (cli);
-	dk_free_tree (ses);
+	PrpcSessionFree (ses);	/* not a box and not free in freeing cli */
 	dk_free (aqt, sizeof (aq_thread_t));
 	return NULL;
       }
@@ -766,6 +767,7 @@ async_queue_t *
 aq_allocate (client_connection_t * cli, int n_threads)
 {
   async_queue_t *aq = (async_queue_t *) dk_alloc_box_zero (sizeof (async_queue_t), DV_ASYNC_QUEUE);
+  aq->aq_creator_cli = cli;
   aq->aq_ref_count = 1;
   aq->aq_requests = hash_table_allocate (101);
   aq->aq_mtx = all_aq_mtx;
@@ -900,6 +902,7 @@ bif_async_queue (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   if (AQ_TXN_BRANCH & flags)
     {
       aq->aq_rc_w_id = LT_MAIN_W_ID (qi->qi_client->cli_trx);
+      qi->qi_client->cli_trx->lt_has_branches = 1;
       lt_timestamp (qi->qi_trx, (char *) &aq->aq_lt_timestamp);
     }
   if (!(flags & AQ_CLUSTER_RECURSIVE))
@@ -947,7 +950,7 @@ aq_sql_func (caddr_t * av, caddr_t * err_ret)
   if (!cli->cli_user || !sec_proc_check (proc, cli->cli_user->usr_id, cli->cli_user->usr_g_id))
     {
       user_t * usr = cli->cli_user;
-      *err_ret = srv_make_new_error ("42000", "SR186", "No permission to execute %s in aq_request() with user ID %d, group ID %d",
+      *err_ret = srv_make_new_error ("42000", "SR186:SECURITY", "No permission to execute %s in aq_request() with user ID %d, group ID %d",
         full_name, (int)(usr ? usr->usr_id : 0), (int)(usr ? usr->usr_g_id : 0) );
       dk_free_tree ((caddr_t) params);
       return NULL;
@@ -956,7 +959,7 @@ aq_sql_func (caddr_t * av, caddr_t * err_ret)
   {
     if (SSL_REF_PARAMETER == ssl->ssl_type)
       {
-	*err_ret = srv_make_new_error ("42000", "AQ002", "Reference parameters not allowed in aq_request");
+	  *err_ret = srv_make_new_error ("42000", "AQ002", "Reference parameters not allowed in aq_request()");
 	dk_free_tree ((caddr_t) params);
 	return NULL;
       }
