@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -41,6 +41,7 @@
 #include "multibyte.h"
 #include "srvmultibyte.h"
 #include "util/strfuns.h"
+#include "uname_const_decl.h"
 
 #define KUBL_ILLEGAL_DATE_VALUE (0)	/* Added by AK 15-JAN-1997. */
 
@@ -57,6 +58,56 @@ bif_date_arg (caddr_t * qst, state_slot_t ** args, int nth, char *func)
   return arg;
 }
 
+caddr_t
+bif_date_arg_rb_type (caddr_t * qst, state_slot_t ** args, int nth, char *func, int *rb_type_ret)
+{
+  rdf_box_t *src_rdf_box;
+  caddr_t arg = bif_arg_unrdf_ext (qst, args, nth, func, (caddr_t *)(&src_rdf_box));
+  dtp_t dtp = DV_TYPE_OF (arg);
+  if (dtp != DV_DATETIME && dtp != DV_BIN)
+    sqlr_new_error ("22007", "DT001",
+	"Function %s needs a datetime, date or time as argument %d, not an arg of type %s (%d)",
+	func, nth + 1, dv_type_title (dtp), dtp);
+  if ((DV_RDF == DV_TYPE_OF (src_rdf_box))
+    && (RDF_BOX_DEFAULT_TYPE != src_rdf_box->rb_type)
+    && ((rb_type__xsd_gDay		== src_rdf_box->rb_type)
+        || (rb_type__xsd_gMonth		== src_rdf_box->rb_type)
+        || (rb_type__xsd_gMonthDay	== src_rdf_box->rb_type)
+        || (rb_type__xsd_gYear		== src_rdf_box->rb_type)
+        || (rb_type__xsd_gYearMonth	== src_rdf_box->rb_type) ) )
+    rb_type_ret[0] = src_rdf_box->rb_type;
+  else
+    rb_type_ret[0] = RDF_BOX_ILL_TYPE;
+  return arg;
+}
+
+int
+dt_print_flags_of_rb_type (int rb_type)
+{
+  if (rb_type__xsd_gDay		== rb_type)	return DT_PRINT_MODE_YMD | DT_PRINT_MODE_NO_Y | DT_PRINT_MODE_NO_M                                         ;
+  if (rb_type__xsd_gMonth	== rb_type)	return DT_PRINT_MODE_YMD | DT_PRINT_MODE_NO_Y                      | DT_PRINT_MODE_NO_D                    ;
+  if (rb_type__xsd_gMonthDay	== rb_type)	return DT_PRINT_MODE_YMD | DT_PRINT_MODE_NO_Y                                                              ;
+  if (rb_type__xsd_gYear	== rb_type)	return DT_PRINT_MODE_YMD                      | DT_PRINT_MODE_NO_M | DT_PRINT_MODE_NO_D                    ;
+  if (rb_type__xsd_gYearMonth	== rb_type)	return DT_PRINT_MODE_YMD                                           | DT_PRINT_MODE_NO_D                    ;
+  if (rb_type__xsd_date		== rb_type)	return DT_PRINT_MODE_YMD                                                                                   ;
+  if (rb_type__xsd_dateTime	== rb_type)	return DT_PRINT_MODE_YMD                                                                | DT_PRINT_MODE_HMS;
+  if (rb_type__xsd_time		== rb_type)	return                                                                                    DT_PRINT_MODE_HMS;
+  return 0;
+}
+
+int
+dt_print_flags_of_xsd_type_uname (ccaddr_t xsd_type_uname)
+{
+  if (uname_xmlschema_ns_uri_hash_gDay		== xsd_type_uname)	return DT_PRINT_MODE_YMD | DT_PRINT_MODE_NO_Y | DT_PRINT_MODE_NO_M                                         ;
+  if (uname_xmlschema_ns_uri_hash_gMonth	== xsd_type_uname)	return DT_PRINT_MODE_YMD | DT_PRINT_MODE_NO_Y                      | DT_PRINT_MODE_NO_D                    ;
+  if (uname_xmlschema_ns_uri_hash_gMonthDay	== xsd_type_uname)	return DT_PRINT_MODE_YMD | DT_PRINT_MODE_NO_Y                                                              ;
+  if (uname_xmlschema_ns_uri_hash_gYear		== xsd_type_uname)	return DT_PRINT_MODE_YMD                      | DT_PRINT_MODE_NO_M | DT_PRINT_MODE_NO_D                    ;
+  if (uname_xmlschema_ns_uri_hash_gYearMonth	== xsd_type_uname)	return DT_PRINT_MODE_YMD |                                           DT_PRINT_MODE_NO_D                    ;
+  if (uname_xmlschema_ns_uri_hash_date		== xsd_type_uname)	return DT_PRINT_MODE_YMD                                                                                   ;
+  if (uname_xmlschema_ns_uri_hash_dateTime	== xsd_type_uname)	return DT_PRINT_MODE_YMD                                                                | DT_PRINT_MODE_HMS;
+  if (uname_xmlschema_ns_uri_hash_time		== xsd_type_uname)	return                                                                                    DT_PRINT_MODE_HMS;
+  return 0;
+}
 
 caddr_t
 bif_date_string (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
@@ -95,7 +146,28 @@ dt_print_to_buffer (char *buf, caddr_t arg, int mode)
     sqlr_new_error ("22023", "SR593", "Bit 2 in print mode requires TIME or DATETIME argument, not DATE");
   dt_to_GMTimestamp_struct (arg, &ts);
   if (DT_PRINT_MODE_YMD & mode)
-    res += sprintf (buf, "%04d-%02d-%02d", ts.year, ts.month, ts.day);
+    {
+      if (DT_PRINT_MODE_NO_D & mode)
+        {
+          if (DT_PRINT_MODE_NO_Y & mode)
+            res += sprintf (buf, "--%02d", ts.month + ((15 <= ts.day) ? 1 : 0));
+          else if (DT_PRINT_MODE_NO_M & mode)
+            res += sprintf (buf, "%04d", ts.year + ((6 <= ts.month) ? 1 : 0));
+          else if (15 <= ts.day)
+            res += sprintf (buf, "%02d-%02d", ts.year + ((12 == ts.month) ? 1 : 0), (ts.month % 12) + 1);
+          else
+            res += sprintf (buf, "%04d-%02d", ts.year, ts.month);
+        }
+      else if (DT_PRINT_MODE_NO_Y & mode)
+        {
+          if (DT_PRINT_MODE_NO_M & mode)
+            res += sprintf (buf, "---%02d", ts.day);
+          else
+            res += sprintf (buf, "--%04d-%02d", ts.month, ts.day);
+        }
+      else
+        res += sprintf (buf, "%04d-%02d-%02d", ts.year, ts.month, ts.day);
+    }
   if ((DT_PRINT_MODE_YMD & mode) && (DT_PRINT_MODE_HMS & mode))
     buf[res++] = ((DT_PRINT_MODE_XML & mode) ? 'T' : ' ');
   if (DT_PRINT_MODE_HMS & mode)
@@ -111,7 +183,9 @@ dt_print_to_buffer (char *buf, caddr_t arg, int mode)
             res += sprintf (buf + res, ".%03d", (int)(ts.fraction / 1000000));
         }
     }
-  if (DT_PRINT_MODE_XML & mode)
+  if (DT_TZL (arg))
+    return res;
+  else if (DT_PRINT_MODE_XML & mode)
     {
       strcpy (buf + res, "Z");
       return res + 1;
@@ -261,6 +335,7 @@ scan_tz:
       return 0;
     }
   dt_ret[0] = dk_alloc_box (DT_LENGTH, DV_DATETIME);
+  DT_ZAP (dt_ret[0]);
   {
     uint32 day;
     day = date2num (ts.year, ts.month, ts.day);
@@ -269,8 +344,10 @@ scan_tz:
     DT_SET_MINUTE (dt_ret[0], ts.minute);
     DT_SET_SECOND (dt_ret[0], ts.second);
     DT_SET_FRACTION (dt_ret[0], ts.fraction);
-    DT_SET_TZ (dt_ret[0], dt_local_tz);
+    DT_SET_TZ (dt_ret[0], 0); /* was DT_SET_TZ (dt_ret[0], dt_local_tz);  before TZL patch */
   }
+  if (!ymd_found)
+    DT_SET_DAY (dt_ret[0], DAY_ZERO);
   SET_DT_TYPE_BY_DTP (dt_ret[0], (ymd_found ? (hms_found ? DV_DATETIME : DV_DATE) : DV_TIME));
   return (tail - buf);
 }
@@ -433,6 +510,26 @@ bif_week (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 }
 
 
+
+caddr_t
+bif_year (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)	\
+{
+  caddr_t dt = bif_date_arg (qst, args, 0, "year");
+  TIMESTAMP_STRUCT ts;    int year, month, day;
+  num2date (DT_DAY (dt), &year, &month, &day);
+  if ((1 == month && 1 == day) || (12 == month && 31 == day))
+    {
+      int tz = DT_TZ (dt);
+      if (!tz)
+	return box_num (year);
+      dt_to_timestamp_struct (dt, &ts);
+      return box_num (ts.year);
+    }
+  return box_num (year);
+}
+
+
+
 #define DT_PART(part) \
 caddr_t \
 bif_##part (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args) \
@@ -444,7 +541,6 @@ bif_##part (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args) \
 }
 
 
-DT_PART (year)
 DT_PART (month)
 DT_PART (day)
 DT_PART (hour)
@@ -452,13 +548,58 @@ DT_PART (minute)
 DT_PART (second)
 
 
+caddr_t
+bif_is_timezoneless (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
+{
+  caddr_t arg = bif_date_arg (qst, args, 0, "is_timezoneless");
+  return box_num (DT_TZL (arg));
+}
+
 caddr_t
 bif_timezone (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 {
   caddr_t arg = bif_date_arg (qst, args, 0, "timezone");
+  int ignore_tzl = ((1 < BOX_ELEMENTS (args)) && bif_long_arg (qst, args, 1, "timezone"));
+  if (DT_TZL (arg) && !ignore_tzl)
+    return NEW_DB_NULL;
   return box_num (DT_TZ (arg));
 }
 
+caddr_t
+bif_adjust_timezone (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
+{
+  caddr_t arg = bif_date_arg (qst, args, 0, "adjust_timezone");
+  long tzmin = bif_long_range_arg (qst, args, 1, "adjust_timezone", -14*60, 14*60);
+  int ignore_tzl = ((2 < BOX_ELEMENTS (args)) && bif_long_arg (qst, args, 2, "adjust_timezone"));
+  if (DT_TZL (arg) && !ignore_tzl)
+    sqlr_new_error ("22023", "SR636", "Timezoneless argument of adjust_timezone()");
+  arg = box_copy (arg);
+  DT_SET_TZL (arg, 0);
+  DT_SET_TZ (arg, tzmin);
+  return arg;
+}
+
+caddr_t
+bif_forget_timezone (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
+{
+  caddr_t arg = bif_date_arg (qst, args, 0, "forget_timezone");
+  int ignore_timezone = ((1 < BOX_ELEMENTS (args)) && bif_long_arg (qst, args, 1, "forget_timezone"));
+  int tzmin = DT_TZ (arg);
+  GMTIMESTAMP_STRUCT ts;
+  if ((DT_TZL_NEVER_COMPAT == timezoneless_datetimes) && !DT_TZL (arg))
+    sqlr_new_error ("22023", "SR636", "BIF forget_timezone() is disabled if TimezonelessDatetimes parameter of virtuoso.ini is set to 0 or the database is old and the parameter is 0 by default");
+  arg = box_copy (arg);
+  if (tzmin && !ignore_timezone)
+    {
+      dt_to_GMTimestamp_struct (arg, &ts);
+      ts_add (&ts, tzmin, "minute");
+      GMTimestamp_struct_to_dt (&ts, arg);
+    }
+  DT_SET_TZ (arg, 0);
+  DT_SET_TZL (arg, 1);
+  return arg;
+}
+
 #define NASA_TJD_OFFSET (2440000 - 1721423)
 
 caddr_t
@@ -523,6 +664,7 @@ bif_dateadd (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   res = dk_alloc_box (DT_LENGTH, DV_DATETIME);
   GMTimestamp_struct_to_dt (&ts, res);
   DT_SET_TZ (res, DT_TZ (dt));
+  DT_SET_TZL (res, DT_TZL (dt));
   if (DT_TYPE_DATE == dt_type
       && (0 == stricmp (part, "year") || 0 == stricmp (part, "month") || 0 == stricmp (part, "day")))
     DT_SET_DT_TYPE (res, dt_type);
@@ -537,12 +679,20 @@ bif_datediff (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   caddr_t unit = bif_string_arg (qst, args, 0, "datediff");
   caddr_t dt1 = bif_date_arg (qst, args, 1, "datediff");
   caddr_t dt2 = bif_date_arg (qst, args, 2, "datediff");
-  boxint s1 = DT_CAST_TO_TOTAL_SECONDS (dt1);
-  boxint s2 = DT_CAST_TO_TOTAL_SECONDS (dt2);
+  TIMESTAMP_STRUCT ts1;
+  TIMESTAMP_STRUCT ts2;
+  boxint day1, day2;
+  boxint s1, s2;
+  int tz_tweak;
   int frac1, frac2;
   int diffyear, diffmonth;
   if (0 == stricmp (unit, "day"))
     return box_num ((boxint)DT_DAY (dt2) - (boxint)DT_DAY (dt1));
+  if ((DT_TZL (dt1) != DT_TZL (dt2)) && stricmp (unit, "month") && stricmp (unit, "year"))
+    sqlr_new_error ("22023", "DT013", "Mixed timezoned and timezoneless arguments in datediff");
+  s1 = DT_CAST_TO_TOTAL_SECONDS (dt1);
+  s2 = DT_CAST_TO_TOTAL_SECONDS (dt2);
+  tz_tweak = DT_TZ (dt1);
   if (0 == stricmp (unit, "hour"))
     return box_num ((s2 - s1) / (60 * 60));
   if (0 == stricmp (unit, "minute"))
@@ -612,6 +762,7 @@ bif_timestampadd (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   res = dk_alloc_box (DT_LENGTH, DV_DATETIME);
   GMTimestamp_struct_to_dt (&ts, res);
   DT_SET_TZ (res, saved_tz);
+  DT_SET_TZL (res, DT_TZL (dt));
   return res;
 }
 
@@ -632,6 +783,9 @@ bif_timestampdiff (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   if (0 == stricmp (unit, "day"))
     return box_num ((boxint)DT_DAY (dt2) - (boxint)DT_DAY (dt1));
 
+  if ((DT_TZL (dt1) != DT_TZL (dt2)) && stricmp (unit, "month") && stricmp (unit, "year"))
+    sqlr_new_error ("22023", "DT014", "Mixed timezoned and timezoneless arguments in timestampdiff");
+
   if (0 == stricmp (unit, "hour"))
     return box_num ((s2 - s1) / (60 * 60));
 
@@ -688,9 +842,10 @@ caddr_t
 bif_dt_set_tz (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 {
   caddr_t arg = bif_date_arg (qst, args, 0, "dt_set_tz");
-  long tz = (long) bif_long_arg (qst, args, 1, "dt_set_tz");
+  long tz = (long) bif_long_range_arg (qst, args, 1, "dt_set_tz", -14*60, 14*60);
   caddr_t res = box_copy (arg);
   DT_SET_TZ (res, tz);
+  DT_SET_TZL (res, 0);
   return res;
 }
 
@@ -817,11 +972,56 @@ bif_curdatetime (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   caddr_t res = dk_alloc_box (DT_LENGTH, DV_DATETIME);
   dt_now (res);
   if (args && BOX_ELEMENTS (args) > 0)
-    fract = (long) bif_long_arg (qst, args, 0, "curdatetime");
-  DT_SET_FRACTION (res, fract);
+    {
+      fract = (long) bif_long_arg (qst, args, 0, "curdatetime");
+      DT_SET_FRACTION (res, fract);
+    }
   return res;
 }
 
+caddr_t
+bif_curdatetime_tz (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
+{
+  long fract = 0;
+  caddr_t res = dk_alloc_box (DT_LENGTH, DV_DATETIME);
+  dt_now_tz (res);
+  if (args && BOX_ELEMENTS (args) > 0)
+    {
+      fract = (long) bif_long_arg (qst, args, 0, "curdatetime_tz");
+      DT_SET_FRACTION (res, fract);
+    }
+  return res;
+}
+
+caddr_t
+bif_curdatetimeoffset (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
+{
+  long fract = 0;
+  caddr_t res = dk_alloc_box (DT_LENGTH, DV_DATETIME);
+  dt_now_GMT (res);
+  if (args && BOX_ELEMENTS (args) > 0)
+    {
+      fract = (long) bif_long_arg (qst, args, 0, "curdatetimeoffset");
+      DT_SET_FRACTION (res, fract);
+    }
+  DT_SET_TZL (res, 0);
+  DT_SET_TZ (res, dt_local_tz_for_logs);
+  return res;
+}
+
+caddr_t
+bif_curutcdatetime (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
+{
+  long fract = 0;
+  caddr_t res = dk_alloc_box (DT_LENGTH, DV_DATETIME);
+  dt_now_GMT (res);
+  if (args && BOX_ELEMENTS (args) > 0)
+    {
+      fract = (long) bif_long_arg (qst, args, 0, "curutcdatetime");
+      DT_SET_FRACTION (res, fract);
+    }
+  return res;
+}
 
 caddr_t
 bif_curdate (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
@@ -840,8 +1040,10 @@ bif_curtime (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   caddr_t res = dk_alloc_box (DT_LENGTH, DV_DATETIME);
   dt_now (res);
   if (args && BOX_ELEMENTS (args) > 0)
-    fract = (long) bif_long_arg (qst, args, 0, "curtime");
-  DT_SET_FRACTION (res, fract);
+    {
+      fract = (long) bif_long_arg (qst, args, 0, "curtime");
+      DT_SET_FRACTION (res, fract);
+    }
   dt_make_day_zero (res);
   return res;
 }
@@ -903,6 +1105,7 @@ arithm_dt_add_num (ccaddr_t box1, ccaddr_t box2, int subtraction, caddr_t *err_r
   res = dk_alloc_box (DT_LENGTH, DV_DATETIME);
   GMTimestamp_struct_to_dt (&ts, res);
   DT_SET_TZ (res, DT_TZ (box1));
+  DT_SET_TZL (res, DT_TZL (box1));
   if ((DT_TYPE_DATE == dt_type) && (0 == (((whole_seconds * 1000000000L) + nanoseconds) % (SPERDAY * 1000000000L))))
     DT_SET_DT_TYPE (res, dt_type);
   DT_AUDIT_FIELDS (dt);
@@ -988,14 +1191,29 @@ bif_date_init ()
   bif_define_ex ("minute"	, bif_minute		, BMD_ALIAS, "rdf_minutes_impl"	, BMD_RET_TYPE, &bt_integer	, BMD_IS_PURE, BMD_DONE);
   bif_define_ex ("week"				, bif_week				, BMD_RET_TYPE, &bt_integer	, BMD_IS_PURE, BMD_DONE);
   bif_define_ex ("second"			, bif_second				, BMD_RET_TYPE, &bt_integer	, BMD_IS_PURE, BMD_DONE);
+  bif_define_ex ("is_timezoneless"		, bif_is_timezoneless			, BMD_RET_TYPE, &bt_integer	, BMD_IS_PURE, BMD_DONE);
   bif_define_ex ("timezone"			, bif_timezone				, BMD_RET_TYPE, &bt_integer	, BMD_IS_PURE, BMD_DONE);
+  bif_define_ex("adjust_timezone"		, bif_adjust_timezone			, BMD_RET_TYPE, &bt_datetime	, BMD_IS_PURE, BMD_DONE);
+  bif_define_ex ("forget_timezone"		, bif_forget_timezone			, BMD_RET_TYPE, &bt_datetime	, BMD_IS_PURE, BMD_DONE);
   bif_define_ex ("nasa_tjd_number"		, bif_nasa_tjd_number			, BMD_RET_TYPE, &bt_integer	, BMD_IS_PURE, BMD_DONE);
   bif_define_ex ("nasa_tjd_fraction"		, bif_nasa_tjd_fraction			, BMD_RET_TYPE, &bt_double	, BMD_IS_PURE, BMD_DONE);
   bif_define_ex ("merge_nasa_tjd_to_datetime"	, bif_merge_nasa_tjd_to_datetime	, BMD_RET_TYPE, &bt_datetime	, BMD_IS_PURE, BMD_DONE);
-  bif_define_ex ("now"	, bif_timestamp	, BMD_ALIAS, "getdate"	, BMD_ALIAS, "rdf_now_impl", BMD_ALIAS, "get_timestamp", BMD_RET_TYPE, &bt_timestamp	, /*BMD_IS_PURE,*/ BMD_DONE);	/* This is standard name */
-  bif_define_ex ("curdate"			, bif_curdate				, BMD_RET_TYPE, &bt_date	, /*BMD_IS_PURE,*/ BMD_DONE);	/* This is standard fun. */
-  bif_define_ex ("curtime"			, bif_curtime				, BMD_RET_TYPE, &bt_time	, /*BMD_IS_PURE,*/ BMD_DONE);	/* This is standard fun. */
-  bif_define_ex ("curdatetime"			, bif_curdatetime			, BMD_RET_TYPE, &bt_timestamp	, /*BMD_IS_PURE,*/ BMD_DONE);	/* This is our own. */
+  bif_define_ex ("now"				, bif_timestamp
+						, BMD_ALIAS, "getdate"
+						, BMD_ALIAS, "rdf_now_impl"
+						, BMD_ALIAS, "get_timestamp"
+						, BMD_ALIAS, "current_timestamp" 	, BMD_RET_TYPE, &bt_timestamp	, /*BMD_IS_PURE,*/ BMD_DONE);	/* This is standard name */
+  bif_define_ex ("curdate"			, bif_curdate
+						, BMD_ALIAS, "current_date"		, BMD_RET_TYPE, &bt_date	, /*BMD_IS_PURE, */ BMD_DONE);	/* This is standard fun. */
+  bif_define_ex ("curtime"			, bif_curtime
+						, BMD_ALIAS, "current_time"		, BMD_RET_TYPE, &bt_time	, /*BMD_IS_PURE,*/ BMD_DONE);	/* This is standard fun. */
+  bif_define_ex ("curdatetime"			, bif_curdatetime
+						, BMD_ALIAS, "sysdatetime"		, BMD_RET_TYPE, &bt_timestamp	, /*BMD_IS_PURE,*/ BMD_DONE);	/* curdatetime() is our own, sysdatetime() is MS SQL */
+  bif_define_ex ("curdatetime_tz"		, bif_curdatetime_tz , BMD_RET_TYPE, &bt_timestamp	, /*BMD_IS_PURE,*/ BMD_DONE);
+  bif_define_ex ("curdatetimeoffset"			, bif_curdatetimeoffset
+						, BMD_ALIAS, "sysdatetimeoffset"	, BMD_RET_TYPE, &bt_timestamp	, /*BMD_IS_PURE,*/ BMD_DONE);	/* curdatetimeoffset() is our own, sysdatetimeoffset() is MS SQL */
+  bif_define_ex ("curutcdatetime"			, bif_curutcdatetime
+						, BMD_ALIAS, "sysutcdatetime"		, BMD_RET_TYPE, &bt_timestamp	, /*BMD_IS_PURE,*/ BMD_DONE);	/* curutcdatetime() is our own, sysutcdatetime() is MS SQL */
   bif_define_ex ("datestring"			, bif_date_string			, BMD_RET_TYPE, &bt_varchar	, BMD_IS_PURE, BMD_DONE);
   bif_define_ex ("datestring_GMT"		, bif_date_string_GMT			, BMD_RET_TYPE, &bt_varchar	, BMD_IS_PURE, BMD_DONE);
   bif_define_ex ("stringdate"	, bif_string_timestamp	, BMD_ALIAS, "ts"		, BMD_RET_TYPE, &bt_datetime	, BMD_IS_PURE, BMD_DONE);
