@@ -4,7 +4,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -651,6 +651,17 @@ bif_st_srid (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   return box_num (GEO_SRID(g->geo_srcode));
 }
 
+void
+geo_set_srcode_trav (geo_t *g, int srcode)
+{
+  g->geo_srcode = srcode;
+  if (g->geo_flags & (GEO_A_COMPOUND | GEO_A_RINGS | GEO_A_MULTI | GEO_A_ARRAY))
+    {
+      int ctr, ct = g->_.parts.len;
+      for (ctr = 0; ctr < ct; ctr++)
+        geo_set_srcode_trav (g->_.parts.items[ctr], srcode);
+    }
+}
 
 caddr_t
 bif_st_setsrid (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
@@ -661,7 +672,7 @@ bif_st_setsrid (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
     return NEW_DB_NULL;
   srid = bif_long_arg (qst, args, 1, "st_setsrid");
   cp = (geo_t*)box_copy ((caddr_t)g);
-  cp->geo_srcode = GEO_SRCODE_OF_SRID(srid);
+  geo_set_srcode_trav (cp, GEO_SRCODE_OF_SRID(srid));
   return (caddr_t)cp;
 }
 
@@ -681,8 +692,25 @@ bif_geo_pred (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args, char * f,
   geo_t * g1 = bif_geo_arg (qst, args, 0, f, GEO_ARG_ANY_NULLABLE);
   geo_t * g2 = bif_geo_arg (qst, args, 1, f, GEO_ARG_ANY_NULLABLE);
   double prec = 0;
+  int srid1, srid2;
   if (BOX_ELEMENTS (args) > 2)
     prec = bif_double_arg (qst, args, 2, f);
+  if ((NULL == g1) || (NULL == g2))
+    return box_num (geo_pred (g1, g2, op, prec));
+  srid1 = GEO_SRID (g1->geo_srcode);
+  srid2 = GEO_SRID (g2->geo_srcode);
+  if (srid2 != srid1)
+    {
+      caddr_t err = NULL;
+      caddr_t res;
+      geo_t *g2_cvt = geo_default_srid_transform_cbk (qst, g2, srid1, &err);
+      if (NULL != err)
+        sqlr_resignal (err);
+      res = box_num (geo_pred (g1, g2_cvt, op, prec));
+      if (g2_cvt != g2)
+        dk_free_box ((caddr_t)g2_cvt);
+      return res;
+    }
   return box_num (geo_pred (g1, g2, op, prec));
 }
 
@@ -758,9 +786,6 @@ geo_point_intersects_line (geo_srcode_t srcode, geoc pX, geoc pY, geoc p1X, geoc
 }
 
 #define GEO_SCALAR_PRODUCT(oX,oY,aX,aY,bX,bY) (((aX)-(oX))*((bY)-(oY)) - ((aY)-(oY))*((bX)-(oX)))
-#define GEOC_SWAP(a,b) do { geoc swap = (a); (a) = (b); (b) = swap; } while (0)
-#define GEOC_MAX(a,b) (((a) > (b)) ? (a) : (b))
-#define GEOC_MIN(a,b) (((a) < (b)) ? (a) : (b))
 int
 geo_line_intersects_XYbox (geo_srcode_t srcode, geoc p1X, geoc p1Y, geoc p2X, geoc p2Y, geo_XYbox_t *b, double prec)
 {
@@ -818,7 +843,7 @@ geo_range_intersects_range (geoc p1c, geoc p2c, geoc q3c, geoc q4c, double prec_
 {
   if (p1c > p2c) GEOC_SWAP (p1c, p2c);
   if (q3c > q4c) GEOC_SWAP (q3c, q4c);
-  if (GEOC_MAX (p1c, q3c) + prec_norm <= GEOC_MIN (p2c, q4c))
+  if (geoc_max (p1c, q3c) + prec_norm <= geoc_min (p2c, q4c))
     return 1;
   return 0;
 }
@@ -1159,6 +1184,8 @@ int
 geo_pred (geo_t * g1, geo_t * g2, int op, double prec)
 {
   int cctr, ictr, itemctr;
+  if (!geo_looks_fine (g1) || !geo_looks_fine (g2))
+    sqlr_new_error ("42000", "GEO..", "Bad geo type");
   if (GSOP_WITHIN == op)
     {
       geo_t * swap;
@@ -1491,8 +1518,11 @@ bif_st_ewkt_read (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 {
   geo_t *res;
   caddr_t strg = bif_string_arg (qst, args, 0, "st_ewkt_read");
+  int srid = SRID_DEFAULT;
   caddr_t err = NULL;
-  res = ewkt_parse (strg, &err);
+  if (1 < BOX_ELEMENTS (args))
+    srid = bif_long_arg (qst, args, 1, "st_ewkt_read");
+  res = ewkt_parse_2 (strg, srid, &err);
   if (NULL != err)
     sqlr_resignal (err);
   geo_calc_bounding (res, GEO_CALC_BOUNDING_DO_ALL);
@@ -1839,6 +1869,25 @@ bif_st_transform_by_custom_projection (caddr_t * qst, caddr_t * err_ret, state_s
   return (caddr_t) res;
 }
 
+geo_t *
+geo_dummy_srid_transform_cbk (caddr_t *qst, geo_t *g, int dest_srid, caddr_t *err_ret)
+{
+  int orig_srid = GEO_SRID (g->geo_srcode);
+  if (dest_srid == orig_srid)
+    return g;
+  err_ret[0] = srv_make_new_error ("22023", "GEOxx", "The function requires an implicit call of ST_Transform(), because argument with SRID %d should be transformed to SRID %d; please try plugin \"v7proj4\" or similar",
+     orig_srid, dest_srid );
+  return NULL;
+}
+
+geo_srid_transform_cbk_t *geo_default_srid_transform_cbk = geo_dummy_srid_transform_cbk;
+
+void
+geo_set_default_srid_transform_cbk (geo_srid_transform_cbk_t *cbk)
+{
+  geo_default_srid_transform_cbk = cbk;
+}
+
 void
 bif_geo_init ()
 {
