@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -100,7 +100,7 @@ smtp_connect (char * host1, caddr_t * err_ret, caddr_t sender, caddr_t recipient
   if (SER_SUCC != rc)
     {
       PrpcSessionFree (ses);
-      *err_ret = srv_make_new_error ("2E000", "SM002", "Cannot resolve host in smtp_send");
+      *err_ret = srv_make_new_error ("2E000", "SM002", "Cannot resolve host: %s in smtp_send", host1);
       return NULL;
     }
   rc = session_connect (ses->dks_session);
