@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -25,9 +25,6 @@
  *
  */
 
-#ifndef USE_TIDYLIB
-#define OLD_TIDY
-#endif
 
 #include "libutil.h"
 #include "sqlnode.h"
@@ -37,19 +34,14 @@
 #ifndef WIN32
 #define __USE_MISC 1 /* hack for platform.h defining ulong & uint */
 #endif
-#ifdef OLD_TIDY
-#include "Tidy/html.h"
-#else
-#include <tidy/tidy.h>
-#include <tidy/buffio.h>
-#endif
+#include "tidy.h"
+#include "buffio.h"
 #ifndef WIN32
 #undef __USE_MISC
 #endif
 
 static dk_mutex_t *tidy_mtx;
 
-#ifndef OLD_TIDY
 static void * TIDY_CALL
 tidy_malloc (size_t len)
 {
@@ -147,7 +139,6 @@ tidy_parse_config (TidyDoc doc, caddr_t config_str)
   dk_free_box (ses);
   return 0;
 }
-#endif
 
 caddr_t
 bif_tidy_html (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
@@ -156,24 +147,6 @@ bif_tidy_html (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   caddr_t config_input = bif_string_arg (qst, args, 1, "tidy_html");
   caddr_t html_output = NULL;
   int res;
-#ifdef OLD_TIDY
-  tidy_io_t tidy_errout;
-  tidy_errout.tio_data.lm_memblock = NULL;
-  tidy_errout.tio_data.lm_length = 0;
-  tidy_errout.tio_pos = 0;
-  mutex_enter (tidy_mtx);
-  errout = &tidy_errout;
-  res = do_tidy (html_input, config_input, &html_output);
-  errout = NULL;
-  mutex_leave (tidy_mtx);
-  if (NULL != tidy_errout.tio_data.lm_memblock)
-    dk_free_box (tidy_errout.tio_data.lm_memblock);
-  if ((NULL == html_output) || (2 == res))	/* errors */
-    {
-      dk_free_box (html_output);
-      sqlr_new_error ("42000", "HT076", "HTML Tidy failed, try tidy_list_errors(...) to get more information");
-    }
-#else
   TidyBuffer output;
   TidyBuffer errbuf;
   TidyDoc doc;
@@ -233,7 +206,6 @@ bif_tidy_html (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
       dk_free_box (html_output);
       sqlr_new_error ("42000", "HT076", "HTML Tidy failed, try tidy_list_errors(...) to get more information");
     }
-#endif
   return html_output;
 }
 
@@ -243,24 +215,6 @@ bif_tidy_list_errors (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   caddr_t html_input = bif_string_arg (qst, args, 0, "tidy_list_errors");
   caddr_t config_input = bif_string_arg (qst, args, 1, "tidy_list_errors");
   caddr_t errlist = NULL;
-#ifdef OLD_TIDY
-  tidy_io_t tidy_errout;
-  tidy_errout.tio_data.lm_memblock = NULL;
-  tidy_errout.tio_data.lm_length = 0;
-  tidy_errout.tio_pos = 0;
-  mutex_enter (tidy_mtx);
-  errout = &tidy_errout;
-  do_tidy (html_input, config_input, null);
-  errout = NULL;
-  mutex_leave (tidy_mtx);
-  if (NULL == tidy_errout.tio_data.lm_memblock)
-    errlist = box_dv_short_string ("");
-  else
-    {
-      errlist = box_dv_short_nchars (tidy_errout.tio_data.lm_memblock, tidy_errout.tio_pos);
-      dk_free_box (tidy_errout.tio_data.lm_memblock);
-    }
-#else
   int res = -1;
   TidyBuffer errbuf;
   TidyDoc doc;
@@ -299,18 +253,13 @@ bif_tidy_list_errors (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
      tidyBufFree( &errbuf );
      tidyRelease (doc);
    */
-#endif
   return errlist;
 }
 
 caddr_t
 bif_tidy_external (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 {
-#ifdef OLD_TIDY
-  return box_num (0);
-#else
   return box_num (1);
-#endif
 }
 
 int bif_tidy_init(void)
@@ -319,11 +268,9 @@ int bif_tidy_init(void)
   bif_define ("tidy_html", bif_tidy_html);
   bif_define ("tidy_list_errors", bif_tidy_list_errors);
   bif_define ("tidy_external", bif_tidy_external);
-#ifndef OLD_TIDY
   tidySetMallocCall (tidy_malloc);
   tidySetReallocCall (tidy_realloc);
   tidySetFreeCall (tidy_free);
   tidySetPanicCall (tidy_panic);
-#endif
   return 0;
 }
