@@ -6,7 +6,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -33,6 +33,8 @@ int dbf_ce_insert_mask = 0;
 int
 ceic_all_dtp (ce_ins_ctx_t * ceic, dtp_t dtp)
 {
+
+  int is_32 = DV_LONG_INT == dtp || DV_IRI_ID == dtp;
   it_cursor_t *itc = ceic->ceic_itc;
   dtp_t can_dtp = dtp_canonical[dtp];
   int inx;
@@ -57,9 +59,16 @@ ceic_all_dtp (ce_ins_ctx_t * ceic, dtp_t dtp)
     {
       for (inx = itc->itc_ce_first_set; inx < last; inx++)
 	{
+	  int64 n;
 	  val = (db_buf_t) itc->itc_vec_rds[itc->itc_param_order[inx]]->rd_values[ceic->ceic_nth_col];
 	  if (can_dtp != dtp_canonical[DV_TYPE_OF (val)])
 	    return 0;
+	  if (is_32)
+	    {
+	      n = unbox_iri_int64 (val);
+	      if (IS_64_T (n, can_dtp))
+		return 0;
+	    }
 	}
     }
   ceic->ceic_dtp_checked = 1;
@@ -74,8 +83,14 @@ ceic_all_dtp (ce_ins_ctx_t * ceic, dtp_t dtp)
 #include "cevecins.c"
 
 
+#define INS_NAME ce_insert_vec_int64
+#define ELT_T int64
+#define SET_NA INT64_SET_CA
+#include "cevecins.c"
+
+
 #define VEC_DTP_CK(dtp) \
-  if ((DV_ANY == ceic->ceic_col->col_sqt.sqt_dtp || !ceic->ceic_col->col_sqt.sqt_non_null) && !ceic_all_dtp (ceic, dtp)) \
+  if ((DV_ANY == ceic->ceic_col->col_sqt.sqt_dtp || DV_LONG_INT == dtp || DV_IRI_ID == dtp ||  !ceic->ceic_col->col_sqt.sqt_non_null) && !ceic_all_dtp (ceic, dtp)) \
     goto general;
 
 
@@ -653,9 +668,19 @@ append:
     {
       /* inserts at the end */
       int64 last_value = values[n_deltas - 1];
+      int inx2, skip_bytes = 0;
       if (value < first || value - first > 160)
 	goto no_order;
-      if ((last_value - first) / (n_deltas - nth) > 32)
+      if ((last_value - first) / (n_deltas - nth) > 20)
+	goto no_order;
+      for (inx2 = nth; inx2 < n_deltas - 1; inx2++)
+	{
+	  int delta =  values[inx2 + 1] - values[inx2];
+	  skip_bytes += delta / 15;
+	  if (delta > 15 * 15)
+	    goto no_order; /* skip of more than 15  bytes */
+	}
+      if (skip_bytes > 1000)
 	goto no_order;
       ce_rld_append (ceic, col_ceic, &ce, first, &values[nth], n_deltas - nth, ce_cur[-1], &space_after);
     }
@@ -1262,6 +1287,18 @@ ce_insert_1 (ce_ins_ctx_t * ceic, ce_ins_ctx_t ** col_ceic, db_buf_t ce, int spa
       VEC_DTP_CK (DV_LONG_INT);
       ce2 = ce_insert_vec_int (ceic, col_ceic, ce, space_after, split_at);
       break;
+    case CE_VEC | CE_IS_IRI:
+      VEC_DTP_CK (DV_IRI_ID);
+      ce2 = ce_insert_vec_int (ceic, col_ceic, ce, space_after, split_at);
+      break;
+    case CE_VEC | CE_IS_64:
+      VEC_DTP_CK (DV_INT64);
+      ce2 = ce_insert_vec_int64 (ceic, col_ceic, ce, space_after, split_at);
+      break;
+    case CE_VEC | CE_IS_IRI | CE_IS_64:
+      VEC_DTP_CK (DV_IRI_ID_8);
+      ce2 = ce_insert_vec_int64 (ceic, col_ceic, ce, space_after, split_at);
+      break;
     case CE_ALL_VARIANTS (CE_RL):
       ce2 = ce_insert_rl (ceic, col_ceic, ce, space_after, split_at);
       break;
