@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -339,6 +339,7 @@ qn_size (data_source_t * qn)
   QNSZ (ddl_node_input, ddl_node_t);
   QNSZ (op_node_input, op_node_t);
   QNSZ (breakup_node_input, breakup_node_t);
+  QNSZ (txs_input, text_node_t);
   return -1;
 }
 
@@ -393,7 +394,6 @@ kpd_free (key_partition_def_t * kpd)
   dk_free ((caddr_t)kpd, sizeof (key_partition_def_t));
 }
 
-
 void
 qr_free (query_t * qr)
 {
@@ -1566,7 +1566,8 @@ qr_no_copy_ssls (query_t * qr, dk_hash_t * no_copy)
 	  hash_area_t * ha = setp->setp_ha;
 	  if (ha && (HA_DISTINCT == ha->ha_op || HA_GROUP == ha->ha_op ||HA_ORDER == ha->ha_op))
 	    sethash ((void*)ha->ha_tree, no_copy, (void*)1);
-	  sethash ((void*)setp->setp_sorted, no_copy, (void*) 1);
+	  if (setp->setp_sorted)
+	    sethash ((void*)setp->setp_sorted, no_copy, (void*) 1);
 	}
     }
   END_DO_SET();
@@ -1651,7 +1652,7 @@ table_source_create (
     char *from_position)
 {
   search_spec_t **sps = ts_preds_to_sps (cc, preds);
-  dbe_table_t *table = sch_name_to_table (cc->cc_schema, table_name);
+  dbe_table_t *table = sch_name_to_table (wi_inst.wi_schema, table_name);
   dbe_key_t *order_key;
   dbe_key_t *main_key;
 
@@ -1755,7 +1756,7 @@ ins_col_slot (comp_context_t * cc, insert_node_t * ins, oid_t col_id)
 	}
     }
   END_DO_BOX;
-  col = sch_id_to_column (cc->cc_schema, col_id);
+  col = sch_id_to_column (wi_inst.wi_schema, col_id);
   return (ssl_new_constant (cc, col->col_default));
 }
 
@@ -1861,7 +1862,7 @@ insert_node_create (comp_context_t * cc, char *tb_name,
   int inx;
   oid_t *col_ids = (oid_t *) dk_alloc_box (box_length ((caddr_t) col_names),
       DV_ARRAY_OF_LONG);
-  dbe_table_t *table = sch_name_to_table (cc->cc_schema, tb_name);
+  dbe_table_t *table = sch_name_to_table (wi_inst.wi_schema, tb_name);
   NEW_VARZ (insert_node_t, ins);
   if (!table)
     sqlc_new_error (cc, "42S02", "SQ051", "No table %s.", tb_name);
@@ -2174,7 +2175,7 @@ update_node_compile (comp_context_t * cc,
   if (len & 1)
     sqlc_new_error (cc, "21S01", "SQ055", "Odd assignment list for update.");
 
-  tb = sch_name_to_table (cc->cc_schema, table);
+  tb = sch_name_to_table (wi_inst.wi_schema, table);
   if (!tb)
     sqlc_new_error (cc, "42S02", "SQ056", "No table %s in update.", table);
 
@@ -2224,7 +2225,7 @@ key_insert_compile (comp_context_t * cc,
     caddr_t * stmt, data_source_t ** head_ret,
     data_source_t ** tail_ret)
 {
-  dbe_key_t *key = sch_table_key (cc->cc_schema, stmt[2], stmt[3], 0);
+  dbe_key_t *key = sch_table_key (wi_inst.wi_schema, stmt[2], stmt[3], 0);
   NODE_INIT (key_insert_node_t, ins, key_insert_node_input, NULL);
 
   if (!key)
@@ -2649,7 +2650,7 @@ retry_dupe_check:
 	    prec = ROW_MAX_COL_BYTES;
 	  if (IS_WIDE_STRING_DTP (dtp) && !prec)
 	    prec = ROW_MAX_COL_BYTES;
-	  if (sl->ssl_type == SSL_COLUMN)
+	  if (sl->ssl_type == SSL_COLUMN || (SSL_VEC == sl->ssl_type && sl->ssl_column))
 	    {
 	      dbe_column_t *col = sl->ssl_column;
 	      long flags = 0;
@@ -2737,7 +2738,7 @@ eql_compile_eql (const char *string, client_connection_t * cli, caddr_t * err)
 
     if (cli->cli_user && !sec_user_has_group (0, cli->cli_user->usr_g_id))
       {
-	sqlc_new_error (&cc, "42000", "SQ060", "Must be in dba group to use EQL.");
+	sqlc_new_error (&cc, "42000", "SQ060:SECURITY", "Must be in DBA group to use EQL.");
       }
     eql_stmt_comp (&cc, (caddr_t) text, &head, &tail);
 
@@ -2779,10 +2780,10 @@ eql_compile_2 (const char *string, client_connection_t * cli, caddr_t * err,
       client_connection_t *old_cli = sqlc_client ();
       if (!parse_mtx)
 	parse_mtx = mutex_allocate ();
-      mutex_enter (parse_mtx);
+      parse_enter ();
       sqlc_set_client (cli);
       qr = eql_compile_eql (string, cli, err);
-      mutex_leave (parse_mtx);
+      parse_leave ();
       sqlc_set_client (old_cli);
       return qr;
     }
