@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -62,7 +62,7 @@ int ext_free_count (extent_t * ext);
 extent_t * em_new_extent (extent_map_t * em, int type, dp_addr_t extends);
 dp_addr_t em_try_get_dp (extent_map_t * em, int pg_type, dp_addr_t near);
 dp_addr_t em_new_dp_1 (extent_map_t * em, int ext_type, dp_addr_t near);
-
+extern int32 c_dense_page_allocation;
 
 int
 fd_extend (dbe_storage_t * dbs, int fd, int n_pages)
@@ -1097,7 +1097,7 @@ em_new_dp (extent_map_t * em, int type, dp_addr_t near, int * hold)
   else if (EXT_BLOB == type)
     dp = em_new_blob (em, near);
   else
-    dp = em_new_dp_1 (em, type, near);
+    dp = em_new_dp_1 (em, type, c_dense_page_allocation ? DP_ANY : near);
   if (EXT_INDEX == type && em != em->em_dbs->dbs_extent_map)
     sethash (DP_ADDR2VOID(dp), em->em_uninitialized, (void*) 1);
   em_printf ((" alloc L=%d t=%d\n", dp, type));
@@ -1500,6 +1500,28 @@ em_compact (extent_map_t * em, int free_em)
   em->em_last_blob_ext = NULL;
   DO_EXT (ext, em)
     {
+      if (free_em)
+	{
+	  dp_addr_t dp;
+	  for (dp = ext->ext_dp; dp < ext->ext_dp + EXTENT_SZ; dp += 32)
+	    {
+	      uint32* array;
+	      int inx, bit;
+	      dp_addr_t array_page;
+	      IN_DBS (em->em_dbs);
+	      dbs_locate_incbackup_bit (em->em_dbs, dp, &array, &array_page, &inx, &bit);
+	      array[inx] = 0;
+	      page_set_checksum_init (array);
+	      if (EXT_INDEX == EXT_TYPE (ext))
+		{
+		  /* when dropping a column extent map in a drop index, cpt remaps are possible, so if any, drop them. The remap pagge is dropped anyway as part of the em */
+		  dp_addr_t dp2;
+		  for (dp2 = dp; dp2 < dp + 32; dp2++)
+		    remhash (DP_ADDR2VOID (dp2), em->em_dbs->dbs_cpt_remap);
+		}
+	      LEAVE_DBS (em->em_dbs);
+	    }
+	}
       if (EXT_FREE == EXT_TYPE (ext))
 	{
 	  if (ext == (extent_t*)gethash (DP_ADDR2VOID (ext->ext_dp), em->em_dp_to_ext))
@@ -1674,6 +1696,15 @@ dbs_cpt_recov_write_extents (dbe_storage_t * dbs)
 	  em_save_dp (it->it_extent_map);
 	  dbs_recov_write_page_set (dbs, it->it_extent_map->em_buf);
 	}
+      if (it->it_col_extent_maps)
+	{
+	  DO_HT (ptrlong, col_id, extent_map_t *, em, it->it_col_extent_maps)
+	    {
+	      em_save_dp (em);
+	      dbs_recov_write_page_set (dbs, em->em_buf);
+	    }
+	  END_DO_HT;
+	}
     }
   END_DO_SET();
   em_save_dp (dbs->dbs_extent_map);
