@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -1110,7 +1110,7 @@ int
 itc_up_transit (it_cursor_t * itc, buffer_desc_t ** buf_ret)
 {
   buffer_desc_t * parent_buf = NULL;
-  int up_ctr = 0;
+  int up_ctr = 0, deld_ctr = 0;
   dp_addr_t up;
   volatile dp_addr_t * up_field;
   up_field = (dp_addr_t *) ((*buf_ret)->bd_buffer + DP_PARENT);
@@ -1133,6 +1133,14 @@ up_again:
     page_wait_access (itc, up, *buf_ret, &parent_buf, ITC_LANDED_PA (itc), RWG_WAIT_NO_ENTRY_IF_WAIT);
   /* the no entry case applies only to a wait. Also, if there is a wait, the original buffer will stay occupied by this itc.
   * So if we got no buffer, there was a wait and if we got one the transit had no wait but could have disk read. */
+    if (PF_OF_DELETED == parent_buf)
+      {
+	deld_ctr++;
+	log_info ("up transit to deleted parent %d on %s", up, itc->itc_insert_key->key_name);
+	if (deld_ctr > 2)
+	  return DVC_INDEX_END;
+	goto up_again;
+      }
   if (!parent_buf)
     {
       up_ctr ++;
@@ -1276,7 +1284,7 @@ it_new_root_image (index_tree_t * tree, buffer_desc_t * buf)
   memcpy (new_image->bd_buffer, buf->bd_buffer, MIN (buf->bd_content_map->pm_filled_to + MAX_KV_GAP_BYTES, PAGE_SZ));
   new_image->bd_tree = tree;
   /* keep the actual size of the new map.  It may be that the old map's size if greater than the size implied by its count */
-  new_image->bd_content_map = resource_get (PM_RC (PM_SIZE (buf->bd_content_map->pm_count)));
+  new_image->bd_content_map = pm_get (new_image, (PM_SIZE (buf->bd_content_map->pm_count)));
   new_map_sz = new_image->bd_content_map->pm_size;
   memcpy (new_image->bd_content_map, buf->bd_content_map, PM_ENTRIES_OFFSET + buf->bd_content_map->pm_count * sizeof (short));
   new_image->bd_content_map->pm_size = new_map_sz;
@@ -1316,6 +1324,7 @@ DBGP_NAME (itc_reset) (DBGP_PARAMS it_cursor_t * it)
   dp_addr_t dp, back_link;
   it->itc_landed = 0;
   it->itc_prev_split_search_res = 0;
+  it->itc_split_search_res = 0;
   it->itc_bm_insert = 0;
   it->itc_siblings_parent = 0;
   if (it->itc_keep_right_leaf)
