@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -185,6 +185,12 @@ typedef struct lt_cl_branch_s {
 #endif
 
 
+typedef struct log_merge_s
+{
+  dk_session_t *	lm_log;
+  dk_set_t	lm_blob_log;
+} log_merge_t;
+
 typedef struct lock_trx_s
   {
     char			lt_status;
@@ -212,6 +218,7 @@ typedef struct lock_trx_s
     unsigned int	lt_trace_ctr;
     int	lt_line[LT_TRACE_SZ];
 #endif
+    char		lt_has_branches;
     dk_hash_t 		lt_lock;
 #define lt_has_locks(lt) ((lt)->lt_lock.ht_count)
     /* all below members are considered data area and cleared with memset in lt_cleare, saving individual ones as needed */
@@ -238,6 +245,7 @@ typedef struct lock_trx_s
     int			lt_log_fd;
     caddr_t		lt_log_name;
     dk_set_t		lt_blob_log; /* pdl of blob start addresses to log. Zero if overwritten in the same trx */
+    dk_set_t	lt_log_merge;
     char		lt_timestamp[DT_LENGTH];
     char		lt_approx_dt[DT_LENGTH];
     char		lt_mt_waits; /* if branch or main lt of mt txn with writing branches with same rc w id.  Do not remove waits from wait graph, could be waiting on many threads */
@@ -311,6 +319,7 @@ typedef struct lock_trx_s
 #define IS_MT_BRANCH(lt)  ((lt)->lt_rc_w_id && (lt)->lt_rc_w_id != (lt)->lt_w_id)
 
 #define LT_MAIN_W_ID(lt) ((lt)->lt_rc_w_id ? (lt)->lt_rc_w_id : (lt)->lt_w_id)
+#define LT_MAIN_TRX_NO(lt) ((lt)->lt_main_trx_no ? (lt)->lt_main_trx_no : (lt)->lt_trx_no)
 
 #define LTN_HOST(ltn) ((uint32)((ltn) >> 32))
 #define LTN_NO(ltn) ((uint32)((ltn) & 0xffffffff))
@@ -885,10 +894,12 @@ extern resource_t * rb_page_rc;
 #if defined (PAGE_TRACE) || 0 /* off unless page trace */
 #define rdbg_printf(a) printf a
 #define rdbg_printf_2(a) printf a
+#define rdbg_printf_if(c, a) if (c) printf a
 
 #else
 #define rdbg_printf(a)
 #define rdbg_printf_2(a)
+#define rdbg_printf_if(c, a)
 
 #endif
 
@@ -905,6 +916,8 @@ lock_trx_t * itc_main_lt (it_cursor_t * itc, buffer_desc_t * buf);
 lock_trx_t * lt_main_lt (lock_trx_t * lt);
 int lt_has_delta (lock_trx_t * lt);
 int lt_set_is_branch (dk_set_t list, lock_trx_t * lt, lock_trx_t ** main_lt_ret);
+void log_merge_commit (lock_trx_t * lt, dk_set_t merges);
+void lt_free_merge (dk_set_t merges);
 int lt_log_merge (lock_trx_t * lt, int in_txn);
 #define NO_LOCK_LT ((lock_trx_t*)-1L)
 
