@@ -6,7 +6,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -119,7 +119,7 @@ sqlc_col_table_1 (sql_comp_t * sc, ST * col_ref, dbe_column_t ** col_ret,
   char *col_prefix;
   int inx;
 
-  if (!ST_P (col_ref, COL_DOTTED))
+  if (!ST_COLUMN (col_ref, COL_DOTTED))
     return NULL;
 
   col_prefix = col_ref->_.col_ref.prefix;
@@ -153,7 +153,7 @@ sqlc_col_table_1 (sql_comp_t * sc, ST * col_ref, dbe_column_t ** col_ret,
       if (ct_found && n_found == 1 && ct_col_ref (sc, ct_found, col_ref, col_ret, crr_ret, err_if_not))
 	return ct_found;
 
-      prefix_table = sch_name_to_table (sc->sc_cc->cc_schema,
+      prefix_table = sch_name_to_table (wi_inst.wi_schema,
 	  col_ref->_.col_ref.prefix);
       DO_BOX (comp_table_t *, ct, inx, sc->sc_tables)
       {
