@@ -1,9 +1,8 @@
-/* A Bison parser, made by GNU Bison 2.4.3.  */
+/* A Bison parser, made by GNU Bison 2.7.  */
 
-/* Skeleton interface for Bison's Yacc-like parsers in C
+/* Bison interface for Yacc-like parsers in C
    
-      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
-   2009, 2010 Free Software Foundation, Inc.
+      Copyright (C) 1984, 1989-1990, 2000-2012 Free Software Foundation, Inc.
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -31,6 +30,15 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
+#ifndef YY_NQYY_NQUAD_P_H_INCLUDED
+# define YY_NQYY_NQUAD_P_H_INCLUDED
+/* Enabling traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 0
+#endif
+#if YYDEBUG
+extern int nqyydebug;
+#endif
 
 /* Tokens.  */
 #ifndef YYTOKENTYPE
@@ -68,26 +76,29 @@
      _AT_prefix_L = 285,
      _AT_this_L = 286,
      _MINUS_INF_L = 287,
-     INF_L = 288,
-     NaN_L = 289,
-     false_L = 290,
-     true_L = 291,
-     __TTL_PUNCT_END = 292,
-     __TTL_NONPUNCT_START = 293,
-     TURTLE_INTEGER = 294,
-     TURTLE_DECIMAL = 295,
-     TURTLE_DOUBLE = 296,
-     TURTLE_STRING = 297,
-     KEYWORD = 298,
-     LANGTAG = 299,
-     QNAME = 300,
-     QNAME_NS = 301,
-     VARIABLE = 302,
-     BLANK_NODE_LABEL = 303,
-     Q_IRI_REF = 304,
-     _GARBAGE_BEFORE_DOT_WS = 305,
-     TTL_RECOVERABLE_ERROR = 306,
-     __NQUAD_NONPUNCT_END = 307
+     BASE_L = 288,
+     INF_L = 289,
+     NaN_L = 290,
+     PREFIX_L = 291,
+     false_L = 292,
+     true_L = 293,
+     __TTL_PUNCT_END = 294,
+     __TTL_NONPUNCT_START = 295,
+     TURTLE_INTEGER = 296,
+     TURTLE_DECIMAL = 297,
+     TURTLE_DOUBLE = 298,
+     TURTLE_STRING = 299,
+     KEYWORD = 300,
+     LANGTAG = 301,
+     QNAME = 302,
+     QNAME_NS = 303,
+     VARIABLE = 304,
+     BLANK_NODE_LABEL_NQ = 305,
+     BLANK_NODE_LABEL_TTL = 306,
+     Q_IRI_REF = 307,
+     _GARBAGE_BEFORE_DOT_WS = 308,
+     TTL_RECOVERABLE_ERROR = 309,
+     __NQUAD_NONPUNCT_END = 310
    };
 #endif
 /* Tokens.  */
@@ -121,35 +132,36 @@
 #define _AT_prefix_L 285
 #define _AT_this_L 286
 #define _MINUS_INF_L 287
-#define INF_L 288
-#define NaN_L 289
-#define false_L 290
-#define true_L 291
-#define __TTL_PUNCT_END 292
-#define __TTL_NONPUNCT_START 293
-#define TURTLE_INTEGER 294
-#define TURTLE_DECIMAL 295
-#define TURTLE_DOUBLE 296
-#define TURTLE_STRING 297
-#define KEYWORD 298
-#define LANGTAG 299
-#define QNAME 300
-#define QNAME_NS 301
-#define VARIABLE 302
-#define BLANK_NODE_LABEL 303
-#define Q_IRI_REF 304
-#define _GARBAGE_BEFORE_DOT_WS 305
-#define TTL_RECOVERABLE_ERROR 306
-#define __NQUAD_NONPUNCT_END 307
-
+#define BASE_L 288
+#define INF_L 289
+#define NaN_L 290
+#define PREFIX_L 291
+#define false_L 292
+#define true_L 293
+#define __TTL_PUNCT_END 294
+#define __TTL_NONPUNCT_START 295
+#define TURTLE_INTEGER 296
+#define TURTLE_DECIMAL 297
+#define TURTLE_DOUBLE 298
+#define TURTLE_STRING 299
+#define KEYWORD 300
+#define LANGTAG 301
+#define QNAME 302
+#define QNAME_NS 303
+#define VARIABLE 304
+#define BLANK_NODE_LABEL_NQ 305
+#define BLANK_NODE_LABEL_TTL 306
+#define Q_IRI_REF 307
+#define _GARBAGE_BEFORE_DOT_WS 308
+#define TTL_RECOVERABLE_ERROR 309
+#define __NQUAD_NONPUNCT_END 310
 
 
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
 {
-
-/* Line 1685 of yacc.c  */
+/* Line 2058 of yacc.c  */
 #line 76 "./nquad_p.y"
 
   caddr_t box;
@@ -157,9 +169,8 @@ typedef union YYSTYPE
   void *nothing;
 
 
-
-/* Line 1685 of yacc.c  */
-#line 163 "nquad_p.h"
+/* Line 2058 of yacc.c  */
+#line 174 "nquad_p.h"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
@@ -167,5 +178,18 @@ typedef union YYSTYPE
 #endif
 
 
+#ifdef YYPARSE_PARAM
+#if defined __STDC__ || defined __cplusplus
+int nqyyparse (void *YYPARSE_PARAM);
+#else
+int nqyyparse ();
+#endif
+#else /* ! YYPARSE_PARAM */
+#if defined __STDC__ || defined __cplusplus
+int nqyyparse (ttlp_t * ttlp_arg, yyscan_t yyscanner);
+#else
+int nqyyparse ();
+#endif
+#endif /* ! YYPARSE_PARAM */
 
-
+#endif /* !YY_NQYY_NQUAD_P_H_INCLUDED  */
