@@ -6,7 +6,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -896,11 +896,14 @@ buf_row_compare (buffer_desc_t * buf1, int i1, buffer_desc_t * buf2, int i2, int
       caddr_t b = page_copy_col (buf2, row2, cl, NULL);
       int res = page_col_cmp (buf1, row1, cl, b);
       if (DVC_GREATER == res && is_assert)
-	    GPF_T1 ("out of order");
-	  dk_free_box (b);
-	  if (res != DVC_MATCH)
-	    return res;
+	{
+	  log_error ("Error on key %s, out of order", buf1->bd_tree->it_key->key_name);
+	  GPF_T1 ("out of order");
 	}
+      dk_free_box (b);
+      if (res != DVC_MATCH)
+	return res;
+    }
   END_DO_BOX;
   return DVC_MATCH;
 }
@@ -1104,6 +1107,7 @@ page_col_cmp_1 (buffer_desc_t * buf, db_buf_t row, dbe_col_loc_t * cl, caddr_t v
 	{
 	  while (1)
 	    {
+              wchar_t xlat1, xlat2;
 	      if (inx == l1)
 		{
 		  if (inx == l2)
@@ -1113,11 +1117,11 @@ page_col_cmp_1 (buffer_desc_t * buf, db_buf_t row, dbe_col_loc_t * cl, caddr_t v
 		}
 	      if (inx == l2)
 		return DVC_GREATER;
-	      if (collation->co_table[(unsigned char)dv1[inx]] <
-		  collation->co_table[(unsigned char)dv2[inx]])
+              xlat1 = COLLATION_XLAT_NARROW (collation, (unsigned char)dv1[inx]);
+              xlat2 = COLLATION_XLAT_NARROW (collation, (unsigned char)dv2[inx]);
+              if (xlat1 < xlat2)
 		return DVC_LESS;
-	      if (collation->co_table[(unsigned char)dv1[inx]] >
-		  collation->co_table[(unsigned char)dv2[inx]])
+	      if (xlat1 > xlat2)
 		return DVC_GREATER;
 	      inx++;
 	    }
@@ -1826,7 +1830,7 @@ pf_rd_insert_1 (page_fill_t * pf, row_delta_t * rd)
     }
   place = pf->pf_current->bd_content_map->pm_filled_to;
   pf_shift_compress (pf, rd, NULL);
-  map_insert_pos (&pf->pf_current->bd_content_map, rd->rd_map_pos, pm->pm_filled_to);
+  map_insert_pos (pf->pf_current, &pf->pf_current->bd_content_map, rd->rd_map_pos, pm->pm_filled_to);
   pm = pf->pf_current->bd_content_map;
   pm->pm_filled_to += ROW_ALIGN (len);
   page_write_gap (pf->pf_current->bd_buffer + pm->pm_filled_to, PAGE_SZ - pm->pm_filled_to);
@@ -1894,7 +1898,7 @@ pf_rd_append (page_fill_t * pf, row_delta_t * rd, row_size_t * split_after)
       if (pf->pf_is_autocompact)
 	{
 	  extend = buffer_allocate (DPF_INDEX);
-	  extend->bd_content_map = resource_get (PM_RC (PM_SZ_1));
+	  extend->bd_content_map = pm_get (extend, (PM_SZ_1));
 	  pg_map_clear (extend);
 	  extend->bd_tree = pf->pf_itc->itc_tree;
 	}
@@ -1946,7 +1950,7 @@ pf_rd_append (page_fill_t * pf, row_delta_t * rd, row_size_t * split_after)
 	IE_SET_FLAGS (rf.rf_row, IEF_DELETE);
       if (len != ROW_ALIGN (len))
 	rf.rf_row[len] = 0; /* 0 for compression */
-      map_insert_pos (&pf->pf_current->bd_content_map, pm->pm_count, pm->pm_filled_to);
+      map_insert_pos (pf->pf_current, &pf->pf_current->bd_content_map, pm->pm_count, pm->pm_filled_to);
       pm = pf->pf_current->bd_content_map;
       pf_rd_move_and_lock (pf, rd);
       if (rd->rd_make_ins_rbe && RD_INSERT == rd->rd_op)
@@ -2228,7 +2232,7 @@ pf_change_org (page_fill_t * pf)
   if (t_buf->bd_content_map->pm_count > org->bd_content_map->pm_size)
     {
       int new_sz = PM_SIZE (t_buf->bd_content_map->pm_count);
-      map_resize (&org->bd_content_map, new_sz);
+      map_resize (org, &org->bd_content_map, new_sz);
     }
   org_sz = org->bd_content_map->pm_size;
   memcpy_16 (org->bd_content_map, t_buf->bd_content_map, PM_ENTRIES_OFFSET + t_buf->bd_content_map->pm_count * sizeof (short));
@@ -2423,7 +2427,7 @@ page_apply_1 (it_cursor_t * itc, buffer_desc_t * buf, int n_delta, row_delta_t *
 	  }
 	else if (irow < org_pm->pm_count)
 	  {
-	    if (itc->itc_insert_key->key_no_compression)
+	    if (itc->itc_insert_key->key_no_compression && !itc->itc_insert_key->key_is_col)
 	      page_whole_row (buf, irow, rd);
 	    else
 	    page_row (buf, irow, rd, 0);
@@ -2498,7 +2502,11 @@ page_apply_1 (it_cursor_t * itc, buffer_desc_t * buf, int n_delta, row_delta_t *
 void
 page_apply_s (it_cursor_t * itc, buffer_desc_t * buf, int n_delta, row_delta_t ** delta, int op)
 {
+#if defined (VALGRIND)
+  page_apply_frame_t paf = {0};
+#else
   page_apply_frame_t paf;
+#endif
   page_apply_1 (itc, buf, n_delta, delta, op, &paf);
 }
 
@@ -2507,8 +2515,10 @@ void
 page_apply (it_cursor_t * itc, buffer_desc_t * buf, int n_delta, row_delta_t ** delta, int op)
 {
   du_thread_t * self = THREAD_CURRENT_THREAD;
+#ifdef DEBUG
   if (THR_IS_STACK_OVERFLOW (self, &self, PAGE_SZ + 1000 * sizeof (caddr_t)))
     GPF_T1 ("page_apply called with not enough stack");
+#endif
   if (THR_IS_STACK_OVERFLOW (self, &self, sizeof (page_apply_frame_t) + PAGE_SZ + 1000 * sizeof (caddr_t)))
     {
       NEW_VAR (page_apply_frame_t, paf);
