@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -180,7 +180,7 @@ blob_ref_check (db_buf_t xx, int len, it_cursor_t * itc, dtp_t col_dtp)
     }
   else if (DV_LONG_WIDE == *xx || DV_WIDE == *xx)
     {
-      return box_utf8_as_wide_char ((caddr_t) xx + 1, NULL, len - 1, 0, DV_LONG_WIDE);
+      return box_utf8_as_wide_char ((caddr_t) xx + 1, NULL, len - 1, 0);
     }
   else if (DV_BIN == *xx)
     { /* GK: from hash fill (hash_cast in specific */
@@ -377,7 +377,7 @@ page_box_col (it_cursor_t * itc, buffer_desc_t * buf, db_buf_t row, dbe_col_loc_
     case DV_LONG_WIDE:
       {
 	VL;
-	return box_utf8_as_wide_char ((caddr_t) xx, NULL, len, 0, DV_LONG_WIDE);
+	return box_utf8_as_wide_char ((caddr_t) xx, NULL, len, 0);
       }
     case DV_SINGLE_FLOAT:
       FX;
@@ -938,7 +938,7 @@ box_varchar_string (db_buf_t place, size_t len, dtp_t dtp)
 
 
 void
-qst_set_wide_string (caddr_t * state, state_slot_t * sl, db_buf_t data, int len, dtp_t dtp, int isUTF8)
+qst_set_wide_string (caddr_t * state, state_slot_t * sl, db_buf_t data, int len, int isUTF8)
 {
 
 #ifdef QST_DEBUG
@@ -959,20 +959,20 @@ qst_set_wide_string (caddr_t * state, state_slot_t * sl, db_buf_t data, int len,
       if (!isUTF8 && IS_WIDE_STRING_DTP (old_dtp) &&
 	  box_length (old) == len + sizeof (wchar_t))
 	{
-	  box_reuse ((box_t) old, (box_t) data, len + sizeof (wchar_t), dtp);
+	  box_reuse ((box_t) old, (box_t) data, len + sizeof (wchar_t), DV_WIDE);
 	}
       else
 	{
 	  ssl_free_data (sl, old);
 	  *place = isUTF8 ?
-	    box_utf8_as_wide_char ((caddr_t) data, NULL, len, 0, dtp) :
-	    box_wide_char_string ((caddr_t) data, len, dtp);
+	    box_utf8_as_wide_char ((caddr_t) data, NULL, len, 0) :
+	    box_wide_char_string ((caddr_t) data, len);
 	}
     }
   else
     *place = isUTF8 ?
-      box_utf8_as_wide_char ((caddr_t) data, NULL, len, 0, dtp) :
-    box_wide_char_string ((caddr_t) data, len, dtp);
+      box_utf8_as_wide_char ((caddr_t) data, NULL, len, 0) :
+    box_wide_char_string ((caddr_t) data, len);
 #ifdef QST_DEBUG
     }
 #endif
@@ -1193,7 +1193,7 @@ itc_qst_set_column (it_cursor_t * it, buffer_desc_t * buf, dbe_col_loc_t * cl,
     case DV_LONG_WIDE:
       {
 	VLI;
-	qst_set_wide_string (qst, target, xx, len, DV_LONG_WIDE, 1);
+	qst_set_wide_string (qst, target, xx, len, 1);
 	return;
       }
 
@@ -1614,7 +1614,7 @@ row_set_col_cast (caddr_t data, sql_type_t *tsqt, caddr_t *err_ret,
 	  case DV_WIDE:
 	      if (!IS_WIDE_STRING_DTP (dtp))
 		{
-		  res = box_cast_to (qst, data, dtp, DV_LONG_WIDE, 0, 0, err_ret);
+		  res = box_cast_to (qst, data, dtp, DV_WIDE, 0, 0, err_ret);
 		}
 	      break;
 
@@ -1842,7 +1842,7 @@ row_insert_cast (row_delta_t * rd, dbe_col_loc_t * cl, caddr_t data,
       if (err_ret && *err_ret)
 	return;
       ITC_OWNS_PARAM (ins_itc, str);
-      if ((DV_STRING == (dtp_t)str[0] || DV_SHORT_STRING_SERIAL == (dtp_t)str[0]) && tb_is_rdf_quad (key->key_table))
+      if ((DV_STRING == (dtp_t)str[0] || DV_SHORT_STRING_SERIAL == (dtp_t)str[0] || IS_WIDE_STRING_DTP ((dtp_t)str[0])) && tb_is_rdf_quad (key->key_table))
 	{
 	  caddr_t err = srv_make_new_error ("42000",  "RDFST", "Inserting a string into O in RDF_QUAD.  RDF box is expected");
 	  if (err_ret)
@@ -1916,7 +1916,7 @@ row_insert_cast (row_delta_t * rd, dbe_col_loc_t * cl, caddr_t data,
     case DV_LONG_WIDE:
       if (!IS_WIDE_STRING_DTP (dtp))
 	{
-	  wide_str = box_cast_to (ins_itc->itc_out_state, data, dtp, DV_LONG_WIDE, 0, 0, err_ret);
+	  wide_str = box_cast_to (ins_itc->itc_out_state, data, dtp, DV_WIDE, 0, 0, err_ret);
 	  if (ins_itc)
 	    ITC_OWNS_PARAM (ins_itc, wide_str);
 	  if (*err_ret)
@@ -2016,9 +2016,11 @@ convert_dt:
       if (ins_itc)
 	{
 	  caddr_t dt_box = box_copy (data);
-	        DT_SET_FRACTION (dt_box, 0);
-      SET_DT_TYPE_BY_DTP (dt_box, cl->cl_sqt.sqt_col_dtp);
-      ITC_SEARCH_PARAM (ins_itc, dt_box);
+	        /*DT_SET_FRACTION (dt_box, 0);*/
+	  if (DV_TIME == cl->cl_sqt.sqt_dtp)
+	    DT_SET_DAY (dt_box, DAY_ZERO);
+          SET_DT_TYPE_BY_DTP (dt_box, cl->cl_sqt.sqt_col_dtp);
+          ITC_SEARCH_PARAM (ins_itc, dt_box);
 	  ITC_OWNS_PARAM (ins_itc, dt_box);
 	}
       break;
@@ -2157,7 +2159,7 @@ xmltype_in_blob_ok: ;
 	  }
 	if (DV_STRING == dtp)
 	  {
-	    data = box_cast_to (ins_itc->itc_out_state, data, DV_TYPE_OF (data), DV_LONG_WIDE, 0, 0, err_ret);
+	    data = box_cast_to (ins_itc->itc_out_state, data, DV_TYPE_OF (data), DV_WIDE, 0, 0, err_ret);
 	    ITC_OWNS_PARAM (ins_itc, data);
 	    if (*err_ret)
 	      return;
@@ -2813,8 +2815,8 @@ rd_col_change (it_cursor_t * itc, buffer_desc_t * buf, row_delta_t * rd, dbe_col
 	*col_buf_ret = col_buf = it_new_col_page (itc->itc_tree, 0, 0, col);
       pm = col_buf->bd_content_map;
       fill = pm->pm_filled_to;
-      map_append (&col_buf->bd_content_map, fill);
-      map_append (&col_buf->bd_content_map, n_rows);
+      map_append (col_buf, &col_buf->bd_content_map, fill);
+      map_append (col_buf, &col_buf->bd_content_map, n_rows);
       pm = col_buf->bd_content_map;
       pm->pm_bytes_free -= len;
       col_buf->bd_buffer[fill] = CET_ANY | CE_RL;
@@ -2883,6 +2885,7 @@ it_key_col_ddl (index_tree_t *it, dbe_key_t * key, dbe_column_t * col, int is_dr
   dp_addr_t prev_dropped = 0;
   caddr_t err = NULL;
   caddr_t deflt = is_drop ? NULL : box_to_any (col->col_default, &err);
+  int crinx;
   ITC_INIT (itc, NULL, NULL);
   itc_from_it (itc, it);
   itc->itc_insert_key = key;
@@ -2948,6 +2951,12 @@ it_key_col_ddl (index_tree_t *it, dbe_key_t * key, dbe_column_t * col, int is_dr
 	    rd_list_free (rd_array);
 	    if (ceic.ceic_mp)
 	      mp_free (ceic.ceic_mp);
+	    DO_BOX (col_data_ref_t *, cr, crinx, itc->itc_col_refs)
+	      {
+		if (cr)
+		  cr->cr_pages[0].cp_ceic = NULL;
+	      }
+	    END_DO_BOX;
 	    buf = itc_set_by_placeholder (itc, pl);
 	    itc_unregister_inner ((it_cursor_t*)pl, buf, 0);
 	    plh_free (pl);
