@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -203,6 +203,23 @@ setp_mem_insert (setp_node_t * setp, caddr_t * qst, int pos, caddr_t ** arr, int
 
 long setp_top_row_limit = 10000;
 
+
+int
+setp_top_get (caddr_t * inst, state_slot_t * ssl, int deflt)
+{
+  /* top, skip are usually const.  can be exp in which case vector and ssl ref from setp not applicable in reader or mrg ctx, different qi's and different set nos.  Use elt 0 if vec */
+  if (!ssl)
+    return deflt;
+  if (SSL_IS_VEC_OR_REF (ssl))
+    {
+      state_slot_t ts = *ssl;
+      ts.ssl_type = SSL_VEC;
+      return unbox (sslr_qst_get (inst, (state_slot_ref_t*)&ts, 0));
+    }
+  return unbox (qst_get (inst, ssl));
+}
+
+
 void
 setp_mem_sort (setp_node_t * setp, caddr_t * qst, int n_sets, int merge_set)
 {
@@ -210,8 +227,8 @@ setp_mem_sort (setp_node_t * setp, caddr_t * qst, int n_sets, int merge_set)
   int set, prev_set;
   QNCAST (query_instance_t, qi, qst);
   caddr_t ** arr;
-  ptrlong top = unbox (qst_get (qst, setp->setp_top));
-  ptrlong skip = setp->setp_top_skip ? unbox (qst_get (qst, setp->setp_top_skip)) : 0;
+  ptrlong top = setp_top_get (qst, setp->setp_top, 0);
+  ptrlong skip = setp_top_get (qst, setp->setp_top_skip, 0);
   ptrlong fill;
   ptrlong rc, guess, at_or_above, below;
   int skip_only = (top == -1 && skip >= 0 ? 1 : 0);
@@ -336,7 +353,7 @@ setp_node_run (setp_node_t * setp, caddr_t * inst, caddr_t * state, int print_bl
     {
       itc_ha_feed_ret_t ihfr;
       if (!setp->src_gen.src_sets)
-      itc_ha_feed (&ihfr, setp->setp_ha, inst, 0);
+	itc_ha_feed (&ihfr, setp->setp_ha, inst, 0, NULL);
       else
 	{
 	  int set, n_sets;
@@ -350,7 +367,7 @@ setp_node_run (setp_node_t * setp, caddr_t * inst, caddr_t * state, int print_bl
 	  for (set = 0; set < n_sets; set++)
 	    {
 	      qi->qi_set = set;
-	      itc_ha_feed (&ihfr, setp->setp_ha, inst, 0);
+	      itc_ha_feed (&ihfr, setp->setp_ha, inst, 0, NULL);
 	    }
 	}
 	return DVC_MATCH;
@@ -359,7 +376,7 @@ setp_node_run (setp_node_t * setp, caddr_t * inst, caddr_t * state, int print_bl
     {
       itc_ha_feed_ret_t ihfr;
       if (!setp->src_gen.src_out_fill)
-	return DVC_MATCH != itc_ha_feed (&ihfr, setp->setp_ha, inst, 0);
+	return DVC_MATCH != itc_ha_feed (&ihfr, setp->setp_ha, inst, 0, NULL);
       else
 	{
 	  int set, n_sets = QST_INT (inst, setp->src_gen.src_prev->src_out_fill);
@@ -371,7 +388,7 @@ setp_node_run (setp_node_t * setp, caddr_t * inst, caddr_t * state, int print_bl
 	      int match = 0;
 	      qi->qi_set = set;
 	      if (SETP_DISTINCT_NO_OP != setp->setp_distinct)
-	      match = DVC_MATCH == itc_ha_feed (&ihfr, setp->setp_ha, inst, 0);
+		match = DVC_MATCH == itc_ha_feed (&ihfr, setp->setp_ha, inst, 0, NULL);
 	      if (setp->setp_set_op == INTERSECT_ST || setp->setp_set_op == INTERSECT_ALL_ST)
 		match = !match;
 	      if (!match)
@@ -532,6 +549,10 @@ setp_temp_clear (setp_node_t * setp, hash_area_t * ha, caddr_t * qst)
     qst_clr (qst, setp->setp_sorted);
   if (setp->setp_row_ctr)
     qst_clr (qst, setp->setp_row_ctr);
+  if (setp->setp_fill_cha)
+    QST_BOX (caddr_t, qst, setp->setp_fill_cha) = NULL;
+  if (setp->setp_hash_fill_partitioned)
+    QST_BOX (caddr_t, qst, setp->setp_hash_fill_partitioned) = NULL;
 }
 
 
@@ -1025,14 +1046,15 @@ void
 sort_read_vec_input (table_source_t * ts, caddr_t * inst, caddr_t * state)
 {
   QNCAST (query_instance_t, qi, inst);
-  key_source_t * ks = ts->ts_order_ks;
-  setp_node_t * setp = ts->ts_order_ks->ks_from_setp;
+  key_source_t *ks = ts->ts_order_ks;
+  setp_node_t *setp = ts->ts_order_ks->ks_from_setp;
   int n_results = 0, last_set, batch;
-  caddr_t ** arr;
-  ptrlong top = unbox (qst_get (inst, setp->setp_top));
-  ptrlong skip = setp->setp_top_skip ? unbox (qst_get (inst, setp->setp_top_skip)) : 0;
+  caddr_t **arr;
+  ptrlong top = setp_top_get (inst, setp->setp_top, 0);
+  ptrlong skip = setp_top_get (inst, setp->setp_top_skip, 0);
   ptrlong fill;
-  int set, n_sets = QST_INT (inst, ts->src_gen.src_prev->src_out_fill);
+  int set, n_sets = ts->src_gen.src_prev ? QST_INT (inst, ts->src_gen.src_prev->src_out_fill) : qi->qi_n_sets;
+
   if (setp->setp_partitioned)
     {
       top += skip;
@@ -1043,19 +1065,19 @@ sort_read_vec_input (table_source_t * ts, caddr_t * inst, caddr_t * state)
       QST_INT (inst, ts->clb.clb_nth_set) = 0;
       last_set = QST_INT (inst, ts->clb.clb_nth_set) = 0;
     }
- next_batch:
+next_batch:
   batch = QST_INT (inst, ts->src_gen.src_batch_size);
   n_results = 0;
   ks_vec_new_results (ks, inst, NULL);
   last_set = QST_INT (inst, ts->clb.clb_nth_set);
   for (set = last_set; set < n_sets; set++)
     {
-      data_col_t * dc;
-      qi->qi_set = qst_vec_get_int64 (inst, ks->ks_set_no, set);
+      data_col_t *dc;
+      qi->qi_set = ks->ks_set_no ? qst_vec_get_int64 (inst, ks->ks_set_no, set) : 0;
       if (qi->qi_set < 0 || qi->qi_set >= n_sets)
 	{
 	  qi->qi_set = 0;
-	  sqlr_new_error ("MISCI", "SORTI",  "set no in reading top order by out of range..  Reprt the query to support");
+	  sqlr_new_error ("MISCI", "SORTI", "set no in reading top order by out of range..  Reprt the query to support");
 	}
       dc = QST_BOX (data_col_t *, inst, setp->setp_sorted->ssl_index);
       if (dc->dc_n_values <= qi->qi_set)
@@ -1065,68 +1087,91 @@ sort_read_vec_input (table_source_t * ts, caddr_t * inst, caddr_t * state)
       QST_INT (inst, ts->clb.clb_nth_set) = set;
       if (!arr)
 	continue;
-  if (state)
-    QST_INT (inst, ks->ks_pos_in_temp) = skip;
-  for (;;)
-    {
-      int nth = QST_INT (inst, ks->ks_pos_in_temp);
-      int k_inx = 0, inx;
-      if (nth >= fill)
+      if (state)
+	QST_INT (inst, ks->ks_pos_in_temp) = skip;
+      for (;;)
+	{
+	  int nth = QST_INT (inst, ks->ks_pos_in_temp);
+	  int k_inx = 0, inx;
+	  if (nth >= fill)
 	    goto next_set;
-	  DO_BOX (state_slot_t *, ssl, inx, setp->setp_keys_box )
-	{
-	      if (!ts->ts_sort_read_mask[k_inx])
-		{ /* not all sort temp cols may be output cols */
-		  k_inx++;
-		  continue;
-	}
-	      if (SSL_IS_VEC_OR_REF (ssl))
-		dc_append_box (QST_BOX (data_col_t*, inst, ssl->ssl_index), arr[nth][k_inx]);
-	      else
-	{
-	  qst_set (inst, ssl, arr[nth][k_inx]);
-	  arr[nth][k_inx] = NULL;
-		}
-	  k_inx++;
-	}
-      END_DO_BOX;
-      DO_BOX (state_slot_t *, ssl, inx, setp->setp_dependent_box)
-	{
-	      if (!ts->ts_sort_read_mask[k_inx])
-		{
-		  k_inx++;
-		  continue;
-		}
-	      if (SSL_IS_VEC_OR_REF (ssl))
-		dc_append_box (QST_BOX (data_col_t*, inst, ssl->ssl_index), arr[nth][k_inx]);
-	      else
-		{
-	  qst_set (inst, ssl, arr[nth][k_inx]);
-	  arr[nth][k_inx] = NULL;
-		}
-	  k_inx++;
-	}
-      END_DO_BOX;
-	  qn_result ((data_source_t*)ts, inst, set);
-      QST_INT (inst, ks->ks_pos_in_temp) = nth + 1;
+	  if (setp->setp_org_slots)
+	    {
+	      DO_BOX (state_slot_t *, ssl, inx, setp->setp_org_slots)
+	      {
+		if (!ts->ts_sort_read_mask[k_inx])
+		  {		/* not all sort temp cols may be output cols */
+		    k_inx++;
+		    continue;
+		  }
+		if (SSL_IS_VEC_OR_REF (ssl))
+		  dc_append_box (QST_BOX (data_col_t *, inst, ssl->ssl_index), arr[nth][k_inx]);
+		else
+		  {
+		    qst_set (inst, ssl, arr[nth][k_inx]);
+		    arr[nth][k_inx] = NULL;
+		  }
+		k_inx++;
+	      }
+	      END_DO_BOX;
+	    }
+	  else
+	    {
+	      DO_BOX (state_slot_t *, ssl, inx, setp->setp_keys_box)
+	      {
+		if (!ts->ts_sort_read_mask[k_inx])
+		  {		/* not all sort temp cols may be output cols */
+		    k_inx++;
+		    continue;
+		  }
+		if (SSL_IS_VEC_OR_REF (ssl))
+		  dc_append_box (QST_BOX (data_col_t *, inst, ssl->ssl_index), arr[nth][k_inx]);
+		else
+		  {
+		    qst_set (inst, ssl, arr[nth][k_inx]);
+		    arr[nth][k_inx] = NULL;
+		  }
+		k_inx++;
+	      }
+	      END_DO_BOX;
+	      DO_BOX (state_slot_t *, ssl, inx, setp->setp_dependent_box)
+	      {
+		if (!ts->ts_sort_read_mask[k_inx])
+		  {
+		    k_inx++;
+		    continue;
+		  }
+		if (SSL_IS_VEC_OR_REF (ssl))
+		  dc_append_box (QST_BOX (data_col_t *, inst, ssl->ssl_index), arr[nth][k_inx]);
+		else
+		  {
+		    qst_set (inst, ssl, arr[nth][k_inx]);
+		    arr[nth][k_inx] = NULL;
+		  }
+		k_inx++;
+	      }
+	      END_DO_BOX;
+	    }
+	  qn_result ((data_source_t *) ts, inst, set);
+	  QST_INT (inst, ks->ks_pos_in_temp) = nth + 1;
 	  if (++n_results == batch)
 	    {
 	      SRC_IN_STATE (ts, inst) = inst;
 	      QST_INT (inst, ts->clb.clb_nth_set) = set;
 	      ts_always_null (ts, inst);
-	      qn_send_output ((data_source_t*)ts, inst);
+	      qn_send_output ((data_source_t *) ts, inst);
 	      state = NULL;
-	      dc_reset_array (inst, (data_source_t*)ts, ts->src_gen.src_continue_reset, -1);
+	      dc_reset_array (inst, (data_source_t *) ts, ts->src_gen.src_continue_reset, -1);
 	      goto next_batch;
 	    }
-    }
+	}
     next_set:
       QST_INT (inst, ks->ks_pos_in_temp) = 0;
     }
-  SRC_IN_STATE ((data_source_t*)ts, inst) = NULL;
+  SRC_IN_STATE ((data_source_t *) ts, inst) = NULL;
   ts_always_null (ts, inst);
   if (QST_INT (inst, ts->src_gen.src_out_fill))
-    qn_ts_send_output ((data_source_t *)ts, inst, ts->ts_after_join_test);
+    qn_ts_send_output ((data_source_t *) ts, inst, ts->ts_after_join_test);
 }
 
 
