@@ -4,7 +4,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -21,6 +21,7 @@
  *
  */
 
+#include "datesupp.h"
 #include "sparql2sql.h"
 #include "sqlparext.h"
 /*#include "arith.h"
@@ -51,7 +52,19 @@ void ssg_sdprin_literal (spar_sqlgen_t *ssg, SPART *tree)
     case DV_ARRAY_OF_POINTER:
       if (SPAR_LIT != tree->type)
         spar_sqlprint_error ("ssg_" "sdprin_literal: non-literal vector as argument");
-      ssg_sdprin_literal (ssg, (SPART *)(tree->_.lit.val));
+        if (DV_DATETIME == DV_TYPE_OF (tree->_.lit.val))
+          {
+            char temp [100];
+            int mode = DT_PRINT_MODE_XML | dt_print_flags_of_xsd_type_uname (tree->_.lit.datatype);
+            dt_to_iso8601_string_ext (tree->_.lit.val, temp, sizeof (temp), mode);
+            ssg_putchar ('"');
+            ssg_puts (temp);
+            ssg_putchar ('"');
+          }
+        else if (NULL != tree->_.lit.original_text)
+          ssg_puts (tree->_.lit.original_text);
+        else
+          ssg_sdprin_literal (ssg, (SPART *)(tree->_.lit.val));
       if (DV_STRING != DV_TYPE_OF (tree->_.lit.val))
         return;
       if (NULL != tree->_.lit.datatype)
@@ -92,7 +105,7 @@ void ssg_sdprin_literal (spar_sqlgen_t *ssg, SPART *tree)
         return;
       }
     default:
-      ssg_print_box_as_sql_atom (ssg, (caddr_t)tree, SQL_ATOM_UTF8_ONLY);
+      ssg_print_box_as_sql_atom (ssg, (caddr_t)tree, SQL_ATOM_SPARQL_INTEROP);
       return;
     }
 }
@@ -475,11 +488,27 @@ ssg_sdprint_equiv_restrs (spar_sqlgen_t *ssg, sparp_equiv_t *eq)
     }
   if ((SPART_VARR_FIXED & eq->e_rvr.rvrRestrictions) && !(SPART_VARR_FIXED & mixed_field_restr))
     {
+      dtp_t val_dtp = DV_TYPE_OF (eq->e_rvr.rvrFixedValue);
       ssg_newline (0);
       ssg_puts (" FILTER (");
       ssg_sdprin_varname (ssg, eq->e_varnames[0]);
       ssg_puts (" = ");
-      ssg_sdprint_tree (ssg, (SPART *)(eq->e_rvr.rvrFixedValue));
+      if ((NULL != eq->e_rvr.rvrFixedOrigText) && ((DV_DOUBLE_FLOAT == val_dtp) || (DV_SINGLE_FLOAT == val_dtp) || (DV_NUMERIC == val_dtp)))
+        ssg_puts (eq->e_rvr.rvrFixedOrigText);
+      else
+        {
+          ssg_sdprint_tree (ssg, (SPART *)(eq->e_rvr.rvrFixedValue));
+          if ((DV_STRING == DV_TYPE_OF (eq->e_rvr.rvrFixedValue)) && (NULL != eq->e_rvr.rvrDatatype))
+            {
+              ssg_puts ("^^");
+              ssg_sdprint_tree (ssg, (SPART *)(eq->e_rvr.rvrDatatype));
+            }
+          if (NULL != eq->e_rvr.rvrLanguage)
+            {
+              ssg_puts ("@");
+              ssg_puts (eq->e_rvr.rvrLanguage);
+            }
+        }
       ssg_puts (")");
       goto end_builtin_checks;
     }
@@ -490,7 +519,7 @@ ssg_sdprint_equiv_restrs (spar_sqlgen_t *ssg, sparp_equiv_t *eq)
   else if ((SPART_VARR_IS_BLANK & eq->e_rvr.rvrRestrictions) && !(SPART_VARR_IS_BLANK & mixed_field_restr))
     builtin_name = "isBLANK";
   else if ((SPART_VARR_IS_REF & eq->e_rvr.rvrRestrictions) && !(SPART_VARR_IS_REF & mixed_field_restr))
-    builtin_name = ((ssg->ssg_sd_flags & (SSG_SD_BI | SSG_SD_VIRTSPECIFIC)) ? "isREF" : "!isLITERAL");
+    builtin_name = ((ssg->ssg_sd_flags & SSG_SD_VIRTSPECIFIC) ? "isREF" : "!isLITERAL");
   else if ((SPART_VARR_NOT_NULL & eq->e_rvr.rvrRestrictions) && !(SPART_VARR_NOT_NULL & mixed_field_restr))
     builtin_name = "BOUND";
   if (NULL != builtin_name)
@@ -741,11 +770,27 @@ static const char *sparql11aggregates[] = { "AVG", "COUNT", "GROUP_CONCAT", "MAX
                 colon[0] = ':';
               }
           }
+        if (!strcmp (fname, "SQLVAL::_STAR"))
+          {
+            ssg_puts (" *");
+            return;
+          }
         ssg_sdprin_qname (ssg, (SPART *)(fname));
 fname_printed:
         ssg_putchar ('(');
         ssg->ssg_indent++;
+        if (tree->_.funcall.agg_mode)
+          {
+            if (DISTINCT_L == tree->_.funcall.agg_mode)
+              ssg_puts (" DISTINCT");
+            if ((1 == BOX_ELEMENTS (tree->_.funcall.argtrees)) && ((SPART *)((ptrlong)_STAR) == tree->_.funcall.argtrees[0]))
+              {
+                ssg_puts (" *");
+                goto args_printed; /* see below */
+              }
+          }
         ssg_sdprint_tree_list (ssg, tree->_.funcall.argtrees, ',');
+args_printed:
         ssg_putchar (')');
         ssg->ssg_indent--;
         return;
@@ -866,6 +911,8 @@ fname_printed:
                       int valctr;
                       if (!strcmp (name, "lang:dialect"))
                         continue;
+                      if (!strcmp (name, "lang:exceptions"))
+                        continue;
                       ssg_puts (" DEFINE ");
                       ssg_puts (name);
                       DO_BOX_FAST (SPART **, val, valctr, vals)
@@ -1191,6 +1238,8 @@ fname_printed:
           ssg_newline (0);
         if (need_new_graph)
           {
+            if (SSG_SD_NO_GRAPH & ssg->ssg_sd_no)
+              spar_error (ssg->ssg_sparp, "%.100s does not support GRAPH <...> {...} syntax, so SPARQL query is not composed", ssg->ssg_sd_service_naming);
             ssg_puts (" GRAPH");
             ssg_sdprint_tree (ssg, curr_graph);
             ssg_puts (" {");
@@ -1320,7 +1369,7 @@ fname_printed:
         SPART *expn = tree->_.oby.expn;
         switch (tree->_.oby.direction)
           {
-          case ASC_L: ssg_puts (" ASC ("); break;
+          case 0: case ASC_L: ssg_puts (" ASC ("); break;
           case DESC_L: ssg_puts (" DESC ("); break;
           }
         ssg->ssg_indent++;
@@ -1391,21 +1440,52 @@ fname_printed:
       }
     case SPAR_PPATH:
       {
-      int ctr, count = BOX_ELEMENTS (tree->_.ppath.parts);
+        SPART **p = tree->_.ppath.parts;
+        int count = BOX_ELEMENTS (p);
+        int fwd_ctr, ctr, fwdstar, invstar;
       switch (tree->_.ppath.subtype)
         {
-        case '!':
-          ssg_puts (" !");
-          /* no break */
-        case 0:
-          if (1 != count) ssg_puts (" (");
+        case 0: case '!':
+          fwd_ctr = count - tree->_.ppath.num_of_invs;
+          fwdstar = ((0 < fwd_ctr) && ((SPART *)_STAR == p[0]));
+          invstar = ((fwd_ctr < count) && ((SPART *)_STAR == p[fwd_ctr]));
+          if (fwdstar || invstar)
+            {
+              ssg_puts (" (");
+              if (fwdstar)
+                ssg_puts (" <star>|!<star>");
+              if (fwdstar && invstar)
+                ssg_puts ("|");
+              if (invstar)
+                ssg_puts (" ^<star>|!(^<star>)");
+              if ((0 < fwd_ctr) && !fwdstar)
+                {
+                  SPART *tmp = spartlist (ssg->ssg_sparp, 6, SPAR_PPATH, tree->_.ppath.subtype, t_list_memcpy (fwd_ctr, (ccaddr_t *)p), (ptrlong)0, (ptrlong)0, (ptrlong)0);
+                  ssg_puts ("|");
+                  ssg_sdprint_tree (ssg, tmp);
+                }
+              if ((0 < fwd_ctr) && !fwdstar)
+                {
+                  SPART *tmp = spartlist (ssg->ssg_sparp, 6, SPAR_PPATH, tree->_.ppath.subtype, t_list_memcpy (count-fwd_ctr, (ccaddr_t *)(p+fwd_ctr)), (ptrlong)0, (ptrlong)0, t_box_num(count-fwd_ctr));
+                  ssg_puts ("|");
+                  ssg_sdprint_tree (ssg, tmp);
+                }
+              ssg_puts (")");
+              break;
+            }
+          if ('!' == tree->_.ppath.subtype)
+            ssg_puts (" !");
+          if (('!' == tree->_.ppath.subtype) || (1 != count)) ssg_puts (" (");
           for (ctr = 0; ctr < count; ctr++)
             {
+              SPART *part = tree->_.ppath.parts[ctr];
               if (0 < ctr) ssg_puts (" |");
               if ((count - tree->_.ppath.num_of_invs) <= ctr) ssg_puts (" ^");
+              if ((SPART *)_STAR == part)
+                ssg_puts (" <x>|!<x>");
               ssg_sdprint_tree (ssg, tree->_.ppath.parts[ctr]);
             }
-          if (1 != count) ssg_puts (" )");
+          if (('!' == tree->_.ppath.subtype) || (1 != count)) ssg_puts (" )");
           return;
         case 'D':
           ssg_puts (" DISTINCT");
@@ -1414,7 +1494,7 @@ fname_printed:
           ssg_puts (" (");
           for (ctr = 0; ctr < count; ctr++)
             {
-              if (0 < ctr) ssg_puts (" |");
+              if (0 < ctr) ssg_puts ("  | ");
               ssg_sdprint_tree (ssg, tree->_.ppath.parts[ctr]);
             }
           ssg_puts (" )");
