@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -133,7 +133,6 @@ typedef struct rdf_inf_slots_s
   state_slot_t *	ris_o;
 } rdf_inf_slots_t;
 
-
 typedef struct sql_comp_s
   {
     comp_context_t *	sc_cc;
@@ -244,7 +243,8 @@ typedef struct sql_comp_s
     char		sc_re_emit_code; /* in conditional expressions, repeating code must be generated for each branch even if overlap */
     char		sc_is_first_cond; /* true if doing 1st condition in a conditional exp, i.e. will always execute */
     char		sc_delay_colocate; /* do not do cluster colocating qr by qr, the qrs may be modified and only then located */
-    dk_set_t		sc_re_emitted_dfes;
+    dk_set_t		sc_re_emitted_dfes; /* dfes assigned in cond part, not defined after the cond part */
+    dk_set_t		sc_cond_defd_dfes; /* stack for saving the containing cond enps'sc_re_emitted_dfes */
     rdf_inf_slots_t *	sc_rdf_inf_slots;
     caddr_t * sc_big_ssl_consts;	/*!< Vector of saved values for SSL consts of unusual types (like vectors) or just too big to fit into SQL text in a plain way */
     dk_set_t		sc_vec_pred;
@@ -547,6 +547,8 @@ void qr_set_local_code_and_funref_flag (query_t * qr);
 state_slot_t * scalar_exp_generate_typed (sql_comp_t * sc,
     ST * tree, dk_set_t * code, sql_type_t * expect);
 
+void sqlg_cond_start (sql_comp_t * sc);
+void sqlg_cond_end (sql_comp_t * sc);
 
 sql_type_t * sqlc_stmt_nth_col_type (sql_comp_t * sc, dbe_table_t * tb, ST * tree, int nth);
 
