@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -452,7 +452,7 @@ qc_make_update (sql_comp_t * sc, query_cursor_t * qc)
   memset (vals, 0, box_length ((caddr_t) vals));
   DO_BOX (ST *, col_ref, inx, org_sel->_.select_stmt.selection)
   {
-    if (!ST_P (col_ref, COL_DOTTED))
+    if (!ST_COLUMN (col_ref, COL_DOTTED))
       {
 /*	dk_free_tree ((caddr_t) cols);
 	dk_free_tree ((caddr_t) vals);*/
@@ -490,7 +490,7 @@ qc_make_insert (sql_comp_t * sc, query_cursor_t * qc)
   memset (cols, 0, box_length ((caddr_t) vals));
   DO_BOX (ST *, col_ref, inx, org_sel->_.select_stmt.selection)
   {
-    if (!ST_P (col_ref, COL_DOTTED))
+    if (!ST_COLUMN (col_ref, COL_DOTTED))
       {
 /*	dk_free_tree ((caddr_t) cols);
 	dk_free_tree ((caddr_t) vals);*/
