@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -143,6 +143,7 @@ sqlc_call_ret_name (ST * tree, char * func_name, state_slot_t * ssl)
     }
 }
 
+int tn_step_refers_to_input = 0;
 
 state_slot_t *
 sqlc_trans_funcs (sql_comp_t * sc, ST * tree, state_slot_t * ret, dk_set_t * code)
@@ -185,14 +186,26 @@ sqlc_trans_funcs (sql_comp_t * sc, ST * tree, state_slot_t * ret, dk_set_t * cod
 	      sc->sc_trans->tn_step_out = (state_slot_t **)box_copy ((caddr_t)sc->sc_trans->tn_input);
 	      memset (sc->sc_trans->tn_step_out, 0, box_length ((caddr_t)sc->sc_trans->tn_step_out));
 	    }
-	  wanted = sc->sc_trans->tn_out_slots[pos];
-	  if (sc->sc_trans->tn_is_second_in_direction3)
-	    wanted_pos = box_position ((caddr_t *)(sc->sc_trans->tn_output_pos), (caddr_t)pos);
-          else
-	    wanted_pos = box_position ((caddr_t *)(sc->sc_trans->tn_input_pos), (caddr_t)pos);
-	  if (-1 == wanted_pos)
-	    sqlc_new_error (sc->sc_cc, "37000", "TR...", "T_STEP argument refers to an index %d of a column %.200s that is not in %s list (T_DIRECTION is set to %d)", pos+1, wanted->ssl_name, ((TRANS_LR == sc->sc_trans->tn_direction) ? "T_IN" : "T_OUT"), sc->sc_trans->tn_direction);
-	  sc->sc_trans->tn_step_out[wanted_pos] = ret;
+	  if (tn_step_refers_to_input)
+	    {
+	      if (pos < 0 || pos >= BOX_ELEMENTS (sc->sc_trans->tn_step_out))
+		sqlc_new_error (sc->sc_cc, "37000", "TR...",
+		    "tn_step column index out of range of inputs, TransStepMode = input, so tn_step (1) refers to 1st in tn_in ");
+	      sc->sc_trans->tn_step_out[pos] = ret;
+	    }
+	  else
+	    {
+	      wanted = sc->sc_trans->tn_out_slots[pos];
+	      if (sc->sc_trans->tn_is_second_in_direction3)
+		wanted_pos = box_position ((caddr_t *) (sc->sc_trans->tn_output_pos), (caddr_t) pos);
+	      else
+		wanted_pos = box_position ((caddr_t *) (sc->sc_trans->tn_input_pos), (caddr_t) pos);
+	      if (-1 == wanted_pos)
+		sqlc_new_error (sc->sc_cc, "37000", "TR...",
+		    "T_STEP argument refers to an index %d of a column %.200s that is not in %s list (T_DIRECTION is set to %d)", pos + 1,
+		    wanted->ssl_name, ((TRANS_LR == sc->sc_trans->tn_direction) ? "T_IN" : "T_OUT"), sc->sc_trans->tn_direction);
+	      sc->sc_trans->tn_step_out[wanted_pos] = ret;
+	    }
 	  cv_artm (code, (ao_func_t)box_identity, ret, ssl_new_constant (sc->sc_cc, NULL), NULL);
 	  ret->ssl_sqt.sqt_dtp = DV_ARRAY_OF_POINTER;
 	}
@@ -313,14 +326,14 @@ sqlc_call_exp (sql_comp_t * sc, dk_set_t * code, state_slot_t * ret, ST * tree)
 	  sch_split_name (cli_qual (sqlc_client ()),
 	      sc->sc_super->sc_cc->cc_query->qr_proc_name,
               rq, ro, rn);
-	  full_name = sch_full_proc_name_1 (sc->sc_cc->cc_schema, fun_name,
+	  full_name = sch_full_proc_name_1 (wi_inst.wi_schema, fun_name,
 	      cli_qual (sqlc_client ()), CLI_OWNER (sqlc_client ()), rn);
 	}
       else
-	full_name = sch_full_proc_name (sc->sc_cc->cc_schema, fun_name,
+	full_name = sch_full_proc_name (wi_inst.wi_schema, fun_name,
 	    cli_qual (sqlc_client ()), CLI_OWNER (sqlc_client ()));
       if (full_name)
-	proc = sch_proc_def (sc->sc_cc->cc_schema, full_name);
+	proc = sch_proc_def (wi_inst.wi_schema, full_name);
     }
 
   if (!fun_udt_name &&
@@ -409,7 +422,7 @@ state_slot_t *
 sqlc_new_temp (sql_comp_t * sc, const char *name, dtp_t dtp)
 {
   state_slot_t *out;
-  if (sc->sc_cc->cc_query->qr_proc_vectored)
+  if (sc->sc_cc->cc_query && sc->sc_cc->cc_query->qr_proc_vectored)
     return ssl_new_vec (sc->sc_cc, name, dtp);
   if (sc->sc_temp_in_qst)
     {
@@ -455,7 +468,7 @@ sqlg_unplace_ssl (sqlo_t * so, ST * tree)
   int inx;
   if (DV_ARRAY_OF_POINTER != DV_TYPE_OF (tree))
     return;
-  if (ST_P(tree, COL_DOTTED))
+  if (ST_COLUMN(tree, COL_DOTTED))
     return;
   dfe = sqlo_df_elt (so, tree);
   if (dfe && dfe->dfe_ssl)
@@ -533,22 +546,28 @@ sqlc_simple_case (sql_comp_t * sc, ST * tree, dk_set_t * code)
 	  if (inx != n_exps - 2 || n_exps == 3)
 	    sqlc_new_error (sc->sc_cc, "37000", "SQ080", "ELSE must be last clause in CASE.");
 	  SET_PRIVATE_ELTS (sc, dfe, inx + 1);
+	  sqlg_cond_start (sc);
 	  GENERATE_CONTROL_EXP (sc, dfe, inx + 1, code);
 	  val = scalar_exp_generate (sc, then, code);
+	  sqlg_cond_end (sc);
 	  cv_artm (code, (ao_func_t)box_identity, res, val, NULL);
 	  was_else = 1;
 	  RESTORE_PRIVATE_ELTS (sc, dfe);
 	  break;
 	}
       SET_PRIVATE_ELTS (sc, dfe, inx);
+      sqlg_cond_start (sc);
       GENERATE_CONTROL_EXP (sc, dfe, inx, code);
       cres = scalar_exp_generate (sc, cond, code);
+      sqlg_cond_end (sc);
       cv_compare (code, BOP_EQ, sel, cres, ok, next, next);
       cv_label (code, ok);
       RESTORE_PRIVATE_ELTS (sc, dfe);
       SET_PRIVATE_ELTS (sc, dfe, inx + 1);
+      sqlg_cond_start (sc);
       GENERATE_CONTROL_EXP (sc, dfe, inx + 1, code);
       val = scalar_exp_generate (sc, then, code);
+      sqlg_cond_end (sc);
       cv_artm (code, (ao_func_t)box_identity, res, val, NULL);
       cv_jump (code, end);
       cv_label (code, next);
@@ -598,21 +617,27 @@ sqlc_searched_case (sql_comp_t * sc, ST * tree, dk_set_t * code)
 	  if (inx != n_exps - 2 || n_exps == 3)
 	    sqlc_new_error (sc->sc_cc, "37000", "SQ081", "ELSE must be last clause in CASE.");
 	  SET_PRIVATE_ELTS (sc, dfe, inx + 1);
+	  sqlg_cond_start (sc);
 	  GENERATE_CONTROL_EXP (sc, dfe, inx + 1, code);
 	  val = scalar_exp_generate (sc, then, code);
+	  sqlg_cond_end (sc);
 	  cv_artm (code, (ao_func_t)box_identity, res, val, NULL);
 	  was_else = 1;
 	  RESTORE_PRIVATE_ELTS (sc, dfe);
 	  break;
 	}
       SET_PRIVATE_ELTS (sc,dfe,inx);
+      sqlg_cond_start (sc);
       GENERATE_CONTROL_EXP (sc, dfe, inx, code);
+      sqlg_cond_end (sc);
       pred_gen_1 (sc, cond, code, ok, next, next);
       cv_label (code, ok);
       RESTORE_PRIVATE_ELTS (sc, dfe);
       SET_PRIVATE_ELTS (sc,dfe,inx + 1);
+      sqlg_cond_start (sc);
       GENERATE_CONTROL_EXP (sc, dfe, inx + 1, code);
       val = scalar_exp_generate (sc, then, code);
+      sqlg_cond_end (sc);
       cv_artm (code, (ao_func_t)box_identity, res, val, NULL);
       cv_jump (code, end);
       cv_label (code, next);
@@ -650,6 +675,8 @@ sqlc_coalesce_exp (sql_comp_t * sc, ST * tree, dk_set_t * code)
       state_slot_t *cres;
       DEF_PRIVATE_ELTS;
 
+      if (1 == inx)
+	sqlg_cond_start (sc);
       SET_PRIVATE_ELTS (sc, dfe, inx);
       GENERATE_CONTROL_EXP (sc, dfe, inx, code);
       cres = scalar_exp_generate (sc, cond, code);
@@ -663,10 +690,36 @@ sqlc_coalesce_exp (sql_comp_t * sc, ST * tree, dk_set_t * code)
     }
   cv_artm (code, (ao_func_t)box_identity, res, nullc, NULL);
   cv_label (code, end);
+  if (n_exps > 1)
+    sqlg_cond_end (sc);
   return res;
 }
 
 
+void
+sqlg_cond_start (sql_comp_t * sc)
+{
+  /* Assignments in code between this and the matching sqlg_cond_end call are not guaranteed to be defined after the sqlg_cond_end */
+  t_set_push (&sc->sc_cond_defd_dfes, sc->sc_re_emitted_dfes);
+  sc->sc_re_emitted_dfes = NULL;
+  sc->sc_re_emit_code = 1;
+}
+
+
+void
+sqlg_cond_end (sql_comp_t * sc)
+{
+  DO_SET (df_elt_t *, dfe, &sc->sc_re_emitted_dfes)
+    {
+      dfe->dfe_ssl = NULL;
+    }
+  END_DO_SET();
+  sc->sc_re_emitted_dfes = (dk_set_t)t_set_pop (&sc->sc_cond_defd_dfes);
+  if (!sc->sc_cond_defd_dfes)
+    sc->sc_re_emit_code = 0;
+}
+
+
 state_slot_t *
 seg_ret_fn (sql_comp_t * sc, df_elt_t * dfe, state_slot_t * ssl, char is_re_emit)
 {
@@ -689,10 +742,12 @@ scalar_exp_generate (sql_comp_t * sc, ST * tree, dk_set_t * code)
   if (sc->sc_so)
     {
       dfe = sqlo_df (sc->sc_so, tree);
-      if (dfe->dfe_ssl && (!is_re_emit || !dk_set_member (sc->sc_re_emitted_dfes, (void*)dfe)))
+      if (st_is_call (tree, "__ro2lo", 1))
+  bing ();
+
+      if (dfe->dfe_ssl)
 	return (dfe->dfe_ssl);
     }
-  sc->sc_re_emit_code = 0;
   if (SYMBOLP (tree))
     {
       state_slot_t * ssl;
@@ -712,7 +767,7 @@ scalar_exp_generate (sql_comp_t * sc, ST * tree, dk_set_t * code)
     {
       seg_return (ssl_new_constant (sc->sc_cc, (caddr_t) tree->_.op.arg_1));
     }
-  if (ST_P (tree, COL_DOTTED) ||
+  if (ST_COLUMN (tree, COL_DOTTED) ||
       ST_P (tree, FUN_REF))
     {
       state_slot_t * ssl;
@@ -842,9 +897,6 @@ ssl_type_to_name (char ssl_type)
   return "<unnamed>";
 }
 
-#define IS_DATE_DTP(dtp) \
-  (DV_TIMESTAMP == (dtp) || DV_DATE == (dtp) || DV_DATETIME == (dtp))
-
 void
 cv_bop_params (state_slot_t * l, state_slot_t * r, const char *op)
 {
@@ -1363,7 +1415,7 @@ cv_call_set_type (sql_comp_t * sc, instruction_t * ins, query_t *qr_found)
     }
   else
     {
-      query_t *qr = qr_found ? qr_found : sch_proc_def (sc->sc_cc->cc_schema, ins->_.call.proc);
+      query_t *qr = qr_found ? qr_found : sch_proc_def (wi_inst.wi_schema, ins->_.call.proc);
       if (!qr || IS_REMOTE_ROUTINE_QR (qr) || !qr->qr_proc_ret_type)
 	goto generic_box;
       else
@@ -1692,10 +1744,10 @@ sqlo_proc_cl_locatable (caddr_t name, int level, query_t ** qr_ret)
     {
       int is_sem = sqlc_inside_sem;
       if (is_sem)
-	mutex_leave (parse_mtx);
+	parse_leave ();
       tree = (ST*) sql_compile (qr->qr_text, sqlc_client (), &err,  SQLC_PARSE_ONLY);
       if (is_sem)
-	mutex_enter (parse_mtx);
+	parse_enter ();
     }
   END_WITHOUT_TMP_POOL;
   if (err)
@@ -1791,7 +1843,7 @@ cv_is_local_1 (code_vec_t cv, int is_cluster)
 	  if (subq_comp_func == ins->_.pred.func)
 	    {
 	      subq_pred_t * subq = (subq_pred_t *) ins->_.pred.cmp;
-	      if (!is_cluster || (CV_NO_INDEX & is_cluster))
+	      if ((CV_NO_INDEX & is_cluster))
 		return 0;
 	      if (!enable_rec_qf && CV_IS_LOCAL_CN == is_cluster)
 		break;
@@ -2500,7 +2552,10 @@ code_to_cv_1 (sql_comp_t * sc, dk_set_t code, int trim_one_long_cv)
       END_CATCH;
       hash_table_free (lblhash);
       if (err)
-	sqlc_resignal_1 (sc->sc_cc, err);
+	{
+	  dk_free_box (cv);
+	  sqlc_resignal_1 (sc->sc_cc, err);
+	}
 
       return cv;
     }
@@ -2532,6 +2587,10 @@ cv_free (code_vec_t cv)
 	    {
 	      ha_free ((hash_area_t *) ins->_.pred.cmp);
 	    }
+	  else if (bop_comp_func == ins->_.pred.func)
+	    dk_free ((box_t) ins->_.pred.cmp, sizeof (bop_comparison_t));
+	  else if (subq_comp_func == ins->_.pred.func)
+	    dk_free ((box_t) ins->_.pred.cmp, sizeof (subq_pred_t));
 	  else
 	    dk_free ((box_t) ins->_.pred.cmp, -1);
 	}
@@ -2575,6 +2634,10 @@ cv_free (code_vec_t cv)
 	{
 	  dk_free_box (ins->_.fetch.targets);
 	}
+      else if (ins->ins_type == IN_AGG && ins->_.agg.distinct)
+	{
+	  ha_free (ins->_.agg.distinct);
+	}
     }
   END_DO_INSTR
   dk_free_box ((caddr_t) cv);
