@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -62,7 +62,7 @@ sqlo_oby_exp_cols (sqlo_t * so, ST * dt, ST** oby)
 	  while (ST_P (spec->_.o_spec.col, BOP_AS))
 	    spec->_.o_spec.col = spec->_.o_spec.col->_.as_exp.left;
 	}
-      if (!ST_P (spec->_.o_spec.col, COL_DOTTED))
+      if (!ST_COLUMN (spec->_.o_spec.col, COL_DOTTED))
 	res = SPEC_MIXED;
     }
   END_DO_BOX;
@@ -174,7 +174,7 @@ sqlo_ot_oby_seq (sqlo_t * so, op_table_t * top_ot)
 
   DO_BOX (ST *, spec, inx, oby)
     {
-      if (!ST_P (spec->_.o_spec.col, COL_DOTTED))
+      if (!ST_COLUMN (spec->_.o_spec.col, COL_DOTTED))
 	return;
       col = spec->_.o_spec.col;
       col_ot = sqlo_cname_ot (so, col->_.col_ref.prefix);
@@ -700,10 +700,19 @@ sqlo_fun_ref_epilogue (sqlo_t * so, op_table_t * from_ot)
   ST * texp = from_ot->ot_dt->_.select_stmt.table_exp;
   ST ** group = texp ? texp->_.table_exp.group_by : NULL;
 
-  if (from_ot->ot_invariant_preds)
+  if (from_ot->ot_invariant_preds && !from_ot->ot_invariant_placed)
     {
-      from_ot->ot_work_dfe->_.sub.invariant_test =
-	  sqlo_and_list_body (so, LOC_LOCAL, so->so_gen_pt, from_ot->ot_invariant_preds);
+      df_elt_t * gen_pt = so->so_gen_pt;
+      df_elt_t * filter = sqlo_new_dfe (so, DFE_FILTER, NULL);
+      df_elt_t ** after_test;
+      filter->_.filter.invariant_of_ot = from_ot;
+      from_ot->ot_invariant_placed = 1;
+      so->so_gen_pt = from_ot->ot_work_dfe->_.sub.first;
+      after_test = sqlo_and_list_body (so, LOC_LOCAL, so->so_gen_pt, from_ot->ot_invariant_preds);
+      filter->_.filter.body = after_test;
+      filter->_.filter.preds = from_ot->ot_invariant_preds;
+      sqlo_place_dfe_after  (so, LOC_LOCAL, so->so_gen_pt, filter);
+      so->so_gen_pt = gen_pt;
     }
 
   if (!from_ot->ot_fun_refs && !group_dfe)
@@ -794,7 +803,7 @@ sqlo_fun_ref_epilogue (sqlo_t * so, op_table_t * from_ot)
 	    locus_to_loclocal = 1;
 	}
       if (arg_dfe)
-	sqlg_find_aggregate_sqt (so->so_sc->sc_cc->cc_schema, &(arg_dfe->dfe_sqt), fref, &(fref_dfe->dfe_sqt));
+	sqlg_find_aggregate_sqt (wi_inst.wi_schema, &(arg_dfe->dfe_sqt), fref, &(fref_dfe->dfe_sqt));
       if (locus_to_loclocal)
 	{
 	  fref_dfe->dfe_locus = LOC_LOCAL;
@@ -858,18 +867,25 @@ sqlo_is_postprocess (sqlo_t *so, df_elt_t * dt_dfe, df_elt_t * last_tb_dfe)
 }
 
 void
-sqlo_exp_cols_from_dt (sqlo_t * so, ST * tree, df_elt_t * dt_dfe, dk_set_t * ret)
+sqlo_exp_cols_from_dt (sqlo_t * so, ST * tree, df_elt_t * dt_dfe, df_elt_t * oby_dfe, dk_set_t * ret)
 {
   /* traverse exp and return all cols that exp depends on which are defined in dt_dfe */
+  df_elt_t * dfe;
   int inx;
-  if (ST_P (tree, COL_DOTTED))
+  if (DV_ARRAY_OF_POINTER == DV_TYPE_OF (tree)
+      && (dfe = sqlo_df_elt (so, tree)))
     {
-      df_elt_t * pt, *col_dfe = sqlo_df (so, tree);
+      int crossed = 0;
+      df_elt_t * pt;
       for (pt = dt_dfe->_.sub.last; pt; pt = pt->dfe_prev)
-	if (dfe_defines (pt, col_dfe))
-	  break;
-      if (pt)
-	t_set_pushnew (ret, col_dfe);
+	{
+	  if (dfe_defines (pt, dfe))
+	    break;
+	  if (pt == oby_dfe)
+	    crossed = 1;
+	}
+      if (pt && crossed)
+	t_set_pushnew (ret, dfe);
     }
   if (DV_ARRAY_OF_POINTER != DV_TYPE_OF (tree))
     return;
@@ -877,12 +893,25 @@ sqlo_exp_cols_from_dt (sqlo_t * so, ST * tree, df_elt_t * dt_dfe, dk_set_t * ret
     return;
   DO_BOX (ST*, x, inx, (ST**)tree)
     {
-      sqlo_exp_cols_from_dt (so, x, dt_dfe, ret);
+      sqlo_exp_cols_from_dt (so, x, dt_dfe, oby_dfe, ret);
     }
   END_DO_BOX;
 }
 
 
+int
+dfe_is_after (df_elt_t * after, df_elt_t * before)
+{
+  while (after)
+    {
+      if (after == before)
+	return 1;
+      after = after->dfe_prev;
+    }
+  return 0;
+}
+
+
 void
 sqlo_post_oby_ref (sqlo_t * so, df_elt_t * dt_dfe, df_elt_t * sel_dfe, int inx)
 {
@@ -905,7 +934,9 @@ sqlo_post_oby_ref (sqlo_t * so, df_elt_t * dt_dfe, df_elt_t * sel_dfe, int inx)
 	return;
     }
   END_DO_BOX;
-  sqlo_exp_cols_from_dt (so, sel_dfe->dfe_tree, dt_dfe, &deps);
+  if (!dfe_is_after (sel_dfe, oby_dfe))
+    return;
+  sqlo_exp_cols_from_dt (so, sel_dfe->dfe_tree, dt_dfe, oby_dfe, &deps);
   if (!oby_dfe->_.setp.oby_dep_cols)
     {
       oby_dfe->_.setp.oby_dep_cols = (dk_set_t*)t_box_copy ((caddr_t)dt_dfe->_.sub.dt_out);
