@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -86,7 +86,7 @@ dbg_print_st (caddr_t * box, FILE * f)
       dk_free_box (n);
       return;
     }
-  if (ST_P (st, COL_DOTTED))
+  if (ST_COLUMN (st, COL_DOTTED))
     sqlo_print (("%s.%s ", st->_.col_ref.prefix, st->_.col_ref.name));
   else if (ST_P (st, CALL_STMT))
     {
@@ -363,7 +363,10 @@ sqlo_dfe_print (df_elt_t * dfe, int offset)
 	    sqlo_print (("%*.*sOut cols :\n", offset, offset, " "));
 	    DO_BOX (df_elt_t *, out, inx, dfe->_.sub.dt_out)
 	      {
-		sqlo_dfe_print (out, offset + OFS_INCR);
+		if (DV_ARRAY_OF_POINTER == DV_TYPE_OF (out))
+		  sqlo_print (("- "));
+		else
+		  sqlo_dfe_print (out, offset + OFS_INCR);
 	      }
 	    END_DO_BOX;
 	  }
@@ -504,8 +507,9 @@ sqlo_next_print (dk_set_t * arr)
   DO_BOX (dk_set_t, dfes, inx, arr)
     {
       df_elt_t * dfe = (df_elt_t*)dfes->data;
-      sqlo_print ((" score %9.2g %s ", dfe->dfe_unit, DFE_TABLE == dfe->dfe_type && dfe->_.table.is_leaf ? "leaf" : ""));
-    DO_SET (df_elt_t *, elt, &dfes) sqlo_print ((" %s ", elt->_.table.ot->ot_new_prefix));
+      sqlo_print ((" score %9.2g %s ", dfe->dfe_unit, dfe_is_leaf(dfe) ? "leaf" : ""));
+      DO_SET (df_elt_t *, elt, &dfes) 
+        sqlo_print ((" %s ", elt->_.table.ot->ot_new_prefix));
       END_DO_SET();
     }
   END_DO_BOX;
@@ -637,7 +641,7 @@ dbg_qi_print_slots (query_instance_t * qi, state_slot_t ** slots, int nthset)
   fflush (stdout);
 }
 
-#endif // DEBUG
+#endif /* DEBUG */
 
 
 void
