@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -440,10 +440,10 @@ wideliteral (char *s)
 	      box = t_alloc_box (len + 1, DV_SHORT_STRING);
 	      start = s + 1;
 	      parse_string_literal ((unsigned char **) &start, (unsigned char *) box, 0);
-	      return t_box_utf8_as_wide_char (box, NULL, strlen (box), 0, DV_WIDE);
+	      return t_box_utf8_as_wide_char (box, NULL, strlen (box), 0);
 	    }
 	  else
-	    return t_box_utf8_as_wide_char (s + 2, NULL, strlen (s + 2) - 1, 0, DV_WIDE);
+	    return t_box_utf8_as_wide_char (s + 2, NULL, strlen (s + 2) - 1, 0);
 	}
       else
 	return NULL;
@@ -671,6 +671,19 @@ t_sqlp_box_id_upcase (const char *str)
   return s;
 }
 
+caddr_t
+t_sqlp_box_id_upcase_nchars (const char * str, int len)
+{
+  /* nothing in 2 */
+  caddr_t s;
+  if (len > MAX_NAME_LEN - 2)
+    len = MAX_NAME_LEN - 2;
+  s = t_box_dv_short_nchars ((caddr_t) str, len);
+  box_tag_modify (s, DV_SYMBOL);
+  if (CM_UPPER == case_mode)
+    sqlp_upcase (s);
+  return s;
+}
 
 caddr_t
 sqlp_box_upcase (const char *str)
@@ -891,7 +904,7 @@ sqlp_table_name (char *q, size_t max_q, char *o, size_t max_o, char *n, int do_c
   char temp[MAX_QUAL_NAME_LEN];
 
   if (do_case)
-    sch_normalize_new_table_case (top_sc->sc_cc->cc_schema, q, max_q, o, max_o);
+    sch_normalize_new_table_case (wi_inst.wi_schema, q, max_q, o, max_o);
 
   if (o)
     sqlp_check_infoschema (o);
@@ -921,7 +934,7 @@ sqlp_type_name (char *q, size_t max_q, char *o, size_t max_o, char *n, int add_i
       if (!CASEMODESTRCMP (as_new_name, sqlp_udt_current_type))
 	return as_new_name;
     }
-  udt = sch_name_to_type (top_sc->sc_cc->cc_schema, as_new_name);
+  udt = sch_name_to_type (wi_inst.wi_schema, as_new_name);
   if (udt)
     return as_new_name;
   if (q)
@@ -930,7 +943,7 @@ sqlp_type_name (char *q, size_t max_q, char *o, size_t max_o, char *n, int add_i
     snprintf (temp, sizeof (temp), "%s.%s", o, n);
   else
     snprintf (temp, sizeof (temp), "%s", n);
-  udt = sch_name_to_type (top_sc->sc_cc->cc_schema, temp);
+  udt = sch_name_to_type (wi_inst.wi_schema, temp);
   if (udt)
     return (t_box_string (udt->scl_name));
   return as_new_name;
@@ -940,7 +953,7 @@ sqlp_type_name (char *q, size_t max_q, char *o, size_t max_o, char *n, int add_i
       caddr_t as_new_name = sqlp_new_table_name (q, o, n);
       udt = udt_alloc_class_def (as_new_name);
       udt->scl_ext_lang = sqlp_udt_current_type_lang == UDT_LANG_NONE ? UDT_LANG_SQL : sqlp_udt_current_type_lang;
-      id_hash_set (top_sc->sc_cc->cc_schema->sc_name_to_type,
+      id_hash_set (wi_inst.wi_schema->sc_name_to_type,
 	  (caddr_t) & udt->scl_name,
 	  (caddr_t) & udt);
       return as_new_name;
@@ -971,7 +984,7 @@ sqlp_new_table_name (char *q, size_t max_q, char *o, size_t max_o, char *n)
   char *q2 = q ? q : sqlc_client ()->cli_qualifier;
   char *o2 = o ? o : CLI_OWNER (sqlc_client ());
 
-  sch_normalize_new_table_case (top_sc->sc_cc->cc_schema, q, max_q, o, max_o);
+  sch_normalize_new_table_case (wi_inst.wi_schema, q, max_q, o, max_o);
 
   snprintf (temp, sizeof (temp), "%s.%s.%s", q2, o2, n);
 /*  dk_free_box (q);
@@ -984,7 +997,7 @@ caddr_t
 sqlp_new_qualifier_name (char *q, size_t max_q)
 {
   caddr_t new_q = t_box_string (q);
-  sch_normalize_new_table_case (top_sc->sc_cc->cc_schema, q, max_q, NULL, 0);
+  sch_normalize_new_table_case (wi_inst.wi_schema, q, max_q, NULL, 0);
   return new_q;
 }
 
@@ -1153,7 +1166,7 @@ sqlp_expand_1_star_table_ref (ST * col, ST * tb_ref, dk_set_t * exp)
 	  dbe_table_t *tb;
 
 	  sqlp_infoschema_redirect_tb (tb_name, new_tb_name, sizeof (new_tb_name), NULL);
-	  tb = sch_name_to_table (top_sc->sc_cc->cc_schema,
+	  tb = sch_name_to_table (wi_inst.wi_schema,
 					       new_tb_name);
 	  if (!tb)
 	    sqlp_no_table (tb_pref, tb_name);
@@ -1537,7 +1550,6 @@ sqlp_view_u_id (void)
 }
 
 
-dk_set_t html_lines;
 
 caddr_t
 sqlp_html_string (void)
@@ -1643,17 +1655,17 @@ sqlp_for_statement (ST * sel, ST * body)
 		      while_fetch,
 		      while_handler,
 		      body),
-		  t_box_num (scn3_lineno),
+		  t_box_num (global_scs->scs_scn3c.lineno),
 		  t_box_num (scn3_get_lineno()),
 		  t_box_string (scn3_get_file_name())
 		        )),
 	  t_list (3, LABELED_STMT, t_box_string (cn),
 		t_list (5, COMPOUND_STMT, t_list (0),
-		  t_box_num (scn3_lineno),
+		  t_box_num (global_scs->scs_scn3c.lineno),
 		  t_box_num (scn3_get_lineno()),
 		  t_box_string (scn3_get_file_name())
 		  ))),
-	t_box_num (scn3_lineno),
+	t_box_num (global_scs->scs_scn3c.lineno),
 	t_box_num (scn3_get_lineno()),
 	t_box_string (scn3_get_file_name())
 	);
@@ -2266,7 +2278,9 @@ generic_check:
               dk_free_box (ret_val);
               goto not_a_constant_pure; /* see below */
             }
-          lit = (ST *)(t_full_box_copy_tree (ret_val));
+	  lit = (ST *)(t_full_box_copy_tree (ret_val));
+	  if (DV_TYPE_OF (ret_val) == DV_RDF)
+	    lit = t_listst (3, CALL_STMT, t_sqlp_box_id_upcase ("__rdflit"), t_list (1, lit));
             dk_free_box (ret_val);
           return lit;
 not_a_constant_pure: ;
@@ -2349,7 +2363,7 @@ sqlp_pl_file (char * text)
           sem++;
 
 	  line_no = atol (sem);
-	  if (scn3_pragmaline_depth == 0)
+	  if (global_scs->scs_scn3c.pragmaline_depth == 0)
 	    { /* only on top level */
 	      pl_file_offs = atol (sem);
 	      pl_file = t_alloc_box (len+1, DV_STRING);
@@ -2367,8 +2381,8 @@ void _br_push (void)
 {
   if (parse_pldbg)
     {
-      t_set_push(&sql3_ppbreaks, t_box_num(scn3_plineno));
-      t_set_push(&sql3_pbreaks, t_box_num(scn3_lineno));
+      t_set_push(&sql3_ppbreaks, t_box_num(global_scs->scs_scn3c.plineno));
+      t_set_push(&sql3_pbreaks, t_box_num(global_scs->scs_scn3c.lineno));
       t_set_push(&sql3_breaks, t_box_num(scn3_get_lineno()));
     }
 }
@@ -2386,10 +2400,10 @@ void _br_pop (void)
 void _br_set (void)
 {
   if (parse_pldbg && sql3_pbreaks &&
-      (int) unbox ((box_t) sql3_pbreaks->data) != scn3_lineno)
+      (int) unbox ((box_t) sql3_pbreaks->data) != global_scs->scs_scn3c.lineno)
     {
-      sql3_ppbreaks->data = (void *)t_box_num(scn3_plineno);
-      sql3_pbreaks->data = (void *)t_box_num(scn3_lineno);
+      sql3_ppbreaks->data = (void *)t_box_num(global_scs->scs_scn3c.plineno);
+      sql3_pbreaks->data = (void *)t_box_num(global_scs->scs_scn3c.lineno);
       sql3_breaks->data = (void *)t_box_num(scn3_get_lineno());
     }
 }
@@ -2446,12 +2460,12 @@ ST * sqlp_c_for_statement (ST **init, ST *cond, ST **inc, ST * body)
 
   cst[init_cnt] = t_listst (3, WHILE_STMT, cond, t_listst (5, COMPOUND_STMT,
 	incst,
-	t_box_num (scn3_lineno),
+	t_box_num (global_scs->scs_scn3c.lineno),
 	t_box_num (scn3_get_lineno()),
 	t_box_string (scn3_get_file_name())));
 
   res = t_listst (5, COMPOUND_STMT, cst,
-      t_box_num (scn3_lineno),
+      t_box_num (global_scs->scs_scn3c.lineno),
       t_box_num (scn3_get_lineno()),
       t_box_string (scn3_get_file_name()));
   return res;
@@ -2497,7 +2511,7 @@ ST * sqlp_foreach_statement (ST *data_type, caddr_t var, ST *arr, ST *body)
 			    CALL_STMT, t_sqlp_box_id_upcase ("aref"),
 			      t_list (2, arr, t_box_copy_tree ((caddr_t) inx))))),
 	  body),
-	t_box_num (scn3_lineno),
+	t_box_num (global_scs->scs_scn3c.lineno),
 	t_box_num (scn3_get_lineno()),
 	t_box_string (scn3_get_file_name())));
 }
@@ -2640,17 +2654,42 @@ sqlp_default_cluster ()
 dk_set_t
 cl_all_host_group_list ()
 {
+  dk_hash_t *visited = NULL;
   dk_set_t res = NULL;
   int inx;
   for (inx = local_cll.cll_max_host; inx > 0; inx--)
     {
-      if (cl_id_to_host (inx))
+      cl_host_t *ch;
+      if ((ch = cl_id_to_host (inx)))
 	{
 	  char name[20];
 	  snprintf (name, sizeof (name), "Host%d", inx);
+	  if (ch->ch_replica)
+	    {
+	      if (!visited)
+		visited = hash_table_allocate (11);
+	      if (gethash ((void *) ch, visited))
+		continue;
+	      {
+		caddr_t *h_list = (caddr_t *) t_alloc_box (sizeof (caddr_t) * dk_set_length (ch->ch_replica), DV_ARRAY_OF_POINTER);
+		int fill = 0;
+		sethash ((void *) ch, visited, (void *) 1);
+		DO_SET (cl_host_t *, repl, &ch->ch_replica)
+		{
+		  snprintf (name, sizeof (name), "Host%d", repl->ch_id);
+		  sethash ((void *) repl, visited, (void *) 1);
+		  h_list[fill++] = t_sym_string (name);
+		}
+		END_DO_SET ();
+		dk_set_push (&res, t_list (3, NULL, h_list, NULL));
+	      }
+	    }
+	  else
 	  dk_set_push (&res, t_list (3, NULL, t_list (1, t_sym_string (name)), NULL));
 	}
     }
+  if (visited)
+    hash_table_free (visited);
   return res;
 }
 
@@ -2674,3 +2713,8 @@ sqlp_index_default_opts(dk_set_t opts)
   return opts;
 }
 
+char *
+sqlp_inx_col_opt ()
+{
+    return "column";
+}
