@@ -1,10 +1,8 @@
+/* A Bison parser, made by GNU Bison 2.7.  */
 
-/* A Bison parser, made by GNU Bison 2.4.1.  */
-
-/* Skeleton interface for Bison's Yacc-like parsers in C
+/* Bison interface for Yacc-like parsers in C
    
-      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
-   Free Software Foundation, Inc.
+      Copyright (C) 1984, 1989-1990, 2000-2012 Free Software Foundation, Inc.
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,6 +30,15 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
+#ifndef YY_TTLYY_TURTLE_P_H_INCLUDED
+# define YY_TTLYY_TURTLE_P_H_INCLUDED
+/* Enabling traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 0
+#endif
+#if YYDEBUG
+extern int ttlyydebug;
+#endif
 
 /* Tokens.  */
 #ifndef YYTOKENTYPE
@@ -86,11 +93,12 @@
      QNAME = 302,
      QNAME_NS = 303,
      VARIABLE = 304,
-     BLANK_NODE_LABEL = 305,
-     Q_IRI_REF = 306,
-     _GARBAGE_BEFORE_DOT_WS = 307,
-     TTL_RECOVERABLE_ERROR = 308,
-     __TTL_NONPUNCT_END = 309
+     BLANK_NODE_LABEL_NQ = 305,
+     BLANK_NODE_LABEL_TTL = 306,
+     Q_IRI_REF = 307,
+     _GARBAGE_BEFORE_DOT_WS = 308,
+     TTL_RECOVERABLE_ERROR = 309,
+     __TTL_NONPUNCT_END = 310
    };
 #endif
 /* Tokens.  */
@@ -141,20 +149,19 @@
 #define QNAME 302
 #define QNAME_NS 303
 #define VARIABLE 304
-#define BLANK_NODE_LABEL 305
-#define Q_IRI_REF 306
-#define _GARBAGE_BEFORE_DOT_WS 307
-#define TTL_RECOVERABLE_ERROR 308
-#define __TTL_NONPUNCT_END 309
-
+#define BLANK_NODE_LABEL_NQ 305
+#define BLANK_NODE_LABEL_TTL 306
+#define Q_IRI_REF 307
+#define _GARBAGE_BEFORE_DOT_WS 308
+#define TTL_RECOVERABLE_ERROR 309
+#define __TTL_NONPUNCT_END 310
 
 
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
 {
-
-/* Line 1676 of yacc.c  */
+/* Line 2058 of yacc.c  */
 #line 73 "./turtle_p.y"
 
   caddr_t box;
@@ -163,9 +170,8 @@ typedef union YYSTYPE
   ptrlong lexlineno;
 
 
-
-/* Line 1676 of yacc.c  */
-#line 169 "turtle_p.h"
+/* Line 2058 of yacc.c  */
+#line 175 "turtle_p.h"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
@@ -173,5 +179,18 @@ typedef union YYSTYPE
 #endif
 
 
+#ifdef YYPARSE_PARAM
+#if defined __STDC__ || defined __cplusplus
+int ttlyyparse (void *YYPARSE_PARAM);
+#else
+int ttlyyparse ();
+#endif
+#else /* ! YYPARSE_PARAM */
+#if defined __STDC__ || defined __cplusplus
+int ttlyyparse (ttlp_t * ttlp_arg, yyscan_t yyscanner);
+#else
+int ttlyyparse ();
+#endif
+#endif /* ! YYPARSE_PARAM */
 
-
+#endif /* !YY_TTLYY_TURTLE_P_H_INCLUDED  */
