@@ -6,7 +6,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -48,6 +48,7 @@ caddr_t uname__txt;
 caddr_t uname__xslt;
 caddr_t uname_SPECIAL_cc_bif_c_AVG;
 caddr_t uname_SPECIAL_cc_bif_c_COUNT;
+caddr_t uname_SPECIAL_cc_bif_c_GROUPING;
 caddr_t uname_SPECIAL_cc_bif_c_MAX;
 caddr_t uname_SPECIAL_cc_bif_c_MIN;
 caddr_t uname_SPECIAL_cc_bif_c_SUM;
@@ -61,6 +62,7 @@ caddr_t uname_bif_c_st_within;
 caddr_t uname_bif_c_xcontains;
 caddr_t uname_bif_c_xpath_contains;
 caddr_t uname_bif_c_xquery_contains;
+caddr_t uname_false;
 caddr_t uname_lang;
 caddr_t uname_nil;
 caddr_t uname_nodeID_ns;
@@ -93,6 +95,7 @@ caddr_t uname_rdfdf_ns_uri_default_iid_nullable;
 caddr_t uname_space;
 caddr_t uname_swap_reify_ns_uri;
 caddr_t uname_swap_reify_ns_uri_statement;
+caddr_t uname_true;
 caddr_t uname_virtrdf_ns_uri;
 caddr_t uname_virtrdf_ns_uri_DefaultQuadMap;
 caddr_t uname_virtrdf_ns_uri_DefaultQuadStorage;
@@ -112,6 +115,7 @@ caddr_t uname_virtrdf_ns_uri_bnode_base;
 caddr_t uname_virtrdf_ns_uri_bnode_label;
 caddr_t uname_virtrdf_ns_uri_bnode_row;
 caddr_t uname_virtrdf_ns_uri_dialect;
+caddr_t uname_virtrdf_ns_uri_dialect_exceptions;
 caddr_t uname_virtrdf_ns_uri_isSpecialPredicate;
 caddr_t uname_virtrdf_ns_uri_isSubclassOf;
 caddr_t uname_virtrdf_ns_uri_loadAs;
@@ -241,6 +245,7 @@ static uname_const_decl_t uname_const_decls[] = {
   { &uname__xslt				, " xslt"			},
   { &uname_SPECIAL_cc_bif_c_AVG			, "SPECIAL::bif:AVG"		},
   { &uname_SPECIAL_cc_bif_c_COUNT		, "SPECIAL::bif:COUNT"		},
+  { &uname_SPECIAL_cc_bif_c_GROUPING			, "SPECIAL::bif:GROUPING"			},
   { &uname_SPECIAL_cc_bif_c_MAX			, "SPECIAL::bif:MAX"		},
   { &uname_SPECIAL_cc_bif_c_MIN			, "SPECIAL::bif:MIN"		},
   { &uname_SPECIAL_cc_bif_c_SUM			, "SPECIAL::bif:SUM"		},
@@ -254,6 +259,7 @@ static uname_const_decl_t uname_const_decls[] = {
   { &uname_bif_c_xcontains			, "bif:xcontains"		},
   { &uname_bif_c_xpath_contains			, "bif:xpath_contains"		},
   { &uname_bif_c_xquery_contains		, "bif:xquery_contains"		},
+  { &uname_false					, "false"					},
   { &uname_lang					, "lang"			},
   { &uname_nil					, "nil"				},
   { &uname_nodeID_ns				, "nodeID://"			},
@@ -286,6 +292,7 @@ static uname_const_decl_t uname_const_decls[] = {
   { &uname_space				, "space"			},
   { &uname_swap_reify_ns_uri			, SWAP_REIFY_NS_URI		},
   { &uname_swap_reify_ns_uri_statement		, SWAP_REIFY_NS_URI "statement"	},
+  { &uname_true						, "true"					},
   { &uname_virtrdf_ns_uri			, VIRTRDF_NS_URI		},
   { &uname_virtrdf_ns_uri_DefaultQuadMap	, VIRTRDF_NS_URI "DefaultQuadMap"	},
   { &uname_virtrdf_ns_uri_DefaultQuadStorage	, VIRTRDF_NS_URI "DefaultQuadStorage"	},
@@ -305,6 +312,7 @@ static uname_const_decl_t uname_const_decls[] = {
   { &uname_virtrdf_ns_uri_bnode_label		, VIRTRDF_NS_URI "bnode-label"	},
   { &uname_virtrdf_ns_uri_bnode_row		, VIRTRDF_NS_URI "bnode-row"	},
   { &uname_virtrdf_ns_uri_dialect		, VIRTRDF_NS_URI "dialect"	},
+  { &uname_virtrdf_ns_uri_dialect_exceptions		, VIRTRDF_NS_URI "dialect-exceptions"		},
   { &uname_virtrdf_ns_uri_isSpecialPredicate	, VIRTRDF_NS_URI "isSpecialPredicate"	},
   { &uname_virtrdf_ns_uri_isSubclassOf		, VIRTRDF_NS_URI "isSubclassOf"	},
   { &uname_virtrdf_ns_uri_loadAs		, VIRTRDF_NS_URI "loadAs"	},
