@@ -6,7 +6,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -71,6 +71,7 @@ extern caddr_t uname__txt;
 extern caddr_t uname__xslt;
 extern caddr_t uname_SPECIAL_cc_bif_c_AVG;
 extern caddr_t uname_SPECIAL_cc_bif_c_COUNT;
+extern caddr_t uname_SPECIAL_cc_bif_c_GROUPING;
 extern caddr_t uname_SPECIAL_cc_bif_c_MAX;
 extern caddr_t uname_SPECIAL_cc_bif_c_MIN;
 extern caddr_t uname_SPECIAL_cc_bif_c_SUM;
@@ -84,6 +85,7 @@ extern caddr_t uname_bif_c_st_within;
 extern caddr_t uname_bif_c_xcontains;
 extern caddr_t uname_bif_c_xpath_contains;
 extern caddr_t uname_bif_c_xquery_contains;
+extern caddr_t uname_false;
 extern caddr_t uname_lang;
 extern caddr_t uname_nil;
 extern caddr_t uname_nodeID_ns;
@@ -116,6 +118,7 @@ extern caddr_t uname_rdfdf_ns_uri_default_iid_nullable;
 extern caddr_t uname_space;
 extern caddr_t uname_swap_reify_ns_uri;
 extern caddr_t uname_swap_reify_ns_uri_statement;
+extern caddr_t uname_true;
 extern caddr_t uname_virtrdf_ns_uri;
 extern caddr_t uname_virtrdf_ns_uri_DefaultQuadMap;
 extern caddr_t uname_virtrdf_ns_uri_DefaultQuadStorage;
@@ -135,6 +138,7 @@ extern caddr_t uname_virtrdf_ns_uri_bnode_base;
 extern caddr_t uname_virtrdf_ns_uri_bnode_label;
 extern caddr_t uname_virtrdf_ns_uri_bnode_row;
 extern caddr_t uname_virtrdf_ns_uri_dialect;
+extern caddr_t uname_virtrdf_ns_uri_dialect_exceptions;
 extern caddr_t uname_virtrdf_ns_uri_isSpecialPredicate;
 extern caddr_t uname_virtrdf_ns_uri_isSubclassOf;
 extern caddr_t uname_virtrdf_ns_uri_loadAs;
