@@ -4,7 +4,7 @@
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --
---  Copyright (C) 1998-2014 OpenLink Software
+--  Copyright (C) 1998-2016 OpenLink Software
 --
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -429,6 +429,11 @@ end_scan:
 	  	sprintf ('%s://%{WSHost}s', case when is_https_ctx () then 'https' else 'http' end));
         }
     }
+  if (http_host () <> 0)
+    {
+      long_path := replace (long_path, '^{Host}^', http_host ());
+      long_path := replace (long_path, '^{FullHost}^', sprintf ('%s://%s', case when is_https_ctx () then 'https' else 'http' end, http_host ()));
+    }
   return long_path;
 }
 ;
@@ -1072,20 +1077,20 @@ create procedure DB.DBA.URLREWRITE_APPLY_TCN (in rulelist_uri varchar, inout pat
   algo := http_request_header_full (lines, 'Negotiate', '*');
   do_cn := 1;
   vlist := trans := guess := 0;
-  if (algo = 'trans');
+  if (algo = 'trans')
     {
       trans := 1;
-      do_cn := 0;
+      do_cn := 1;
     }
-  if (algo = 'vlist');
+  if (algo = 'vlist')
     {
       trans := vlist := 1;
-      do_cn := 0;
+      do_cn := 1;
     }
-  if (algo = 'guess-small');
+  if (algo = 'guess-small')
     {
       trans := vlist := 1;
-      do_cn := 0;
+      do_cn := 1;
     }
   if (atof (algo) >= 1)
     do_cn := 1;
@@ -1102,10 +1107,10 @@ create procedure DB.DBA.URLREWRITE_APPLY_TCN (in rulelist_uri varchar, inout pat
     {
        declare alang, aenc, variant, path_str varchar;
 
-       if (VM_URI not like '/%' and path like '%/') -- directory and non-absolute variant pattern
+       if (ltrim (VM_URI, '^') not like '/%' and path like '%/') -- directory and non-absolute variant pattern
 	 goto next_variant;
 
-       if (VM_URI like '/%')
+       if (ltrim (VM_URI, '^') like '/%')
 	 path_str := path;
        else
          path_str := rel_uri;
@@ -1142,7 +1147,7 @@ create procedure DB.DBA.URLREWRITE_APPLY_TCN (in rulelist_uri varchar, inout pat
 	   best_id := VM_ID;
 	   hook := VM_CONTENT_LOCATION_HOOK;
 	 }
-       if (not do_cn)
+       if (vlist)
          {
 	   alang := '';
 	   aenc := '';
@@ -1177,6 +1182,8 @@ create procedure DB.DBA.URLREWRITE_APPLY_TCN (in rulelist_uri varchar, inout pat
       if (hook is not null and __proc_exists (hook) is not null)
 	cl := call (hook) (best_id, best_variant);
       http_headers := sprintf ('TCN: choice\r\nVary: negotiate,accept\r\nContent-Location: %s\r\n%s', cl, ct);
+      if (list <> '')
+	http_headers := http_headers || sprintf ('Alternates: %s\r\n', rtrim (list, ', '));
       -- since best_variant is a relative path, we ignore semicolon, otherwise it will not expand thinking it's absolute
       path := WS.WS.EXPAND_URL (path, replace (best_variant, ':', '\x1'));
       path := replace (path, '\x1', ':');
