@@ -8,7 +8,7 @@
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --
---  Copyright (C) 1998-2014 OpenLink Software
+--  Copyright (C) 1998-2016 OpenLink Software
 --
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -371,7 +371,7 @@ create procedure USER_PASSWORD_SET (in name varchar, in passwd varchar)
     }
   select U_ID, U_GROUP into _u_id, _u_group from DB.DBA.SYS_USERS where U_NAME = USER;
   if (not (_u_id = 0 or _u_group = 0))
-    signal ('42000', 'Function DB.DBA.USER_PASSWORD_SET is restricted to dba group', 'SR285');
+    signal ('42000', 'Function DB.DBA.USER_PASSWORD_SET() is restricted to DBA group', 'SR285:SECURITY');
   if (not exists (select 1 from DB.DBA.SYS_USERS where U_NAME = name and U_DAV_ENABLE = 1 and U_IS_ROLE = 0))
     signal ('42000', concat ('The user ''', name, ''' does not exist'), 'SR286');
   if (not isstring (passwd) or length (passwd) < 1)
@@ -447,11 +447,11 @@ USER_GRANT_ROLE (in _name varchar, in _role varchar, in grant_opt integer := 0)
     }
   if (_g_id = http_nobody_uid () or _g_id = http_nogroup_gid ())
     {
-	signal ('37000', sprintf ('System role "%s" can not be granted to "%s"', _role, _name), 'U0013');
+	signal ('37000', sprintf ('System role "%s" cannot be granted to "%s"', _role, _name), 'U0013');
     }
   if (_u_id = http_nobody_uid () or _u_id = http_nogroup_gid ())
     {
-	signal ('37000', sprintf ('Role "%s" can not be granted to special account "%s"', _role, _name), 'U0014');
+	signal ('37000', sprintf ('Role "%s" cannot be granted to special account "%s"', _role, _name), 'U0014');
     }
 
     {
@@ -499,7 +499,7 @@ USER_REVOKE_ROLE (in _name varchar, in _role varchar)
   GET_SEC_OBJECT_ID (_role, _g_id, _g_is_sql, opts);
   if ((_g_id = http_nogroup_gid () and _u_id = http_nobody_uid ()) or (_g_id = http_admin_gid() and _u_id = http_dav_uid()))
     {
-      signal ('37000', sprintf ('Built-in role "%s" can not be revoked from built-in user "%s"', _role, _name), 'U0015');
+      signal ('37000', sprintf ('Built-in role "%s" cannot be revoked from built-in user "%s"', _role, _name), 'U0015');
     }
   for select distinct GI_SUB as sub from SYS_ROLE_GRANTS where
     GI_SUPER = _u_id and GI_GRANT = _g_id
@@ -767,7 +767,7 @@ USER_KEY_STORE (in username varchar, in key_name varchar, in key_type varchar, i
       if (key_value is null)
 	key_value := xenc_key_serialize (key_name, 1);
       if (key_value is null)
-	signal ('22023', 'Can not serialize the key');
+	signal ('22023', 'Cannot serialize the key');
     }
 
   keys := coalesce (USER_GET_OPTION (username, 'KEYS'), vector ());
@@ -787,7 +787,7 @@ USER_KEY_DELETE (in username varchar, in key_name varchar)
   declare keys any;
   declare inx int;
   if (lower(user) <> 'dba' and username <> user)
-    signal ('42000', 'Can\'t delete non own keys');
+    signal ('42000', 'Cannot delete non own keys');
   keys := coalesce (USER_GET_OPTION (username, 'KEYS'), vector ());
   inx := position (key_name, keys);
   if (inx > 0)
@@ -808,7 +808,7 @@ USER_KEYS_INIT (in username varchar, in opts any)
   declare keys, path, key_value, key_type, key_passwd, key_pkey, os_u_name, os_u_pass any;
   declare certs any;
 
-  debug := 1;
+  debug := 0;
 
   opts := deserialize (blob_to_string (opts));
 
@@ -862,7 +862,7 @@ USER_KEYS_INIT (in username varchar, in opts any)
  	    key_value := file_to_string (path);
 	  else
 	    {
-	      log_message (sprintf ('XENC: Can\'t open key file: %s', path));
+	      log_message (sprintf ('XENC: Cannot open key file: %s', path));
 	      goto next;
 	    }
 	}
@@ -1043,16 +1043,18 @@ create procedure
     {
       declare ext_oid varchar;
       declare exit handler for sqlstate '*' {
-        goto normal_auth;
+        goto cert_auth;
       };
 
+      user_name := null;
+
 	-- subject
 	cn := get_certificate_info (2);
 	-- fingerprint
 	fp := get_certificate_info (6);
 
 	if (fp is null)
-	  goto normal_auth;
+	  goto cert_auth;
 
         ext_oid := virtuoso_ini_item_value ('Parameters', 'X509ExtensionOID');
 
@@ -1075,18 +1077,27 @@ create procedure
 	    rc := 1;
 	  }
     }
-  -- normal verification
+
+  cert_auth:;
+
+
+  -- normal verification (only if rc != 0 which means "no access")
 normal_auth:
-  if (__proc_exists ('DB.DBA.DBEV_LOGIN'))
+  if (rc <> 0)
     {
-      rc := "DB"."DBA"."DBEV_LOGIN" (user_name, digest, session_random);
-    }
-  else if (rc <= 0) -- only if not authenticated 
-    {
-      rc := DB.DBA.FOAF_SSL_LOGIN (user_name, digest, session_random);
-      if (rc = 0)
-        rc := DB.DBA.LDAP_LOGIN (user_name, digest, session_random);
+      if (__proc_exists ('DB.DBA.DBEV_LOGIN'))
+	{
+	  rc := "DB"."DBA"."DBEV_LOGIN" (user_name, digest, session_random);
+	}
+      else if (rc <= 0) -- only if not authenticated 
+	{
+	  rc := DB.DBA.FOAF_SSL_LOGIN (user_name, digest, session_random);
+	  if (rc = 0)
+	    rc := DB.DBA.LDAP_LOGIN (user_name, digest, session_random);
+	}
     }
+  if (user_name is null)
+    user_name := '';
   return rc;
 }
 ;
@@ -1104,7 +1115,7 @@ SET_USER_OS_ACOUNT (in username varchar, in os_u_name varchar,
 	return 1;
     }
   signal ('42000',
-      concat ('Can''t login system user ', os_u_name, '. Logon failure: unknown user name or bad password.'),
+      concat ('Cannot login system user ', os_u_name, '. Logon failure: unknown user name or bad password.'),
       'SR359');
 }
 ;
@@ -1260,6 +1271,8 @@ DB.DBA.LDAP_LOGIN (inout user_name varchar, in digest varchar, in session_random
       select LS_BASE, LS_BIND_DN, LS_ACCOUNT, LS_PASSWORD, LS_UID_FLD, LS_TRY_SSL, LS_LDAP_VERSION
 	  into base, bind, lacc, lpwd, luid, ltry, lver from SYS_LDAP_SERVERS where LS_ADDRESS = lserv;
 
+      lpwd := pwd_magic_calc (lacc, lpwd, 1);
+
       if (is_http_ctx())
 	{
           if (get_keyword ('authtype', session_random) = 'basic')
