@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -585,8 +585,9 @@ dc_mp_insert_copy_any (mem_pool_t * mp, data_col_t * dc, int inx, dbe_column_t *
       db_buf_t rdf_id = mp_dv_rdf_to_db_serial (mp, dv);
       return (caddr_t) rdf_id;
     }
-  if ((DV_STRING == dv[0] || DV_SHORT_STRING_SERIAL == dv[0] || DV_DB_NULL == dv[0]) && col && 'O' == col->col_name[0]
-      && tb_is_rdf_quad (col->col_defined_in) && !f_read_from_rebuilt_database)
+  if ((DV_STRING == dv[0] || DV_SHORT_STRING_SERIAL == dv[0] || DV_DB_NULL == dv[0] ||
+	  DV_UNAME == dv[0] || DV_SYMBOL == dv[0] || DV_BOX_FLAGS == dv[0] || IS_WIDE_STRING_DTP (dv[0]))
+      && col && 'O' == col->col_name[0] && tb_is_rdf_quad (col->col_defined_in) && !f_read_from_rebuilt_database)
     {
       if (THR_TMP_POOL == mp)
 	SET_THR_TMP_POOL (NULL);
@@ -841,7 +842,7 @@ key_vec_insert (insert_node_t * ins, caddr_t * qst, it_cursor_t * itc, ins_key_t
   dk_set_t parts;
   int null_skipped = 0;
   int n_rows, log_needed;
-  mem_pool_t *ins_mp = mem_pool_alloc ();
+  mem_pool_t * ins_mp;
   row_delta_t rd;
   row_delta_t **rds;
   int icol;
@@ -852,6 +853,8 @@ key_vec_insert (insert_node_t * ins, caddr_t * qst, it_cursor_t * itc, ins_key_t
   query_instance_t *qi = (query_instance_t *) qst;
   LOCAL_RD (right_rd);
   QI_CHECK_STACK (qi, &qst, INS_STACK_MARGIN);
+  ins_mp = mem_pool_alloc ();
+
   if (dbf_ko_pk && dbf_ko_pk == key->key_table->tb_primary_key->key_id && dbf_ko_key != key->key_id)
     return 0;
   right_rd.rd_temp = right_temp;
@@ -903,6 +906,7 @@ key_vec_insert (insert_node_t * ins, caddr_t * qst, it_cursor_t * itc, ins_key_t
       rd1->rd_values = (caddr_t *) mp_alloc_box (ins_mp, sizeof (caddr_t) * n_parts, DV_ARRAY_OF_POINTER);
       for (icol = 0; icol < n_parts; icol++)
 	{
+	  dtp_t val_dtp;
 	  dbe_column_t *col = ik->ik_cols[icol];
 	  if (SSL_IS_VEC (ik->ik_slots[icol]))
 	    {
@@ -917,7 +921,8 @@ key_vec_insert (insert_node_t * ins, caddr_t * qst, it_cursor_t * itc, ins_key_t
 	      else
 		{
 		  caddr_t val = dc_mp_box_for_rd (ins_mp, dc, inx);
-		  if (DV_WIDE == col->col_sqt.sqt_col_dtp && DV_WIDE == DV_TYPE_OF (val))
+		  val_dtp = DV_TYPE_OF (val);
+		  if (DV_WIDE == col->col_sqt.sqt_col_dtp && DV_WIDE == val_dtp)
 		    val = mp_box_wide_as_utf8_char (ins_mp, val, (box_length (val) / sizeof (wchar_t)) - 1, DV_STRING);
 		  rd1->rd_values[icol] = val;
 		}
@@ -928,8 +933,18 @@ key_vec_insert (insert_node_t * ins, caddr_t * qst, it_cursor_t * itc, ins_key_t
 	    rd_vec_blob (itc, rd1, col, icol, ins_mp);
 
 	len_ck:
+	  val_dtp = DV_TYPE_OF (rd1->rd_values[icol]);
+	  if (col->col_sqt.sqt_non_null && DV_DB_NULL == val_dtp)
+	    {
+	      SET_THR_TMP_POOL (NULL);
+	      mp_free (ins_mp);
+	      itc->itc_ltrx->lt_status = LT_BLOWN_OFF;
+	      itc->itc_ltrx->lt_error = LTE_SQL_ERROR;
+	      sqlr_new_error ("22026", "INULL", "Insert of null for non null column %s , key %s",
+			      col->col_name, key->key_name);
+	    }
 	  if (key->key_not_null && !col->col_sqt.sqt_non_null && icol < key->key_n_significant
-	      && DV_DB_NULL == DV_TYPE_OF (rd1->rd_values[icol]))
+	      && DV_DB_NULL == val_dtp)
 	    {
 	      rds[inx] = (row_delta_t *) - 1;
 	      null_skipped = 1;
