@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -477,6 +477,9 @@ next_set:
 	    {
 	      all_eq = 0;
 	      first_diff = nth_part;
+	      /* if there are ranges after eqs and then do a reset for a different range start, not guaranteed to be to the right  of prev range. */
+	      if (CMP_EQ != sp->sp_min_op)
+		first_diff = itc->itc_n_vec_sort_cols;
 	    }
 	}
       if (DCT_NUM_INLINE & dc->dc_type)
