@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2014 OpenLink Software
+ *  Copyright (C) 1998-2016 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -125,12 +125,12 @@ loaded_xe_is_ready:
   if (XE_IS_TREE (loaded_xe))
     {
       memcpy (xlazye->xe_doc.xd, loaded_doc, sizeof (xml_tree_doc_t));
-      dk_free (loaded_doc, sizeof (xml_tree_doc_t));
+      dk_free_box (loaded_doc);
     }
   else
     {
       memcpy (xlazye->xe_doc.xd, loaded_doc, sizeof (xper_doc_t));
-      dk_free (loaded_doc, sizeof (xper_doc_t));
+      dk_free_box (loaded_doc);
     }
   xlazye->xe_doc.xd->xd_top_doc = xlazye->xe_doc.xd;
   loaded_xe->xe_doc.xd = xlazye->xe_doc.xd;
